@@ -1,4 +1,6 @@
 // Copyright (c) 2018-2023, Sylabs Inc. All rights reserved.
+// Copyright (c) Contributors to the Apptainer project, established as
+//   Apptainer a Series of LF Projects LLC.
 // This software is licensed under a 3-clause BSD license. Please consult the
 // LICENSE.md file distributed with the sources of this project regarding your
 // rights to use or distribute this software.
@@ -8,7 +10,7 @@ package cli
 import (
 	"os"
 
-	"github.com/sylabs/singularity/pkg/cmdline"
+	"github.com/sylabs/singularity/v4/pkg/cmdline"
 )
 
 // actionflags.go contains flag variables for action-like commands to draw from
@@ -20,7 +22,7 @@ var (
 	overlayPath        []string
 	scratchPath        []string
 	workdirPath        string
-	pwdPath            string
+	cwdPath            string
 	shellPath          string
 	hostname           string
 	network            string
@@ -28,21 +30,21 @@ var (
 	dns                string
 	security           []string
 	cgroupsTOMLFile    string
-	vmRAM              string
-	vmCPU              string
-	vmIP               string
 	containLibsPath    []string
 	fuseMount          []string
 	singularityEnv     map[string]string
 	singularityEnvFile string
 	noMount            []string
 	proot              string
+	device             []string
+	cdiDirs            []string
 
 	isBoot          bool
 	isFakeroot      bool
 	noSetgroups     bool
 	isCleanEnv      bool
 	isCompat        bool
+	noCompat        bool
 	isContained     bool
 	isContainAll    bool
 	isWritable      bool
@@ -57,16 +59,14 @@ var (
 	noNvidia        bool
 	noRocm          bool
 	noUmask         bool
-	vm              bool
-	vmErr           bool
-	isSyOS          bool
 	disableCache    bool
 
-	netNamespace  bool
-	utsNamespace  bool
-	userNamespace bool
-	pidNamespace  bool
-	ipcNamespace  bool
+	netNamespace   bool
+	utsNamespace   bool
+	userNamespace  bool
+	pidNamespace   bool
+	noPidNamespace bool
+	ipcNamespace   bool
 
 	allowSUID bool
 	keepPrivs bool
@@ -85,8 +85,6 @@ var (
 	memorySwap        string // bytes
 	oomKillDisable    bool
 	pidsLimit         int
-
-	ociRuntime bool
 )
 
 // --app
@@ -106,7 +104,7 @@ var actionBindFlag = cmdline.Flag{
 	DefaultValue: []string{},
 	Name:         "bind",
 	ShortHand:    "B",
-	Usage:        "a user-bind path specification.  spec has the format src[:dest[:opts]], where src and dest are outside and inside paths.  If dest is not given, it is set equal to src.  Mount options ('opts') may be specified as 'ro' (read-only) or 'rw' (read/write, which is the default). Multiple bind paths can be given by a comma separated list.",
+	Usage:        "a user-bind path specification. spec has the format src[:dest[:opts]], where src and dest are outside and inside paths. If dest is not given, it is set equal to src. Mount options ('opts') may be specified as 'ro' (read-only) or 'rw' (read/write, which is the default). Multiple bind paths can be given by a comma separated list.",
 	EnvKeys:      []string{"BIND", "BINDPATH"},
 	Tag:          "<spec>",
 	EnvHandler:   cmdline.EnvAppendValue,
@@ -132,7 +130,7 @@ var actionHomeFlag = cmdline.Flag{
 	DefaultValue: CurrentUser.HomeDir,
 	Name:         "home",
 	ShortHand:    "H",
-	Usage:        "a home directory specification.  spec can either be a src path or src:dest pair.  src is the source path of the home directory outside the container and dest overrides the home directory within the container.",
+	Usage:        "a home directory specification. spec can either be a src path or src:dest pair. src is the source path of the home directory outside the container and dest overrides the home directory within the container.",
 	EnvKeys:      []string{"HOME"},
 	Tag:          "<spec>",
 }
@@ -168,7 +166,7 @@ var actionWorkdirFlag = cmdline.Flag{
 	DefaultValue: "",
 	Name:         "workdir",
 	ShortHand:    "W",
-	Usage:        "working directory to be used for /tmp, /var/tmp and $HOME (if -c/--contain was also used)",
+	Usage:        "working directory to be used for /tmp and /var/tmp (if -c/--contain was also used)",
 	EnvKeys:      []string{"WORKDIR"},
 	Tag:          "<path>",
 }
@@ -195,13 +193,25 @@ var actionShellFlag = cmdline.Flag{
 	Tag:          "<path>",
 }
 
+// --cwd
+var actionCwdFlag = cmdline.Flag{
+	ID:           "actionCwdFlag",
+	Value:        &cwdPath,
+	DefaultValue: "",
+	Name:         "cwd",
+	Usage:        "initial working directory for payload process inside the container (synonym for --pwd)",
+	EnvKeys:      []string{"CWD", "TARGET_CWD"},
+	Tag:          "<path>",
+}
+
 // --pwd
 var actionPwdFlag = cmdline.Flag{
 	ID:           "actionPwdFlag",
-	Value:        &pwdPath,
+	Value:        &cwdPath,
 	DefaultValue: "",
 	Name:         "pwd",
-	Usage:        "initial working directory for payload process inside the container",
+	Usage:        "initial working directory for payload process inside the container (synonym for --cwd)",
+	Hidden:       true,
 	EnvKeys:      []string{"PWD", "TARGET_PWD"},
 	Tag:          "<path>",
 }
@@ -269,39 +279,6 @@ var actionApplyCgroupsFlag = cmdline.Flag{
 	EnvKeys:      []string{"APPLY_CGROUPS"},
 }
 
-// --vm-ram
-var actionVMRAMFlag = cmdline.Flag{
-	ID:           "actionVMRAMFlag",
-	Value:        &vmRAM,
-	DefaultValue: "1024",
-	Name:         "vm-ram",
-	Usage:        "amount of RAM in MiB to allocate to Virtual Machine (implies --vm)",
-	Tag:          "<size>",
-	EnvKeys:      []string{"VM_RAM"},
-}
-
-// --vm-cpu
-var actionVMCPUFlag = cmdline.Flag{
-	ID:           "actionVMCPUFlag",
-	Value:        &vmCPU,
-	DefaultValue: "1",
-	Name:         "vm-cpu",
-	Usage:        "number of CPU cores to allocate to Virtual Machine (implies --vm)",
-	Tag:          "<CPU #>",
-	EnvKeys:      []string{"VM_CPU"},
-}
-
-// --vm-ip
-var actionVMIPFlag = cmdline.Flag{
-	ID:           "actionVMIPFlag",
-	Value:        &vmIP,
-	DefaultValue: "dhcp",
-	Name:         "vm-ip",
-	Usage:        "IP Address to assign for container usage. Defaults to DHCP within bridge network.",
-	Tag:          "<IP Address>",
-	EnvKeys:      []string{"VM_IP"},
-}
-
 // hidden flag to handle SINGULARITY_CONTAINLIBS environment variable
 var actionContainLibsFlag = cmdline.Flag{
 	ID:           "actionContainLibsFlag",
@@ -385,6 +362,16 @@ var actionCompatFlag = cmdline.Flag{
 	EnvKeys:      []string{"COMPAT"},
 }
 
+// --no-compat
+var actionNoCompatFlag = cmdline.Flag{
+	ID:           "actionNoCompatFlag",
+	Value:        &noCompat,
+	DefaultValue: false,
+	Name:         "no-compat",
+	Usage:        "(--oci mode) do not apply settings for increased OCI/Docker compatibility. Emulate native runtime defaults without --contain etc.",
+	EnvKeys:      []string{"NO_COMPAT"},
+}
+
 // -c|--contain
 var actionContainFlag = cmdline.Flag{
 	ID:           "actionContainFlag",
@@ -508,37 +495,6 @@ var actionNoRocmFlag = cmdline.Flag{
 	EnvKeys:      []string{"ROCM_OFF", "NO_ROCM"},
 }
 
-// --vm
-var actionVMFlag = cmdline.Flag{
-	ID:           "actionVMFlag",
-	Value:        &vm,
-	DefaultValue: false,
-	Name:         "vm",
-	Usage:        "enable VM support",
-	EnvKeys:      []string{"VM"},
-}
-
-// --vm-err
-var actionVMErrFlag = cmdline.Flag{
-	ID:           "actionVMErrFlag",
-	Value:        &vmErr,
-	DefaultValue: false,
-	Name:         "vm-err",
-	Usage:        "enable attaching stderr from VM",
-	EnvKeys:      []string{"VMERROR"},
-}
-
-// --syos
-// TODO: Keep this in production?
-var actionSyOSFlag = cmdline.Flag{
-	ID:           "actionSyOSFlag",
-	Value:        &isSyOS,
-	DefaultValue: false,
-	Name:         "syos",
-	Usage:        "execute SyOS shell",
-	EnvKeys:      []string{"SYOS"},
-}
-
 // -p|--pid
 var actionPidNamespaceFlag = cmdline.Flag{
 	ID:           "actionPidNamespaceFlag",
@@ -550,6 +506,16 @@ var actionPidNamespaceFlag = cmdline.Flag{
 	EnvKeys:      []string{"PID", "UNSHARE_PID"},
 }
 
+// --no-pid
+var actionNoPidNamespaceFlag = cmdline.Flag{
+	ID:           "actionNoPidNamespaceFlag",
+	Value:        &noPidNamespace,
+	DefaultValue: false,
+	Name:         "no-pid",
+	Usage:        "do not run container in a new PID namespace",
+	EnvKeys:      []string{"NO_PID"},
+}
+
 // -i|--ipc
 var actionIpcNamespaceFlag = cmdline.Flag{
 	ID:           "actionIpcNamespaceFlag",
@@ -609,7 +575,7 @@ var actionNoPrivsFlag = cmdline.Flag{
 	Value:        &noPrivs,
 	DefaultValue: false,
 	Name:         "no-privs",
-	Usage:        "drop all privileges from root user in container)",
+	Usage:        "drop all privileges in container (root only in non-OCI mode)",
 	EnvKeys:      []string{"NO_PRIVS"},
 }
 
@@ -813,14 +779,22 @@ var actionProotFlag = cmdline.Flag{
 	Hidden:       true,
 }
 
-// --oci
-var actionOCIFlag = cmdline.Flag{
-	ID:           "actionOCI",
-	Value:        &ociRuntime,
-	DefaultValue: false,
-	Name:         "oci",
-	Usage:        "Launch container with OCI runtime (experimental)",
-	EnvKeys:      []string{"OCI"},
+// --device
+var actionDevice = cmdline.Flag{
+	ID:           "actionDevice",
+	Value:        &device,
+	DefaultValue: []string{},
+	Name:         "device",
+	Usage:        "fully-qualified CDI device name(s). A fully-qualified CDI device name consists of a VENDOR, CLASS, and NAME, which are combined as follows: <VENDOR>/<CLASS>=<NAME> (e.g. vendor.com/device=mydevice). Multiple fully-qualified CDI device names can be given as a comma separated list.",
+}
+
+// --cdi-dirs
+var actionCdiDirs = cmdline.Flag{
+	ID:           "actionCdiDirs",
+	Value:        &cdiDirs,
+	DefaultValue: []string{},
+	Name:         "cdi-dirs",
+	Usage:        "comma-separated list of directories in which CDI should look for device definition JSON files. If omitted, default will be: /etc/cdi,/var/run/cdi",
 }
 
 func init() {
@@ -843,11 +817,12 @@ func init() {
 
 		cmdManager.RegisterFlagForCmd(&actionAddCapsFlag, actionsInstanceCmd...)
 		cmdManager.RegisterFlagForCmd(&actionAllowSetuidFlag, actionsInstanceCmd...)
-		cmdManager.RegisterFlagForCmd(&actionAppFlag, actionsCmd...)
+		cmdManager.RegisterFlagForCmd(&actionAppFlag, actionsInstanceCmd...)
 		cmdManager.RegisterFlagForCmd(&actionApplyCgroupsFlag, actionsInstanceCmd...)
 		cmdManager.RegisterFlagForCmd(&actionBindFlag, actionsInstanceCmd...)
 		cmdManager.RegisterFlagForCmd(&actionCleanEnvFlag, actionsInstanceCmd...)
 		cmdManager.RegisterFlagForCmd(&actionCompatFlag, actionsInstanceCmd...)
+		cmdManager.RegisterFlagForCmd(&actionNoCompatFlag, actionsInstanceCmd...)
 		cmdManager.RegisterFlagForCmd(&actionContainAllFlag, actionsInstanceCmd...)
 		cmdManager.RegisterFlagForCmd(&actionContainFlag, actionsInstanceCmd...)
 		cmdManager.RegisterFlagForCmd(&actionContainLibsFlag, actionsInstanceCmd...)
@@ -878,19 +853,15 @@ func init() {
 		cmdManager.RegisterFlagForCmd(&commonPromptForPassphraseFlag, actionsInstanceCmd...)
 		cmdManager.RegisterFlagForCmd(&commonPEMFlag, actionsInstanceCmd...)
 		cmdManager.RegisterFlagForCmd(&actionPidNamespaceFlag, actionsCmd...)
+		cmdManager.RegisterFlagForCmd(&actionNoPidNamespaceFlag, actionsCmd...)
+		cmdManager.RegisterFlagForCmd(&actionCwdFlag, actionsCmd...)
 		cmdManager.RegisterFlagForCmd(&actionPwdFlag, actionsCmd...)
 		cmdManager.RegisterFlagForCmd(&actionScratchFlag, actionsInstanceCmd...)
 		cmdManager.RegisterFlagForCmd(&actionSecurityFlag, actionsInstanceCmd...)
 		cmdManager.RegisterFlagForCmd(&actionShellFlag, ShellCmd)
-		cmdManager.RegisterFlagForCmd(&actionSyOSFlag, ShellCmd)
 		cmdManager.RegisterFlagForCmd(&actionTmpDirFlag, actionsInstanceCmd...)
 		cmdManager.RegisterFlagForCmd(&actionUserNamespaceFlag, actionsInstanceCmd...)
 		cmdManager.RegisterFlagForCmd(&actionUtsNamespaceFlag, actionsInstanceCmd...)
-		cmdManager.RegisterFlagForCmd(&actionVMCPUFlag, actionsCmd...)
-		cmdManager.RegisterFlagForCmd(&actionVMErrFlag, actionsCmd...)
-		cmdManager.RegisterFlagForCmd(&actionVMFlag, actionsCmd...)
-		cmdManager.RegisterFlagForCmd(&actionVMIPFlag, actionsCmd...)
-		cmdManager.RegisterFlagForCmd(&actionVMRAMFlag, actionsCmd...)
 		cmdManager.RegisterFlagForCmd(&actionWorkdirFlag, actionsInstanceCmd...)
 		cmdManager.RegisterFlagForCmd(&actionWritableFlag, actionsInstanceCmd...)
 		cmdManager.RegisterFlagForCmd(&actionWritableTmpfsFlag, actionsInstanceCmd...)
@@ -917,6 +888,10 @@ func init() {
 		cmdManager.RegisterFlagForCmd(&actionPidsLimitFlag, actionsInstanceCmd...)
 		cmdManager.RegisterFlagForCmd(&actionSIFFUSEFlag, actionsCmd...)
 		cmdManager.RegisterFlagForCmd(&actionProotFlag, actionsCmd...)
-		cmdManager.RegisterFlagForCmd(&actionOCIFlag, actionsCmd...)
+		cmdManager.RegisterFlagForCmd(&commonOCIFlag, actionsInstanceCmd...)
+		cmdManager.RegisterFlagForCmd(&commonNoOCIFlag, actionsInstanceCmd...)
+		cmdManager.RegisterFlagForCmd(&actionNoTmpSandbox, actionsInstanceCmd...)
+		cmdManager.RegisterFlagForCmd(&actionDevice, actionsCmd...)
+		cmdManager.RegisterFlagForCmd(&actionCdiDirs, actionsCmd...)
 	})
 }
