@@ -1,5 +1,7 @@
 // Copyright (c) 2020, Control Command Inc. All rights reserved.
 // Copyright (c) 2018-2023, Sylabs Inc. All rights reserved.
+// Copyright (c) Contributors to the Apptainer project, established as
+//   Apptainer a Series of LF Projects LLC.
 // This software is licensed under a 3-clause BSD license. Please consult the
 // LICENSE.md file distributed with the sources of this project regarding your
 // rights to use or distribute this software.
@@ -14,41 +16,43 @@ import (
 
 	ocitypes "github.com/containers/image/v5/types"
 	"github.com/spf13/cobra"
-	"github.com/sylabs/singularity/docs"
-	"github.com/sylabs/singularity/internal/pkg/util/fs"
-	"github.com/sylabs/singularity/internal/pkg/util/interactive"
-	"github.com/sylabs/singularity/pkg/build/types"
-	"github.com/sylabs/singularity/pkg/build/types/parser"
-	"github.com/sylabs/singularity/pkg/cmdline"
-	"github.com/sylabs/singularity/pkg/image"
-	"github.com/sylabs/singularity/pkg/sylog"
+	"github.com/sylabs/singularity/v4/docs"
+	"github.com/sylabs/singularity/v4/internal/pkg/util/fs"
+	"github.com/sylabs/singularity/v4/internal/pkg/util/interactive"
+	"github.com/sylabs/singularity/v4/pkg/build/types"
+	"github.com/sylabs/singularity/v4/pkg/build/types/parser"
+	"github.com/sylabs/singularity/v4/pkg/cmdline"
+	"github.com/sylabs/singularity/v4/pkg/image"
+	"github.com/sylabs/singularity/v4/pkg/sylog"
 	"golang.org/x/term"
 )
 
 var buildArgs struct {
-	sections      []string
-	bindPaths     []string
-	mounts        []string
-	arch          string
-	builderURL    string
-	libraryURL    string
-	keyServerURL  string
-	webURL        string
-	detached      bool
-	encrypt       bool
-	fakeroot      bool
-	fixPerms      bool
-	isJSON        bool
-	noCleanUp     bool
-	noTest        bool
-	noSetgroups   bool
-	remote        bool
-	sandbox       bool
-	update        bool
-	nvidia        bool
-	nvccli        bool
-	rocm          bool
-	writableTmpfs bool // For test section only
+	sections        []string
+	bindPaths       []string
+	mounts          []string
+	arch            string
+	builderURL      string
+	libraryURL      string
+	keyServerURL    string
+	webURL          string
+	detached        bool
+	encrypt         bool
+	fakeroot        bool
+	fixPerms        bool
+	isJSON          bool
+	noCleanUp       bool
+	noTest          bool
+	noSetgroups     bool
+	remote          bool
+	sandbox         bool
+	update          bool
+	nvidia          bool
+	nvccli          bool
+	rocm            bool
+	writableTmpfs   bool     // For test section only
+	buildVarArgs    []string // Variables passed to build procedure.
+	buildVarArgFile string   // Variables file passed to build procedure.
 }
 
 // -s|--sandbox
@@ -287,6 +291,24 @@ var buildWritableTmpfsFlag = cmdline.Flag{
 	EnvKeys:      []string{"WRITABLE_TMPFS"},
 }
 
+// --build-arg
+var buildVarArgsFlag = cmdline.Flag{
+	ID:           "buildVarArgsFlag",
+	Value:        &buildArgs.buildVarArgs,
+	DefaultValue: []string{},
+	Name:         "build-arg",
+	Usage:        "defines variable=value to replace {{ variable }} entries in build definition file",
+}
+
+// --build-arg-file
+var buildVarArgFileFlag = cmdline.Flag{
+	ID:           "buildVarArgFileFlag",
+	Value:        &buildArgs.buildVarArgFile,
+	DefaultValue: "",
+	Name:         "build-arg-file",
+	Usage:        "specifies a file containing variable=value lines to replace '{{ variable }}' with value in build definition files",
+}
+
 func init() {
 	addCmdInit(func(cmdManager *cmdline.CommandManager) {
 		cmdManager.RegisterCmd(buildCmd)
@@ -325,6 +347,11 @@ func init() {
 		cmdManager.RegisterFlagForCmd(&buildBindFlag, buildCmd)
 		cmdManager.RegisterFlagForCmd(&buildMountFlag, buildCmd)
 		cmdManager.RegisterFlagForCmd(&buildWritableTmpfsFlag, buildCmd)
+		cmdManager.RegisterFlagForCmd(&buildVarArgsFlag, buildCmd)
+		cmdManager.RegisterFlagForCmd(&buildVarArgFileFlag, buildCmd)
+
+		cmdManager.RegisterFlagForCmd(&commonOCIFlag, buildCmd)
+		cmdManager.RegisterFlagForCmd(&commonNoOCIFlag, buildCmd)
 	})
 }
 
@@ -342,13 +369,17 @@ var buildCmd = &cobra.Command{
 	TraverseChildren: true,
 }
 
-func preRun(cmd *cobra.Command, args []string) {
+func preRun(cmd *cobra.Command, _ []string) {
+	if isOCI {
+		sylog.Fatalf("Builds are not yet supported in OCI-mode. Omit --oci, or use --no-oci, to build a non-OCI Singularity container.")
+	}
+
 	if buildArgs.noSetgroups && !buildArgs.fakeroot {
 		sylog.Warningf("--no-setgroups only applies to --fakeroot builds")
 	}
 
 	if buildArgs.fakeroot && !buildArgs.remote {
-		fakerootExec(args)
+		fakerootExec()
 	}
 
 	// Always perform remote build when builder flag is set
@@ -398,7 +429,7 @@ func checkBuildTarget(path string) error {
 				return fmt.Errorf("build target '%s' already exists. Use --force if you want to overwrite it", f.Name())
 			}
 
-			question := fmt.Sprintf("Build target '%s' already exists and will be deleted during the build process. Do you want to continue? [N/y] ", f.Name())
+			question := fmt.Sprintf("Build target '%s' already exists and will be deleted during the build process. Do you want to continue? [y/N] ", f.Name())
 
 			img, err := image.Init(abspath, false)
 			if err != nil {
@@ -406,7 +437,7 @@ func checkBuildTarget(path string) error {
 					return fmt.Errorf("while determining '%s' format: %s", f.Name(), err)
 				}
 				// unknown image file format
-				question = fmt.Sprintf("Build target '%s' may be a definition file or a text/binary file that will be overwritten. Do you still want to overwrite it? [N/y] ", f.Name())
+				question = fmt.Sprintf("Build target '%s' may be a definition file or a text/binary file that will be overwritten. Do you still want to overwrite it? [y/N] ", f.Name())
 			} else {
 				img.File.Close()
 			}
@@ -427,7 +458,7 @@ func checkBuildTarget(path string) error {
 }
 
 // definitionFromSpec is specifically for parsing specs for the remote builder
-// it uses a different version the the definition struct and parser
+// it uses a different version the definition struct and parser
 func definitionFromSpec(spec string) (types.Definition, error) {
 	// Try spec as URI first
 	def, err := types.NewDefinitionFromURI(spec)
