@@ -1,5 +1,7 @@
 // Copyright (c) 2020, Control Command Inc. All rights reserved.
 // Copyright (c) 2018-2023, Sylabs Inc. All rights reserved.
+// Copyright (c) Contributors to the Apptainer project, established as
+//   Apptainer a Series of LF Projects LLC.
 // This software is licensed under a 3-clause BSD license. Please consult the
 // LICENSE.md file distributed with the sources of this project regarding your
 // rights to use or distribute this software.
@@ -18,25 +20,28 @@ import (
 
 	"github.com/spf13/cobra"
 	keyclient "github.com/sylabs/scs-key-client/client"
-	"github.com/sylabs/singularity/internal/pkg/build"
-	"github.com/sylabs/singularity/internal/pkg/build/remotebuilder"
-	"github.com/sylabs/singularity/internal/pkg/buildcfg"
-	"github.com/sylabs/singularity/internal/pkg/cache"
-	"github.com/sylabs/singularity/internal/pkg/remote/endpoint"
-	fakerootConfig "github.com/sylabs/singularity/internal/pkg/runtime/engine/fakeroot/config"
-	"github.com/sylabs/singularity/internal/pkg/util/bin"
-	"github.com/sylabs/singularity/internal/pkg/util/fs"
-	"github.com/sylabs/singularity/internal/pkg/util/interactive"
-	"github.com/sylabs/singularity/internal/pkg/util/starter"
-	"github.com/sylabs/singularity/internal/pkg/util/user"
-	"github.com/sylabs/singularity/pkg/build/types"
-	"github.com/sylabs/singularity/pkg/image"
-	"github.com/sylabs/singularity/pkg/runtime/engine/config"
-	"github.com/sylabs/singularity/pkg/sylog"
-	"github.com/sylabs/singularity/pkg/util/cryptkey"
+	"github.com/sylabs/singularity/v4/internal/pkg/build"
+	"github.com/sylabs/singularity/v4/internal/pkg/build/args"
+	"github.com/sylabs/singularity/v4/internal/pkg/build/remotebuilder"
+	"github.com/sylabs/singularity/v4/internal/pkg/buildcfg"
+	"github.com/sylabs/singularity/v4/internal/pkg/cache"
+	"github.com/sylabs/singularity/v4/internal/pkg/ociplatform"
+	"github.com/sylabs/singularity/v4/internal/pkg/remote/endpoint"
+	fakerootConfig "github.com/sylabs/singularity/v4/internal/pkg/runtime/engine/fakeroot/config"
+	"github.com/sylabs/singularity/v4/internal/pkg/util/bin"
+	"github.com/sylabs/singularity/v4/internal/pkg/util/fs"
+	"github.com/sylabs/singularity/v4/internal/pkg/util/interactive"
+	"github.com/sylabs/singularity/v4/internal/pkg/util/rootless"
+	"github.com/sylabs/singularity/v4/internal/pkg/util/starter"
+	"github.com/sylabs/singularity/v4/internal/pkg/util/user"
+	"github.com/sylabs/singularity/v4/pkg/build/types"
+	"github.com/sylabs/singularity/v4/pkg/image"
+	"github.com/sylabs/singularity/v4/pkg/runtime/engine/config"
+	"github.com/sylabs/singularity/v4/pkg/sylog"
+	"github.com/sylabs/singularity/v4/pkg/util/cryptkey"
 )
 
-func fakerootExec(cmdArgs []string) {
+func fakerootExec() {
 	if buildArgs.nvccli && !buildArgs.noTest {
 		sylog.Warningf("Due to writable-tmpfs limitations, %%test sections will fail with --nvccli & --fakeroot")
 		sylog.Infof("Use -T / --notest to disable running tests during the build")
@@ -65,7 +70,6 @@ func fakerootExec(cmdArgs []string) {
 			args[idx] = arg
 			idx++
 		}
-
 	}
 
 	user, err := user.GetPwUID(uint32(os.Getuid()))
@@ -73,11 +77,12 @@ func fakerootExec(cmdArgs []string) {
 		sylog.Fatalf("failed to retrieve user information: %s", err)
 	}
 
-	// Append the user's real UID to the environment as _CONTAINERS_ROOTLESS_UID.
+	// Append the user's real UID/GID to the environment as _CONTAINERS_ROOTLESS_UID/GID.
 	// This is required in fakeroot builds that may use containers/image 5.7 and above.
 	// https://github.com/containers/image/issues/1066
 	// https://github.com/containers/image/blob/master/internal/rootless/rootless.go
-	os.Setenv("_CONTAINERS_ROOTLESS_UID", strconv.Itoa(os.Getuid()))
+	os.Setenv(rootless.UIDEnv, strconv.Itoa(os.Getuid()))
+	os.Setenv(rootless.GIDEnv, strconv.Itoa(os.Getgid()))
 
 	engineConfig := &fakerootConfig.EngineConfig{
 		Args:        args,
@@ -180,6 +185,10 @@ func runBuildRemote(ctx context.Context, cmd *cobra.Command, dst, spec string) {
 		sylog.Fatalf("Building encrypted container with the remote builder is not currently supported.")
 	}
 
+	if (len(buildArgs.buildVarArgs) > 1) || (buildArgs.buildVarArgFile != "") {
+		sylog.Fatalf("The remote builder does not currently support build-argument substitution (--build-arg / --build-arg-file).")
+	}
+
 	// TODO - the keyserver config needs to go to the remote builder for fingerprint verification at
 	// build time to be fully supported.
 
@@ -330,7 +339,11 @@ func runBuildLocal(ctx context.Context, cmd *cobra.Command, dst, spec string) {
 	}
 
 	// parse definition to determine build source
-	defs, err := build.MakeAllDefs(spec)
+	buildArgsMap, err := args.ReadBuildArgs(buildArgs.buildVarArgs, buildArgs.buildVarArgFile)
+	if err != nil {
+		sylog.Fatalf("While processing the definition file: %v", err)
+	}
+	defs, err := build.MakeAllDefs(spec, buildArgsMap)
 	if err != nil {
 		sylog.Fatalf("Unable to build from %s: %v", spec, err)
 	}
@@ -380,7 +393,11 @@ func runBuildLocal(ctx context.Context, cmd *cobra.Command, dst, spec string) {
 	if buildArgs.sandbox {
 		buildFormat = "sandbox"
 		sandboxTarget = true
+	}
 
+	dp, err := ociplatform.DefaultPlatform()
+	if err != nil {
+		sylog.Fatalf("%v", err)
 	}
 
 	b, err := build.New(
@@ -406,6 +423,9 @@ func runBuildLocal(ctx context.Context, cmd *cobra.Command, dst, spec string) {
 				EncryptionKeyInfo: keyInfo,
 				FixPerms:          buildArgs.fixPerms,
 				SandboxTarget:     sandboxTarget,
+				// Only perform a build with the host DefaultPlatform at present.
+				// TODO: rework --arch handling for remote builds so that local builds can specify --arch and --platform.
+				Platform: *dp,
 			},
 		})
 	if err != nil {
