@@ -7,8 +7,8 @@ package cli
 
 import (
 	"github.com/spf13/cobra"
-	"github.com/sylabs/singularity/docs"
-	"github.com/sylabs/singularity/pkg/cmdline"
+	"github.com/sylabs/singularity/v4/docs"
+	"github.com/sylabs/singularity/v4/pkg/cmdline"
 )
 
 // configCmd is the config command
