@@ -4,8 +4,8 @@ import (
 	"errors"
 
 	"github.com/spf13/cobra"
-	"github.com/sylabs/singularity/docs"
-	"github.com/sylabs/singularity/pkg/cmdline"
+	"github.com/sylabs/singularity/v4/docs"
+	"github.com/sylabs/singularity/v4/pkg/cmdline"
 )
 
 func init() {
