@@ -7,9 +7,9 @@ package cli
 
 import (
 	"github.com/spf13/cobra"
-	"github.com/sylabs/singularity/docs"
-	"github.com/sylabs/singularity/internal/app/singularity"
-	"github.com/sylabs/singularity/pkg/sylog"
+	"github.com/sylabs/singularity/v4/docs"
+	"github.com/sylabs/singularity/v4/internal/app/singularity"
+	"github.com/sylabs/singularity/v4/pkg/sylog"
 )
 
 // PluginInstallCmd takes a compiled plugin.sif file and installs it
