@@ -1,5 +1,5 @@
 // Copyright (c) 2020, Control Command Inc. All rights reserved.
-// Copyright (c) 2018-2021, Sylabs Inc. All rights reserved.
+// Copyright (c) 2018-2023, Sylabs Inc. All rights reserved.
 // This software is licensed under a 3-clause BSD license. Please consult the
 // LICENSE.md file distributed with the sources of this project regarding your
 // rights to use or distribute this software.
@@ -9,20 +9,19 @@ package cli
 import (
 	"os"
 	"path/filepath"
-	"runtime"
 
 	"github.com/spf13/cobra"
-	"github.com/sylabs/singularity/docs"
-	"github.com/sylabs/singularity/internal/pkg/cache"
-	"github.com/sylabs/singularity/internal/pkg/client/library"
-	"github.com/sylabs/singularity/internal/pkg/client/net"
-	"github.com/sylabs/singularity/internal/pkg/client/oci"
-	"github.com/sylabs/singularity/internal/pkg/client/oras"
-	"github.com/sylabs/singularity/internal/pkg/client/shub"
-	"github.com/sylabs/singularity/internal/pkg/remote/endpoint"
-	"github.com/sylabs/singularity/internal/pkg/util/uri"
-	"github.com/sylabs/singularity/pkg/cmdline"
-	"github.com/sylabs/singularity/pkg/sylog"
+	"github.com/sylabs/singularity/v4/docs"
+	"github.com/sylabs/singularity/v4/internal/pkg/cache"
+	"github.com/sylabs/singularity/v4/internal/pkg/client/library"
+	"github.com/sylabs/singularity/v4/internal/pkg/client/net"
+	"github.com/sylabs/singularity/v4/internal/pkg/client/oci"
+	"github.com/sylabs/singularity/v4/internal/pkg/client/oras"
+	"github.com/sylabs/singularity/v4/internal/pkg/client/shub"
+	"github.com/sylabs/singularity/v4/internal/pkg/remote/endpoint"
+	"github.com/sylabs/singularity/v4/internal/pkg/util/uri"
+	"github.com/sylabs/singularity/v4/pkg/cmdline"
+	"github.com/sylabs/singularity/v4/pkg/sylog"
 )
 
 const (
@@ -38,6 +37,8 @@ const (
 	HTTPSProtocol = "https"
 	// OrasProtocol holds the oras URI.
 	OrasProtocol = "oras"
+	// Docker Registry protocol
+	DockerProtocol = "docker"
 )
 
 var (
@@ -49,21 +50,8 @@ var (
 	unauthenticatedPull bool
 	// pullDir is the path that the containers will be pulled to, if set.
 	pullDir string
-	// pullArch is the architecture for which containers will be pulled from the
-	// SCS library.
-	pullArch string
 )
 
-// --arch
-var pullArchFlag = cmdline.Flag{
-	ID:           "pullArchFlag",
-	Value:        &pullArch,
-	DefaultValue: runtime.GOARCH,
-	Name:         "arch",
-	Usage:        "architecture to pull from library",
-	EnvKeys:      []string{"PULL_ARCH"},
-}
-
 // --library
 var pullLibraryURIFlag = cmdline.Flag{
 	ID:           "pullLibraryURIFlag",
@@ -149,7 +137,12 @@ func init() {
 		cmdManager.RegisterFlagForCmd(&buildNoCleanupFlag, PullCmd)
 		cmdManager.RegisterFlagForCmd(&pullAllowUnsignedFlag, PullCmd)
 		cmdManager.RegisterFlagForCmd(&pullAllowUnauthenticatedFlag, PullCmd)
-		cmdManager.RegisterFlagForCmd(&pullArchFlag, PullCmd)
+
+		cmdManager.RegisterFlagForCmd(&commonOCIFlag, PullCmd)
+		cmdManager.RegisterFlagForCmd(&commonNoOCIFlag, PullCmd)
+
+		cmdManager.RegisterFlagForCmd(&commonArchFlag, PullCmd)
+		cmdManager.RegisterFlagForCmd(&commonPlatformFlag, PullCmd)
 	})
 }
 
@@ -178,14 +171,19 @@ func pullRun(cmd *cobra.Command, args []string) {
 		sylog.Fatalf("Bad URI %s", pullFrom)
 	}
 
+	suffix := "sif"
+	if isOCI {
+		suffix = "oci.sif"
+	}
+
 	pullTo := pullImageName
 	if pullTo == "" {
 		pullTo = args[0]
 		if len(args) == 1 {
 			if transport == "" {
-				pullTo = uri.GetName("library://" + pullFrom)
+				pullTo = uri.Filename("library://"+pullFrom, suffix)
 			} else {
-				pullTo = uri.GetName(pullFrom) // TODO: If not library/shub & no name specified, simply put to cache
+				pullTo = uri.Filename(pullFrom, suffix) // TODO: If not library/shub & no name specified, simply put to cache
 			}
 		}
 	}
@@ -234,7 +232,15 @@ func pullRun(cmd *cobra.Command, args []string) {
 			sylog.Fatalf("Unable to get keyserver client configuration: %v", err)
 		}
 
-		_, err = library.PullToFile(ctx, imgCache, pullTo, ref, pullArch, tmpDir, lc, co)
+		pullOpts := library.PullOptions{
+			Endpoint:      currentRemoteEndpoint,
+			KeyClientOpts: co,
+			LibraryConfig: lc,
+			RequireOciSif: isOCI,
+			TmpDir:        tmpDir,
+			Platform:      getOCIPlatform(),
+		}
+		_, err = library.PullToFile(ctx, imgCache, pullTo, ref, pullOpts)
 		if err != nil && err != library.ErrLibraryPullUnsigned {
 			sylog.Fatalf("While pulling library image: %v", err)
 		}
@@ -242,22 +248,43 @@ func pullRun(cmd *cobra.Command, args []string) {
 			sylog.Warningf("Skipping container verification")
 		}
 	case ShubProtocol:
-		_, err := shub.PullToFile(ctx, imgCache, pullTo, pullFrom, tmpDir, noHTTPS)
+		if isOCI {
+			sylog.Fatalf("Pull from shub:// to OCI-SIF (--oci) is not supported. Omit --oci, or use --no-oci, to pull a non-OCI Singularity container.")
+		}
+		if platform != "" || arch != "" {
+			sylog.Warningf("Pull from shub:// is a direct download, --arch and --platform have no effect.")
+		}
+
+		_, err := shub.PullToFile(ctx, imgCache, pullTo, pullFrom, noHTTPS)
 		if err != nil {
 			sylog.Fatalf("While pulling shub image: %v\n", err)
 		}
 	case OrasProtocol:
+		if isOCI {
+			sylog.Warningf("Pull from oras:// URIs is a direct download, --oci has no effect.")
+		}
+		if platform != "" || arch != "" {
+			sylog.Warningf("Pull from oras:// is a direct download, --arch and --platform have no effect.")
+		}
+
 		ociAuth, err := makeDockerCredentials(cmd)
 		if err != nil {
 			sylog.Fatalf("Unable to make docker oci credentials: %s", err)
 		}
 
-		_, err = oras.PullToFile(ctx, imgCache, pullTo, pullFrom, tmpDir, ociAuth)
+		_, err = oras.PullToFile(ctx, imgCache, pullTo, pullFrom, ociAuth)
 		if err != nil {
 			sylog.Fatalf("While pulling image from oci registry: %v", err)
 		}
 	case HTTPProtocol, HTTPSProtocol:
-		_, err := net.PullToFile(ctx, imgCache, pullTo, pullFrom, tmpDir)
+		if isOCI {
+			sylog.Warningf("Pull from http[s]:// URIs is a direct download, --oci has no effect.")
+		}
+		if platform != "" || arch != "" {
+			sylog.Warningf("Pull from http[s]:// is a direct download, --arch and --platform have no effect.")
+		}
+
+		_, err := net.PullToFile(ctx, imgCache, pullTo, pullFrom)
 		if err != nil {
 			sylog.Fatalf("While pulling from image from http(s): %v\n", err)
 		}
@@ -273,6 +300,8 @@ func pullRun(cmd *cobra.Command, args []string) {
 			DockerHost: dockerHost,
 			NoHTTPS:    noHTTPS,
 			NoCleanUp:  buildArgs.noCleanUp,
+			OciSif:     isOCI,
+			Platform:   getOCIPlatform(),
 		}
 
 		_, err = oci.PullToFile(ctx, imgCache, pullTo, pullFrom, pullOpts)
