@@ -1,5 +1,5 @@
 // Copyright (c) 2020, Control Command Inc. All rights reserved.
-// Copyright (c) 2018-2022, Sylabs Inc. All rights reserved.
+// Copyright (c) 2018-2023, Sylabs Inc. All rights reserved.
 // This software is licensed under a 3-clause BSD license. Please consult the
 // LICENSE.md file distributed with the sources of this project regarding your
 // rights to use or distribute this software.
@@ -20,20 +20,24 @@ import (
 	"text/template"
 
 	ocitypes "github.com/containers/image/v5/types"
+	ggcrv1 "github.com/google/go-containerregistry/pkg/v1"
 	"github.com/spf13/cobra"
 	scskeyclient "github.com/sylabs/scs-key-client/client"
 	scslibclient "github.com/sylabs/scs-library-client/client"
-	"github.com/sylabs/singularity/docs"
-	"github.com/sylabs/singularity/internal/pkg/buildcfg"
-	"github.com/sylabs/singularity/internal/pkg/plugin"
-	"github.com/sylabs/singularity/internal/pkg/remote"
-	"github.com/sylabs/singularity/internal/pkg/remote/endpoint"
-	"github.com/sylabs/singularity/internal/pkg/util/fs"
-	"github.com/sylabs/singularity/pkg/cmdline"
-	clicallback "github.com/sylabs/singularity/pkg/plugin/callback/cli"
-	"github.com/sylabs/singularity/pkg/syfs"
-	"github.com/sylabs/singularity/pkg/sylog"
-	"github.com/sylabs/singularity/pkg/util/singularityconf"
+	"github.com/sylabs/singularity/v4/docs"
+	"github.com/sylabs/singularity/v4/internal/pkg/buildcfg"
+	"github.com/sylabs/singularity/v4/internal/pkg/ociplatform"
+	"github.com/sylabs/singularity/v4/internal/pkg/plugin"
+	"github.com/sylabs/singularity/v4/internal/pkg/remote"
+	"github.com/sylabs/singularity/v4/internal/pkg/remote/endpoint"
+	ocilauncher "github.com/sylabs/singularity/v4/internal/pkg/runtime/launcher/oci"
+	"github.com/sylabs/singularity/v4/internal/pkg/util/fs"
+	"github.com/sylabs/singularity/v4/internal/pkg/util/rootless"
+	"github.com/sylabs/singularity/v4/pkg/cmdline"
+	clicallback "github.com/sylabs/singularity/v4/pkg/plugin/callback/cli"
+	"github.com/sylabs/singularity/v4/pkg/syfs"
+	"github.com/sylabs/singularity/v4/pkg/sylog"
+	"github.com/sylabs/singularity/v4/pkg/util/singularityconf"
 	"golang.org/x/term"
 )
 
@@ -47,23 +51,11 @@ var CurrentUser = getCurrentUser()
 // currentRemoteEndpoint holds the current remote endpoint
 var currentRemoteEndpoint *endpoint.Config
 
-var (
-	dockerAuthConfig ocitypes.DockerAuthConfig
-	dockerLogin      bool
-	dockerHost       string
-
-	encryptionPEMPath   string
-	promptForPassphrase bool
-	forceOverwrite      bool
-	noHTTPS             bool
-	tmpDir              string
-)
-
 const (
 	envPrefix = "SINGULARITY_"
 )
 
-// singularity command flags
+// Top level options on the `singularity` root command.
 var (
 	debug   bool
 	nocolor bool
@@ -74,6 +66,39 @@ var (
 	configurationFile string
 )
 
+// Common options used with multiple sub-commands.
+var (
+	// OCI Registry Authentication
+	dockerAuthConfig ocitypes.DockerAuthConfig
+	dockerLogin      bool
+	dockerHost       string
+	noHTTPS          bool
+
+	// Encryption Material
+	encryptionPEMPath   string
+	promptForPassphrase bool
+
+	// Paths / file handling
+	tmpDir         string
+	forceOverwrite bool
+
+	// Options controlling the unpacking of images to temporary sandboxes
+	canUseTmpSandbox bool
+	noTmpSandbox     bool
+
+	// Use OCI runtime and OCI SIF?
+	isOCI bool
+	noOCI bool
+
+	// Platform for retrieving images
+	arch     string
+	platform string
+)
+
+//
+// Top level option flags
+//
+
 // -d|--debug
 var singDebugFlag = cmdline.Flag{
 	ID:           "singDebugFlag",
@@ -124,6 +149,21 @@ var singVerboseFlag = cmdline.Flag{
 	Usage:        "print additional information",
 }
 
+// -c|--config
+var singConfigFileFlag = cmdline.Flag{
+	ID:           "singConfigFileFlag",
+	Value:        &configurationFile,
+	DefaultValue: buildcfg.SINGULARITY_CONF_FILE,
+	Name:         "config",
+	ShortHand:    "c",
+	Usage:        "specify a configuration file (for root or unprivileged installation only)",
+	EnvKeys:      []string{"CONFIG_FILE"},
+}
+
+//
+// Common option flags for multiple subcommands
+//
+
 // --docker-username
 var dockerUsernameFlag = cmdline.Flag{
 	ID:            "dockerUsernameFlag",
@@ -169,6 +209,26 @@ var dockerHostFlag = cmdline.Flag{
 	WithoutPrefix: true,
 }
 
+// --no-https
+var commonNoHTTPSFlag = cmdline.Flag{
+	ID:           "commonNoHTTPSFlag",
+	Value:        &noHTTPS,
+	DefaultValue: false,
+	Name:         "no-https",
+	Usage:        "use http instead of https for docker:// oras:// and library://<hostname>/... URIs",
+	EnvKeys:      []string{"NOHTTPS", "NO_HTTPS"},
+}
+
+// --nohttps (deprecated)
+var commonOldNoHTTPSFlag = cmdline.Flag{
+	ID:           "commonOldNoHTTPSFlag",
+	Value:        &noHTTPS,
+	DefaultValue: false,
+	Name:         "nohttps",
+	Deprecated:   "use --no-https",
+	Usage:        "use http instead of https for docker:// oras:// and library://<hostname>/... URIs",
+}
+
 // --passphrase
 var commonPromptForPassphraseFlag = cmdline.Flag{
 	ID:           "commonPromptForPassphraseFlag",
@@ -198,26 +258,6 @@ var commonForceFlag = cmdline.Flag{
 	EnvKeys:      []string{"FORCE"},
 }
 
-// --no-https
-var commonNoHTTPSFlag = cmdline.Flag{
-	ID:           "commonNoHTTPSFlag",
-	Value:        &noHTTPS,
-	DefaultValue: false,
-	Name:         "no-https",
-	Usage:        "use http instead of https for docker:// oras:// and library://<hostname>/... URIs",
-	EnvKeys:      []string{"NOHTTPS", "NO_HTTPS"},
-}
-
-// --nohttps (deprecated)
-var commonOldNoHTTPSFlag = cmdline.Flag{
-	ID:           "commonOldNoHTTPSFlag",
-	Value:        &noHTTPS,
-	DefaultValue: false,
-	Name:         "nohttps",
-	Deprecated:   "use --no-https",
-	Usage:        "use http instead of https for docker:// oras:// and library://<hostname>/... URIs",
-}
-
 // --tmpdir
 var commonTmpDirFlag = cmdline.Flag{
 	ID:           "commonTmpDirFlag",
@@ -229,15 +269,54 @@ var commonTmpDirFlag = cmdline.Flag{
 	EnvKeys:      []string{"TMPDIR"},
 }
 
-// -c|--config
-var singConfigFileFlag = cmdline.Flag{
-	ID:           "singConfigFileFlag",
-	Value:        &configurationFile,
-	DefaultValue: buildcfg.SINGULARITY_CONF_FILE,
-	Name:         "config",
-	ShortHand:    "c",
-	Usage:        "specify a configuration file (for root or unprivileged installation only)",
-	EnvKeys:      []string{"CONFIG_FILE"},
+// --oci
+var commonOCIFlag = cmdline.Flag{
+	ID:           "actionOCI",
+	Value:        &isOCI,
+	DefaultValue: false,
+	Name:         "oci",
+	Usage:        "Launch container with OCI runtime (experimental)",
+	EnvKeys:      []string{"OCI"},
+}
+
+// --no-oci
+var commonNoOCIFlag = cmdline.Flag{
+	ID:           "commonNoOCI",
+	Value:        &noOCI,
+	DefaultValue: false,
+	Name:         "no-oci",
+	Usage:        "Launch container with native runtime",
+	EnvKeys:      []string{"NO_OCI"},
+}
+
+// --no-tmp-sandbox
+var actionNoTmpSandbox = cmdline.Flag{
+	ID:           "actionNoTmpSandbox",
+	Value:        &noTmpSandbox,
+	DefaultValue: false,
+	Name:         "no-tmp-sandbox",
+	Usage:        "Prohibits unpacking of images into temporary sandbox dirs",
+	EnvKeys:      []string{"NO_TMP_SANDBOX"},
+}
+
+// --arch
+var commonArchFlag = cmdline.Flag{
+	ID:           "commonArchFlag",
+	Value:        &arch,
+	DefaultValue: "",
+	Name:         "arch",
+	Usage:        "architecture to use when pulling images",
+	EnvKeys:      []string{"PULL_ARCH", "ARCH"},
+}
+
+// --platform
+var commonPlatformFlag = cmdline.Flag{
+	ID:           "commonPlatformFlag",
+	Value:        &platform,
+	DefaultValue: "",
+	Name:         "platform",
+	Usage:        "platform (OS/Architecture/Variant) to use when pulling images",
+	EnvKeys:      []string{"PLATFORM"},
 }
 
 func getCurrentUser() *user.User {
@@ -334,6 +413,28 @@ func persistentPreRun(*cobra.Command, []string) error {
 	}
 	singularityconf.SetCurrentConfig(config)
 
+	// Honor 'oci mode' in singularity.conf, and allow negation with `--no-oci`.
+	if isOCI && noOCI {
+		return fmt.Errorf("--oci and --no-oci cannot be used together")
+	}
+	isOCI = isOCI || config.OCIMode
+	if noOCI {
+		isOCI = false
+	}
+
+	// Honor 'tmp sandbox' in singularity.conf, and allow negation with
+	// `--no-tmp-sandbox`.
+	canUseTmpSandbox = config.TmpSandboxAllowed
+	if noTmpSandbox {
+		canUseTmpSandbox = false
+	}
+
+	// If we need to enter a namespace (oci-mode) do the re-exec now, before any
+	// other handling happens.
+	if err := maybeReExec(); err != nil {
+		return err
+	}
+
 	// Handle the config dir (~/.singularity),
 	// then check the remove conf file permission.
 	if err := handleConfDir(syfs.ConfigDir()); err != nil {
@@ -392,6 +493,7 @@ func Init(loadPlugins bool) {
 	}
 
 	// load plugins and register commands/flags if any
+	//nolint:forcetypeassert
 	if loadPlugins {
 		callbackType := (clicallback.Command)(nil)
 		callbacks, err := plugin.LoadCallbacks(callbackType)
@@ -613,7 +715,7 @@ func singularityExec(image string, args []string) (string, error) {
 }
 
 // CheckRoot ensures that a command is executed with root privileges.
-func CheckRoot(cmd *cobra.Command, args []string) {
+func CheckRoot(cmd *cobra.Command, _ []string) {
 	if os.Geteuid() != 0 {
 		sylog.Fatalf("%q command requires root privileges", cmd.CommandPath())
 	}
@@ -621,7 +723,7 @@ func CheckRoot(cmd *cobra.Command, args []string) {
 
 // CheckRootOrUnpriv ensures that a command is executed with root
 // privileges or that Singularity is installed unprivileged.
-func CheckRootOrUnpriv(cmd *cobra.Command, args []string) {
+func CheckRootOrUnpriv(cmd *cobra.Command, _ []string) {
 	if os.Geteuid() != 0 && buildcfg.SINGULARITY_SUID_INSTALL == 1 {
 		sylog.Fatalf("%q command requires root privileges or an unprivileged installation", cmd.CommandPath())
 	}
@@ -689,3 +791,45 @@ func getBuilderClientConfig(uri string) (baseURI, authToken string, err error) {
 
 	return currentRemoteEndpoint.BuilderClientConfig(uri)
 }
+
+func maybeReExec() error {
+	sylog.Debugf("Checking whether to re-exec")
+	// The OCI runtime must always be launched where the effective uid/gid is 0 (root or fake-root).
+	if isOCI && !rootless.InNS() {
+		// If we need to, enter a new cgroup now, to workaround an issue with crun container cgroup creation (#1538).
+		if err := ocilauncher.CrunNestCgroup(); err != nil {
+			return fmt.Errorf("while applying crun cgroup workaround: %w", err)
+		}
+		// If we are root already, run the launcher in a new mount namespace only.
+		if os.Geteuid() == 0 {
+			return rootless.RunInMountNS(os.Args[1:])
+		}
+		// If we are not root, re-exec in a root-mapped user namespace and mount namespace.
+		return rootless.ExecWithFakeroot(os.Args[1:])
+	}
+	return nil
+}
+
+// getOCIPlatform returns the appropriate OCI platform to use according to `--arch` and `--platform`
+func getOCIPlatform() ggcrv1.Platform {
+	var (
+		p   *ggcrv1.Platform
+		err error
+	)
+	if arch != "" && platform != "" {
+		err = fmt.Errorf("--arch and --platform cannot be used together")
+	}
+	if arch == "" && platform == "" {
+		p, err = ociplatform.DefaultPlatform()
+	}
+	if arch != "" {
+		p, err = ociplatform.PlatformFromArch(arch)
+	}
+	if platform != "" {
+		p, err = ociplatform.PlatformFromString(platform)
+	}
+	if err != nil {
+		sylog.Fatalf("%v", err)
+	}
+	return *p
+}
