@@ -1,10 +1,17 @@
-// Copyright (c) 2018-2021, Sylabs Inc. All rights reserved.
+// Copyright (c) 2018-2023, Sylabs Inc. All rights reserved.
+// Copyright (c) Contributors to the Apptainer project, established as
+//   Apptainer a Series of LF Projects LLC.
 // This software is licensed under a 3-clause BSD license. Please consult the
 // LICENSE.md file distributed with the sources of this project regarding your
 // rights to use or distribute this software.
 
 package main
 
+// Note that the inclusion of builddir here only works when mconfig -b has not
+//  renamed it; that is handled via a setting of CGO_CFLAGS in mconfig. It is
+//  included here also so that Go tools such as code editors and linters can
+//  find config.h when the default builddir is used.
+
 // #cgo CFLAGS: -I${SRCDIR}/../../builddir
 // #include <config.h>
 // #include "c/message.c"
@@ -17,14 +24,14 @@ import (
 	"runtime"
 	"unsafe"
 
-	"github.com/sylabs/singularity/internal/app/starter"
-	"github.com/sylabs/singularity/internal/pkg/runtime/engine"
-	starterConfig "github.com/sylabs/singularity/internal/pkg/runtime/engine/config/starter"
-	"github.com/sylabs/singularity/internal/pkg/util/mainthread"
-	"github.com/sylabs/singularity/pkg/sylog"
+	"github.com/sylabs/singularity/v4/internal/app/starter"
+	"github.com/sylabs/singularity/v4/internal/pkg/runtime/engine"
+	starterConfig "github.com/sylabs/singularity/v4/internal/pkg/runtime/engine/config/starter"
+	"github.com/sylabs/singularity/v4/internal/pkg/util/mainthread"
+	"github.com/sylabs/singularity/v4/pkg/sylog"
 
 	// register engines
-	_ "github.com/sylabs/singularity/cmd/starter/engines"
+	_ "github.com/sylabs/singularity/v4/cmd/starter/engines"
 )
 
 func getEngine(jsonConfig []byte) *engine.Engine {
