@@ -1,4 +1,4 @@
-// Copyright (c) 2019-2022, Sylabs Inc. All rights reserved.
+// Copyright (c) 2019-2023, Sylabs Inc. All rights reserved.
 // Copyright (c) Contributors to the Apptainer project, established as
 //   Apptainer a Series of LF Projects LLC.
 // This software is licensed under a 3-clause BSD license. Please consult the
@@ -22,11 +22,12 @@ import (
 	"time"
 
 	"github.com/pkg/errors"
-	"github.com/sylabs/singularity/e2e/internal/e2e"
-	"github.com/sylabs/singularity/e2e/internal/testhelper"
-	"github.com/sylabs/singularity/internal/pkg/test/tool/exec"
-	"github.com/sylabs/singularity/internal/pkg/test/tool/require"
-	"github.com/sylabs/singularity/internal/pkg/util/fs"
+	"github.com/sylabs/singularity/v4/e2e/internal/e2e"
+	"github.com/sylabs/singularity/v4/e2e/internal/testhelper"
+	"github.com/sylabs/singularity/v4/internal/pkg/test/tool/exec"
+	"github.com/sylabs/singularity/v4/internal/pkg/test/tool/require"
+	"github.com/sylabs/singularity/v4/internal/pkg/util/bin"
+	"github.com/sylabs/singularity/v4/internal/pkg/util/fs"
 )
 
 type actionTests struct {
@@ -81,39 +82,11 @@ func (c actionTests) actionRun(t *testing.T) {
 	}
 }
 
-// exec tests min fuctionality for singularity exec
+// actionExec tests min fuctionality for singularity exec
 func (c actionTests) actionExec(t *testing.T) {
 	e2e.EnsureImage(t, c.env)
 
-	user := e2e.CurrentUser(t)
-
-	// Create a temp testfile
-	testdata, err := fs.MakeTmpDir(c.env.TestDir, "testdata", 0o755)
-	if err != nil {
-		t.Fatal(err)
-	}
-	t.Cleanup(func() {
-		if !t.Failed() {
-			os.RemoveAll(testdata)
-		}
-	})
-
-	testdataTmp := filepath.Join(testdata, "tmp")
-	if err := os.Mkdir(testdataTmp, 0o755); err != nil {
-		t.Fatal(err)
-	}
-
-	// Create a temp testfile
-	tmpfile, err := fs.MakeTmpFile(testdataTmp, "testSingularityExec.", 0o644)
-	if err != nil {
-		t.Fatal(err)
-	}
-	tmpfile.Close()
-
-	basename := filepath.Base(tmpfile.Name())
-	tmpfilePath := filepath.Join("/tmp", basename)
-	vartmpfilePath := filepath.Join("/var/tmp", basename)
-	homePath := filepath.Join("/home", basename)
+	user := e2e.UserProfile.HostUser(t)
 
 	tests := []struct {
 		name        string
@@ -203,86 +176,26 @@ func (c actionTests) actionExec(t *testing.T) {
 			argv: []string{c.env.ImagePath, "test", "-d", "/scif/data/foo/input"},
 			exit: 0,
 		},
-		{
-			name: "ContainOnly",
-			argv: []string{"--contain", c.env.ImagePath, "test", "-f", tmpfilePath},
-			exit: 1,
-		},
-		{
-			name: "WorkdirOnly",
-			argv: []string{"--workdir", testdata, c.env.ImagePath, "test", "-f", tmpfilePath},
-			exit: 1,
-		},
-		{
-			name: "WorkdirContain",
-			argv: []string{"--workdir", testdata, "--contain", c.env.ImagePath, "test", "-f", tmpfilePath},
-			exit: 0,
-		},
-		{
-			name: "PwdGood",
-			argv: []string{"--pwd", "/etc", c.env.ImagePath, "true"},
-			exit: 0,
-		},
-		{
-			name: "Home",
-			argv: []string{"--home", "/myhomeloc", c.env.ImagePath, "env"},
-			wantOutputs: []e2e.SingularityCmdResultOp{
-				e2e.ExpectOutput(e2e.RegexMatch, `\bHOME=/myhomeloc\b`),
-			},
-			exit: 0,
-		},
-		{
-			name: "HomePath",
-			argv: []string{"--home", testdataTmp + ":/home", c.env.ImagePath, "test", "-f", homePath},
-			exit: 0,
-		},
-		{
-			name: "HomeTmp",
-			argv: []string{"--home", "/tmp", c.env.ImagePath, "true"},
-			exit: 0,
-		},
-		{
-			name: "HomeTmpExplicit",
-			argv: []string{"--home", "/tmp:/home", c.env.ImagePath, "true"},
-			exit: 0,
-		},
-		{
-			name: "UserBindTmp",
-			argv: []string{"--bind", testdataTmp + ":/tmp", c.env.ImagePath, "test", "-f", tmpfilePath},
-			exit: 0,
-		},
-		{
-			name: "UserBindVarTmp",
-			argv: []string{"--bind", testdataTmp + ":/var/tmp", c.env.ImagePath, "test", "-f", vartmpfilePath},
-			exit: 0,
-		},
 		{
 			name: "NoHome",
 			argv: []string{"--no-home", c.env.ImagePath, "ls", "-ld", user.Dir},
 			exit: 1,
 		},
+		// PID namespace, and override, in --containall mode. Uses --no-init to be able to check PID=1
 		{
-			name: "Hostname",
-			argv: []string{"--hostname", "whats-in-a-native-name", c.env.ImagePath, "hostname"},
+			name: "ContainAllPID",
+			argv: []string{"--containall", "--no-init", c.env.ImagePath, "sh", "-c", "echo $$"},
 			exit: 0,
 			wantOutputs: []e2e.SingularityCmdResultOp{
-				e2e.ExpectOutput(e2e.ExactMatch, "whats-in-a-native-name"),
+				e2e.ExpectOutput(e2e.ExactMatch, "1"),
 			},
 		},
 		{
-			name: "ResolvConfGoogle",
-			argv: []string{"--dns", "8.8.8.8,8.8.4.4", c.env.ImagePath, "nslookup", "w3.org"},
+			name: "ContainAllNoPID",
+			argv: []string{"--containall", "--no-init", "--no-pid", c.env.ImagePath, "sh", "-c", "echo $$"},
 			exit: 0,
 			wantOutputs: []e2e.SingularityCmdResultOp{
-				e2e.ExpectOutput(e2e.RegexMatch, `^(\s*)Server:(\s+)(8\.8\.8\.8|8\.8\.4\.4)(\s*)\n`),
-			},
-		},
-		{
-			name: "ResolvConfCloudflare",
-			argv: []string{"--dns", "1.1.1.1", c.env.ImagePath, "nslookup", "w3.org"},
-			exit: 0,
-			wantOutputs: []e2e.SingularityCmdResultOp{
-				e2e.ExpectOutput(e2e.RegexMatch, `^(\s*)Server:(\s+)(1\.1\.1\.1)(\s*)\n`),
+				e2e.ExpectOutput(e2e.UnwantedExactMatch, "1"),
 			},
 		},
 	}
@@ -300,6 +213,153 @@ func (c actionTests) actionExec(t *testing.T) {
 	}
 }
 
+// actionExecMultiProfile tests fuctionality using singularity exec under all native profiles that do not involve user namespaces.
+func (c actionTests) actionExecMultiProfile(t *testing.T) {
+	e2e.EnsureImage(t, c.env)
+
+	for _, profile := range []e2e.Profile{e2e.RootProfile, e2e.FakerootProfile, e2e.UserProfile} {
+		t.Run(profile.String(), func(t *testing.T) {
+			// Create a temp testfile
+			testdata, err := fs.MakeTmpDir(c.env.TestDir, "testdata", 0o755)
+			if err != nil {
+				t.Fatal(err)
+			}
+			t.Cleanup(func() {
+				if !t.Failed() {
+					os.RemoveAll(testdata)
+				}
+			})
+
+			testdataTmp := filepath.Join(testdata, "tmp")
+			if err := os.Mkdir(testdataTmp, 0o755); err != nil {
+				t.Fatal(err)
+			}
+
+			// Create a temp testfile
+			tmpfile, err := fs.MakeTmpFile(testdataTmp, "testSingularityExec.", 0o644)
+			if err != nil {
+				t.Fatal(err)
+			}
+			tmpfile.Close()
+
+			basename := filepath.Base(tmpfile.Name())
+			tmpfilePath := filepath.Join("/tmp", basename)
+			vartmpfilePath := filepath.Join("/var/tmp", basename)
+			homePath := filepath.Join("/home", basename)
+
+			tests := []struct {
+				name        string
+				argv        []string
+				exit        int
+				wantOutputs []e2e.SingularityCmdResultOp
+			}{
+				{
+					name: "ContainOnly",
+					argv: []string{"--contain", c.env.ImagePath, "test", "-f", tmpfilePath},
+					exit: 1,
+				},
+				{
+					name: "WorkdirOnly",
+					argv: []string{"--workdir", testdata, c.env.ImagePath, "test", "-f", tmpfilePath},
+					exit: 1,
+				},
+				{
+					name: "WorkdirContain",
+					argv: []string{"--workdir", testdata, "--contain", c.env.ImagePath, "test", "-f", tmpfilePath},
+					exit: 0,
+				},
+				{
+					name: "CwdGood",
+					argv: []string{"--cwd", "/etc", c.env.ImagePath, "true"},
+					exit: 0,
+				},
+				{
+					name: "PwdGood",
+					argv: []string{"--pwd", "/etc", c.env.ImagePath, "true"},
+					exit: 0,
+				},
+				{
+					name: "Home",
+					argv: []string{"--home", "/myhomeloc", c.env.ImagePath, "env"},
+					wantOutputs: []e2e.SingularityCmdResultOp{
+						e2e.ExpectOutput(e2e.RegexMatch, `\bHOME=/myhomeloc\b`),
+					},
+					exit: 0,
+				},
+				{
+					name: "HomePath",
+					argv: []string{"--home", testdataTmp + ":/home", c.env.ImagePath, "test", "-f", homePath},
+					exit: 0,
+				},
+				{
+					name: "HomeTmp",
+					argv: []string{"--home", "/tmp", c.env.ImagePath, "true"},
+					exit: 0,
+				},
+				{
+					name: "HomeTmpExplicit",
+					argv: []string{"--home", "/tmp:/home", c.env.ImagePath, "true"},
+					exit: 0,
+				},
+				{
+					name: "UserBindTmp",
+					argv: []string{"--bind", testdataTmp + ":/tmp", c.env.ImagePath, "test", "-f", tmpfilePath},
+					exit: 0,
+				},
+				{
+					name: "UserBindVarTmp",
+					argv: []string{"--bind", testdataTmp + ":/var/tmp", c.env.ImagePath, "test", "-f", vartmpfilePath},
+					exit: 0,
+				},
+				{
+					name: "Hostname",
+					argv: []string{"--hostname", "whats-in-a-native-name", c.env.ImagePath, "hostname"},
+					exit: 0,
+					wantOutputs: []e2e.SingularityCmdResultOp{
+						e2e.ExpectOutput(e2e.ExactMatch, "whats-in-a-native-name"),
+					},
+				},
+				{
+					name: "ResolvConfGoogle",
+					argv: []string{"--dns", "8.8.8.8,8.8.4.4", c.env.ImagePath, "nslookup", "w3.org"},
+					exit: 0,
+					wantOutputs: []e2e.SingularityCmdResultOp{
+						e2e.ExpectOutput(e2e.RegexMatch, `^(\s*)Server:(\s+)(8\.8\.8\.8|8\.8\.4\.4)(\s*)\n`),
+					},
+				},
+				{
+					name: "ResolvConfCloudflare",
+					argv: []string{"--dns", "1.1.1.1", c.env.ImagePath, "nslookup", "w3.org"},
+					exit: 0,
+					wantOutputs: []e2e.SingularityCmdResultOp{
+						e2e.ExpectOutput(e2e.RegexMatch, `^(\s*)Server:(\s+)(1\.1\.1\.1)(\s*)\n`),
+					},
+				},
+				{
+					name: "CustomHomePreservesRootShell",
+					argv: []string{"--home", "/tmp", c.env.ImagePath, "cat", "/etc/passwd"},
+					exit: 0,
+					wantOutputs: []e2e.SingularityCmdResultOp{
+						e2e.ExpectOutput(e2e.RegexMatch, `^root:x:0:0:root:[^:]*:/bin/ash\n`),
+					},
+				},
+			}
+
+			for _, tt := range tests {
+				c.env.RunSingularity(
+					t,
+					e2e.AsSubtest(tt.name),
+					e2e.WithProfile(e2e.UserProfile),
+					e2e.WithCommand("exec"),
+					e2e.WithDir("/tmp"),
+					e2e.WithArgs(tt.argv...),
+					e2e.ExpectExit(tt.exit, tt.wantOutputs...),
+				)
+			}
+		})
+	}
+}
+
 // Shell interaction tests
 func (c actionTests) actionShell(t *testing.T) {
 	e2e.EnsureImage(t, c.env)
@@ -456,6 +516,13 @@ func (c actionTests) STDPipe(t *testing.T) {
 			output:  "RUNNING FOO",
 			exit:    0,
 		},
+		{
+			name:    "CwdPath",
+			command: "exec",
+			argv:    []string{"--cwd", "/etc", c.env.ImagePath, "pwd"},
+			output:  "/etc",
+			exit:    0,
+		},
 		{
 			name:    "PwdPath",
 			command: "exec",
@@ -718,7 +785,11 @@ func (c actionTests) PersistentOverlay(t *testing.T) {
 		t.Fatalf("Unexpected error while running command.\n%s", res)
 	}
 
-	cmd = exec.Command("mkfs.ext3", "-q", "-F", ext3Img)
+	mkfsExt3Cmd, err := bin.FindBin("mkfs.ext3")
+	if err != nil {
+		t.Fatalf("Unable to find 'mkfs.ext3' binary even though require.Command() was called: %v", err)
+	}
+	cmd = exec.Command(mkfsExt3Cmd, "-q", "-F", ext3Img)
 	if res := cmd.Run(t); res.Error != nil {
 		t.Fatalf("Unexpected error while running command.\n%s", res)
 	}
@@ -1107,33 +1178,7 @@ func (c actionTests) actionBinds(t *testing.T) {
 	hostWorkDir := filepath.Join(workspace, "workdir")
 
 	createWorkspaceDirs := func(t *testing.T) {
-		e2e.Privileged(func(t *testing.T) {
-			if err := os.RemoveAll(hostCanaryDir); err != nil && !os.IsNotExist(err) {
-				t.Fatalf("failed to delete canary_dir: %s", err)
-			}
-			if err := os.RemoveAll(hostHomeDir); err != nil && !os.IsNotExist(err) {
-				t.Fatalf("failed to delete workspace home: %s", err)
-			}
-			if err := os.RemoveAll(hostWorkDir); err != nil && !os.IsNotExist(err) {
-				t.Fatalf("failed to delete workspace work: %s", err)
-			}
-		})(t)
-
-		if err := fs.Mkdir(hostCanaryDir, 0o777); err != nil {
-			t.Fatalf("failed to create canary_dir: %s", err)
-		}
-		if err := fs.Touch(hostCanaryFile); err != nil {
-			t.Fatalf("failed to create canary_file: %s", err)
-		}
-		if err := os.Chmod(hostCanaryFile, 0o777); err != nil {
-			t.Fatalf("failed to apply permissions on canary_file: %s", err)
-		}
-		if err := fs.Mkdir(hostHomeDir, 0o777); err != nil {
-			t.Fatalf("failed to create workspace home directory: %s", err)
-		}
-		if err := fs.Mkdir(hostWorkDir, 0o777); err != nil {
-			t.Fatalf("failed to create workspace work directory: %s", err)
-		}
+		mkWorkspaceDirs(t, hostCanaryDir, hostHomeDir, hostWorkDir, hostCanaryFile)
 	}
 
 	// convert test image to sandbox
@@ -1199,6 +1244,16 @@ func (c actionTests) actionBinds(t *testing.T) {
 			},
 			exit: 0,
 		},
+		{
+			name: "SimpleFileCwd",
+			args: []string{
+				"--bind", canaryFileBind,
+				"--cwd", contCanaryDir,
+				sandbox,
+				"test", "-f", "file",
+			},
+			exit: 0,
+		},
 		{
 			name: "SimpleFilePwd",
 			args: []string{
@@ -1218,6 +1273,16 @@ func (c actionTests) actionBinds(t *testing.T) {
 			},
 			exit: 0,
 		},
+		{
+			name: "SimpleDirCwd",
+			args: []string{
+				"--bind", canaryDirBind,
+				"--cwd", contCanaryDir,
+				sandbox,
+				"test", "-f", "file",
+			},
+			exit: 0,
+		},
 		{
 			name: "SimpleDirPwd",
 			args: []string{
@@ -1832,6 +1897,11 @@ func (c actionTests) bindImage(t *testing.T) {
 		t.Fatal(err)
 	}
 
+	scratchDir := filepath.Join(testdir, "scratch")
+	if err := os.MkdirAll(filepath.Join(scratchDir, "bin"), 0o700); err != nil {
+		t.Fatal(err)
+	}
+
 	cleanup := func(t *testing.T) {
 		if t.Failed() {
 			t.Logf("Not removing directory %s for test %s", testdir, t.Name())
@@ -1875,7 +1945,11 @@ func (c actionTests) bindImage(t *testing.T) {
 		t.Fatalf("Unexpected error while running command.\n%s", res)
 	}
 
-	cmd = exec.Command("mkfs.ext3", "-q", "-F", ext3Img)
+	mkfsExt3Cmd, err := bin.FindBin("mkfs.ext3")
+	if err != nil {
+		t.Fatalf("Unable to find 'mkfs.ext3' binary even though require.Command() was called: %v", err)
+	}
+	cmd = exec.Command(mkfsExt3Cmd, "-q", "-F", ext3Img)
 	if res := cmd.Run(t); res.Error != nil {
 		t.Fatalf("Unexpected error while running command.\n%s", res)
 	}
@@ -2112,6 +2186,29 @@ func (c actionTests) bindImage(t *testing.T) {
 			},
 			exit: 0,
 		},
+		// check ordering between image and user bind
+		{
+			name:    "SquashfsBeforeScratch",
+			profile: e2e.UserProfile,
+			args: []string{
+				"--bind", sifSquashImage + ":/scratch/bin:image-src=/",
+				"--bind", scratchDir + ":/scratch",
+				c.env.ImagePath,
+				"test", "-f", filepath.Join("/scratch/bin", squashMarkerFile),
+			},
+			exit: 1,
+		},
+		{
+			name:    "ScratchBeforeSquashfs",
+			profile: e2e.UserProfile,
+			args: []string{
+				"--bind", scratchDir + ":/scratch",
+				"--bind", sifSquashImage + ":/scratch/bin:image-src=/",
+				c.env.ImagePath,
+				"test", "-f", filepath.Join("/scratch/bin", squashMarkerFile),
+			},
+			exit: 0,
+		},
 		// For the --mount variants we are really just verifying the CLI
 		// acceptance of one or more image bind mount strings. Translation from
 		// --mount strings to BindPath structs is checked in unit tests. The
@@ -2261,6 +2358,17 @@ func (c actionTests) actionNoMount(t *testing.T) {
 			testDefault:   true,
 			testContained: true,
 			exit:          0,
+			// run from / to avoid /home mount via CWD mount
+			cwd: "/",
+		},
+		{
+			// test by excluding both home and cwd without chdir
+			name:          "home,cwd",
+			noMount:       "home,cwd",
+			noMatch:       "on /home",
+			testDefault:   true,
+			testContained: true,
+			exit:          0,
 		},
 		{
 			// /srv is an LSB directory we should be able to rely on for our CWD test
@@ -2315,7 +2423,6 @@ func (c actionTests) actionNoMount(t *testing.T) {
 	}
 
 	for _, tt := range tests {
-
 		if tt.testDefault {
 			c.env.RunSingularity(
 				t,
@@ -2450,6 +2557,67 @@ func (c actionTests) actionNoSIFFUSE(t *testing.T) {
 	}
 }
 
+// actionTmpSandboxFlag tests the command-line option prohibiting unpacking of image
+// files into temporary sandbox dirs.
+func (c actionTests) actionTmpSandboxFlag(t *testing.T) {
+	e2e.EnsureImage(t, c.env)
+
+	profiles := []e2e.Profile{e2e.UserProfile, e2e.RootProfile, e2e.FakerootProfile, e2e.UserNamespaceProfile}
+
+	for _, p := range profiles {
+		c.env.RunSingularity(
+			t,
+			e2e.AsSubtest(p.String()),
+			e2e.WithProfile(p),
+			e2e.WithCommand("exec"),
+			e2e.WithArgs("--sif-fuse=false", "--no-tmp-sandbox", "-u", c.env.ImagePath, "/bin/true"),
+			e2e.ExpectExit(255),
+		)
+	}
+}
+
+// Make sure --workdir and --scratch work together nicely even when workdir is a
+// relative path. Test needs to be run in non-parallel mode, because it changes
+// the current working directory of the host.
+func (c actionTests) relWorkdirScratch(t *testing.T) {
+	e2e.EnsureImage(t, c.env)
+
+	testdir, cleanup := e2e.MakeTempDir(t, c.env.TestDir, "persistent-overlay-", "")
+	t.Cleanup(func() {
+		if !t.Failed() {
+			e2e.Privileged(cleanup)
+		}
+	})
+
+	const subdirName string = "mysubdir"
+	if err := os.Mkdir(filepath.Join(testdir, subdirName), 0o777); err != nil {
+		t.Fatalf("could not create subdirectory %q in %q: %s", subdirName, testdir, err)
+	}
+
+	// Change current working directory, with deferred undoing of change.
+	prevCwd, err := os.Getwd()
+	if err != nil {
+		t.Fatalf("could not get current working directory: %s", err)
+	}
+	defer os.Chdir(prevCwd)
+	if err = os.Chdir(testdir); err != nil {
+		t.Fatalf("could not change cwd to %q: %s", testdir, err)
+	}
+
+	profiles := []e2e.Profile{e2e.UserProfile, e2e.RootProfile, e2e.FakerootProfile, e2e.UserNamespaceProfile}
+
+	for _, p := range profiles {
+		c.env.RunSingularity(
+			t,
+			e2e.AsSubtest(p.String()),
+			e2e.WithProfile(p),
+			e2e.WithCommand("exec"),
+			e2e.WithArgs("--contain", "--workdir", "./"+subdirName, "--scratch", "/myscratch", c.env.ImagePath, "true"),
+			e2e.ExpectExit(0),
+		)
+	}
+}
+
 func countSquashfuseMounts(t *testing.T) int {
 	count := 0
 
@@ -2497,53 +2665,69 @@ func E2ETests(env e2e.TestEnv) testhelper.Tests {
 	np := testhelper.NoParallel
 
 	return testhelper.Tests{
-		"action URI":                c.RunFromURI,              // action_URI
-		"exec":                      c.actionExec,              // singularity exec
-		"persistent overlay":        c.PersistentOverlay,       // Persistent Overlay
-		"persistent overlay unpriv": c.PersistentOverlayUnpriv, // Persistent Overlay Unprivileged
-		"run":                       c.actionRun,               // singularity run
-		"shell":                     c.actionShell,             // shell interaction
-		"STDPIPE":                   c.STDPipe,                 // stdin/stdout pipe
-		"action basic profiles":     c.actionBasicProfiles,     // run basic action under different profiles
-		"issue 4488":                c.issue4488,               // https://github.com/sylabs/singularity/issues/4488
-		"issue 4587":                c.issue4587,               // https://github.com/sylabs/singularity/issues/4587
-		"issue 4755":                c.issue4755,               // https://github.com/sylabs/singularity/issues/4755
-		"issue 4768":                c.issue4768,               // https://github.com/sylabs/singularity/issues/4768
-		"issue 4797":                c.issue4797,               // https://github.com/sylabs/singularity/issues/4797
-		"issue 4823":                c.issue4823,               // https://github.com/sylabs/singularity/issues/4823
-		"issue 4836":                c.issue4836,               // https://github.com/sylabs/singularity/issues/4836
-		"issue 5211":                c.issue5211,               // https://github.com/sylabs/singularity/issues/5211
-		"issue 5228":                c.issue5228,               // https://github.com/sylabs/singularity/issues/5228
-		"issue 5271":                c.issue5271,               // https://github.com/sylabs/singularity/issues/5271
-		"issue 5307":                c.issue5307,               // https://github.com/sylabs/singularity/issues/5307
-		"issue 5399":                c.issue5399,               // https://github.com/sylabs/singularity/issues/5399
-		"issue 5455":                c.issue5455,               // https://github.com/sylabs/singularity/issues/5455
-		"issue 5465":                c.issue5465,               // https://github.com/sylabs/singularity/issues/5465
-		"issue 5599":                c.issue5599,               // https://github.com/sylabs/singularity/issues/5599
-		"issue 5631":                c.issue5631,               // https://github.com/sylabs/singularity/issues/5631
-		"issue 5690":                c.issue5690,               // https://github.com/sylabs/singularity/issues/5690
-		"network":                   c.actionNetwork,           // test basic networking
-		"binds":                     c.actionBinds,             // test various binds with --bind and --mount
-		"exit and signals":          c.exitSignals,             // test exit and signals propagation
-		"fuse mount":                c.fuseMount,               // test fusemount option
-		"bind image":                c.bindImage,               // test bind image with --bind and --mount
-		"no-mount":                  c.actionNoMount,           // test --no-mount
-		"no-setgroups":              c.actionNoSetgroups,       // test --no-setgroups
-		"compat":                    np(c.actionCompat),        // test --compat
-		"umask":                     np(c.actionUmask),         // test umask propagation
-		"invalidRemote":             np(c.invalidRemote),       // GHSA-5mv9-q7fq-9394
-		"SIFFUSE":                   np(c.actionSIFFUSE),       // test --sif-fuse
-		"NoSIFFUSE":                 np(c.actionNoSIFFUSE),     // test absence of squashfs and CleanupHost()
+		"action URI":                   c.RunFromURI,                     // action_URI
+		"exec":                         c.actionExec,                     // singularity exec
+		"exec under multiple profiles": c.actionExecMultiProfile,         // singularity exec
+		"persistent overlay":           c.PersistentOverlay,              // Persistent Overlay
+		"persistent overlay unpriv":    c.PersistentOverlayUnpriv,        // Persistent Overlay Unprivileged
+		"run":                          c.actionRun,                      // singularity run
+		"shell":                        c.actionShell,                    // shell interaction
+		"STDPIPE":                      c.STDPipe,                        // stdin/stdout pipe
+		"action basic profiles":        c.actionBasicProfiles,            // run basic action under different profiles
+		"issue 4488":                   c.issue4488,                      // https://github.com/sylabs/singularity/issues/4488
+		"issue 4587":                   c.issue4587,                      // https://github.com/sylabs/singularity/issues/4587
+		"issue 4755":                   c.issue4755,                      // https://github.com/sylabs/singularity/issues/4755
+		"issue 4768":                   c.issue4768,                      // https://github.com/sylabs/singularity/issues/4768
+		"issue 4797":                   c.issue4797,                      // https://github.com/sylabs/singularity/issues/4797
+		"issue 4823":                   c.issue4823,                      // https://github.com/sylabs/singularity/issues/4823
+		"issue 4836":                   c.issue4836,                      // https://github.com/sylabs/singularity/issues/4836
+		"issue 5211":                   c.issue5211,                      // https://github.com/sylabs/singularity/issues/5211
+		"issue 5228":                   c.issue5228,                      // https://github.com/sylabs/singularity/issues/5228
+		"issue 5271":                   c.issue5271,                      // https://github.com/sylabs/singularity/issues/5271
+		"issue 5307":                   c.issue5307,                      // https://github.com/sylabs/singularity/issues/5307
+		"issue 5399":                   c.issue5399,                      // https://github.com/sylabs/singularity/issues/5399
+		"issue 5455":                   c.issue5455,                      // https://github.com/sylabs/singularity/issues/5455
+		"issue 5465":                   c.issue5465,                      // https://github.com/sylabs/singularity/issues/5465
+		"issue 5599":                   c.issue5599,                      // https://github.com/sylabs/singularity/issues/5599
+		"issue 5631":                   c.issue5631,                      // https://github.com/sylabs/singularity/issues/5631
+		"issue 5690":                   c.issue5690,                      // https://github.com/sylabs/singularity/issues/5690
+		"issue 1950":                   c.issue1950,                      // https://github.com/sylabs/singularity/issues/1950
+		"network":                      c.actionNetwork,                  // test basic networking
+		"binds":                        c.actionBinds,                    // test various binds with --bind and --mount
+		"exit and signals":             c.exitSignals,                    // test exit and signals propagation
+		"fuse mount":                   c.fuseMount,                      // test fusemount option
+		"bind image":                   c.bindImage,                      // test bind image with --bind and --mount
+		"no-mount":                     c.actionNoMount,                  // test --no-mount
+		"no-setgroups":                 c.actionNoSetgroups,              // test --no-setgroups
+		"compat":                       np(c.actionCompat),               // test --compat
+		"umask":                        np(c.actionUmask),                // test umask propagation
+		"invalidRemote":                np(c.invalidRemote),              // GHSA-5mv9-q7fq-9394
+		"SIFFUSE":                      np(c.actionSIFFUSE),              // test --sif-fuse
+		"NoSIFFUSE":                    np(c.actionNoSIFFUSE),            // test absence of squashfs and CleanupHost()
+		"TmpSandboxFlag":               c.actionTmpSandboxFlag,           // test --no-tmp-sandbox flag
+		"relWorkdirScratch":            np(c.relWorkdirScratch),          // test relative --workdir with --scratch
+		"ociRelWorkdirScratch":         np(c.actionOciRelWorkdirScratch), // test relative --workdir with --scratch in OCI mode
 		//
 		// OCI Runtime Mode
 		//
-		"ociRun":     c.actionOciRun,        // singularity run --oci
-		"ociExec":    c.actionOciExec,       // singularity exec --oci
-		"ociShell":   c.actionOciShell,      // singularity shell --oci
-		"ociSTDPIPE": c.ociSTDPipe,          // stdin/stdout pipe --oci
-		"ociNetwork": c.actionOciNetwork,    // singularity exec --oci --net
-		"ociBinds":   c.actionOciBinds,      // singularity exec --oci --bind / --mount
-		"ociIDMaps":  c.actionOciIDMaps,     // check uid/gid mapping on host for --oci as user / --fakeroot
-		"ociCompat":  np(c.actionOciCompat), // --oci equivalence to native mode --compat
+		"ociRun":               c.actionOciRun,                 // singularity run --oci
+		"ociExec":              c.actionOciExec,                // singularity exec --oci
+		"ociShell":             c.actionOciShell,               // singularity shell --oci
+		"ociSTDPIPE":           c.ociSTDPipe,                   // stdin/stdout pipe --oci
+		"ociNetwork":           c.actionOciNetwork,             // singularity exec --oci --net
+		"ociBinds":             c.actionOciBinds,               // singularity exec --oci --bind / --mount
+		"ociNo-setgroups":      c.actionOciNoSetgroups,         // --no-setgroups in OCI mode
+		"ociCdi":               c.actionOciCdi,                 // singularity exec --oci --cdi
+		"ociIDMaps":            c.actionOciIDMaps,              // check uid/gid mapping on host for --oci as user / --fakeroot
+		"ociCompat":            np(c.actionOciCompat),          // --oci equivalence to native mode --compat
+		"ociNoCompat":          np(c.actionOciNoCompat),        // --oci equivalence to native mode defaults with --no-compat
+		"ociOverlay":           (c.actionOciOverlay),           // --overlay in OCI mode
+		"ociOverlayExtfsPerms": (c.actionOciOverlayExtfsPerms), // permissions in writable extfs overlays mounted with FUSE in OCI mode
+		"ociOverlayTeardown":   np(c.actionOciOverlayTeardown), // proper overlay unmounting in OCI mode
+		"ociBindImage":         c.actionOciBindImage,           // test binding of images in OCI mode
+		"ociNo-mount":          c.actionOciNoMount,             // --no-mount in OCI mode
+		"ociHomeCwdPasswd":     c.actionOciHomeCwdPasswd,       // $HOME is correct in /etc/passwd, and is default cwd
+		"ociAllowSetuid":       c.actionOciAllowSetuid,         // --allow-setuid / check for nosuid mount options
+		"ociExitSignals":       c.ociExitSignals,               // test exit and signals propagation
 	}
 }
