@@ -12,11 +12,12 @@ import (
 	"path/filepath"
 	"testing"
 
-	"github.com/sylabs/singularity/e2e/internal/e2e"
-	"github.com/sylabs/singularity/e2e/internal/testhelper"
-	"github.com/sylabs/singularity/internal/pkg/test/tool/require"
-	"github.com/sylabs/singularity/internal/pkg/util/fs"
-	"github.com/sylabs/singularity/internal/pkg/util/user"
+	"github.com/sylabs/singularity/v4/e2e/internal/e2e"
+	"github.com/sylabs/singularity/v4/e2e/internal/testhelper"
+	"github.com/sylabs/singularity/v4/internal/pkg/test/tool/require"
+	"github.com/sylabs/singularity/v4/internal/pkg/util/bin"
+	"github.com/sylabs/singularity/v4/internal/pkg/util/fs"
+	"github.com/sylabs/singularity/v4/internal/pkg/util/user"
 )
 
 type configTests struct {
@@ -76,7 +77,11 @@ func (c *configTests) prepImages(t *testing.T) (cleanup func(t *testing.T)) {
 			defer cleanup(t)
 			t.Fatalf("Error creating blank ext3 image: %v: %s", err, out)
 		}
-		cmd = exec.Command("mkfs.ext3", "-d", c.sandboxImage, c.ext3Image)
+		mkfsExt3Cmd, err := bin.FindBin("mkfs.ext3")
+		if err != nil {
+			t.Fatalf("Unable to find 'mkfs.ext3' binary even though require.Command() was called: %v", err)
+		}
+		cmd = exec.Command(mkfsExt3Cmd, "-d", c.sandboxImage, c.ext3Image)
 		if out, err := cmd.CombinedOutput(); err != nil {
 			defer cleanup(t)
 			t.Fatalf("Error creating populated ext3 image: %v: %s", err, out)
@@ -135,6 +140,15 @@ func (c configTests) configGlobal(t *testing.T) {
 			e2e.ExpectExit(0),
 		)
 	}
+	unsetDirective := func(t *testing.T, directive, value string) {
+		c.env.RunSingularity(
+			t,
+			e2e.WithProfile(e2e.RootProfile),
+			e2e.WithCommand("config global"),
+			e2e.WithArgs("--unset", directive, value),
+			e2e.ExpectExit(0),
+		)
+	}
 	resetDirective := func(t *testing.T, directive string) {
 		c.env.RunSingularity(
 			t,
@@ -159,6 +173,7 @@ func (c configTests) configGlobal(t *testing.T) {
 		cwd               string
 		directive         string
 		directiveValue    string
+		unset             bool // unset instead of reset
 		exit              int
 		resultOp          e2e.SingularityCmdResultOp
 	}{
@@ -353,6 +368,7 @@ func (c configTests) configGlobal(t *testing.T) {
 			profile:        e2e.UserProfile,
 			directive:      "bind path",
 			directiveValue: "/etc/passwd:/passwd",
+			unset:          true,
 			exit:           0,
 		},
 		{
@@ -372,6 +388,7 @@ func (c configTests) configGlobal(t *testing.T) {
 			exit:           0,
 		},
 		// overlay may or not be available, just test with no
+		//nolint:dupword
 		{
 			name:           "EnableOverlayNo",
 			argv:           []string{c.env.ImagePath, "grep", "\\- overlay overlay", "/proc/self/mountinfo"},
@@ -683,6 +700,33 @@ func (c configTests) configGlobal(t *testing.T) {
 			directiveValue: "no",
 			exit:           0,
 		},
+		{
+			name:           "OCIModeYes",
+			argv:           []string{c.sifImage, "true"},
+			profile:        e2e.UserProfile,
+			directive:      "oci mode",
+			directiveValue: "yes",
+			exit:           0,
+			resultOp:       e2e.ExpectError(e2e.ContainMatch, "Running a non-OCI SIF in OCI mode."),
+		},
+		{
+			name:           "OCIModeYes_NoOCI",
+			argv:           []string{"--no-oci", c.sifImage, "true"},
+			profile:        e2e.UserProfile,
+			directive:      "oci mode",
+			directiveValue: "yes",
+			exit:           0,
+			resultOp:       e2e.ExpectError(e2e.UnwantedContainMatch, "Running a non-OCI SIF in OCI mode."),
+		},
+		{
+			name:           "OCIModeNo",
+			argv:           []string{c.sifImage, "true"},
+			profile:        e2e.UserProfile,
+			directive:      "oci mode",
+			directiveValue: "no",
+			exit:           0,
+			resultOp:       e2e.ExpectError(e2e.UnwantedContainMatch, "Running a non-OCI SIF in OCI mode."),
+		},
 	}
 
 	for _, tt := range tests {
@@ -698,7 +742,11 @@ func (c configTests) configGlobal(t *testing.T) {
 				setDirective(t, tt.directive, tt.directiveValue)
 			}),
 			e2e.PostRun(func(t *testing.T) {
-				resetDirective(t, tt.directive)
+				if tt.unset {
+					unsetDirective(t, tt.directive, tt.directiveValue)
+				} else {
+					resetDirective(t, tt.directive)
+				}
 			}),
 			e2e.WithCommand("exec"),
 			e2e.WithArgs(tt.argv...),
