@@ -21,11 +21,14 @@ import (
 	"testing"
 
 	dockerclient "github.com/docker/docker/client"
+	v1 "github.com/google/go-containerregistry/pkg/v1"
 	"github.com/pkg/errors"
-	"github.com/sylabs/singularity/e2e/internal/e2e"
-	"github.com/sylabs/singularity/e2e/internal/testhelper"
-	"github.com/sylabs/singularity/internal/pkg/test/tool/require"
-	"github.com/sylabs/singularity/internal/pkg/util/fs"
+	ocisif "github.com/sylabs/oci-tools/pkg/sif"
+	"github.com/sylabs/sif/v2/pkg/sif"
+	"github.com/sylabs/singularity/v4/e2e/internal/e2e"
+	"github.com/sylabs/singularity/v4/e2e/internal/testhelper"
+	"github.com/sylabs/singularity/v4/internal/pkg/test/tool/require"
+	"github.com/sylabs/singularity/v4/internal/pkg/util/fs"
 	"golang.org/x/sys/unix"
 )
 
@@ -488,28 +491,28 @@ func (c ctx) testDockerRegistry(t *testing.T) {
 		dfd  e2e.DefFileDetails
 	}{
 		{
-			name: "BusyBox",
+			name: "Alpine",
 			exit: 0,
 			dfd: e2e.DefFileDetails{
 				Bootstrap: "docker",
-				From:      c.env.TestRegistry + "/my-busybox",
+				From:      c.env.TestRegistry + "/my-alpine",
 			},
 		},
 		{
-			name: "BusyBoxRegistry",
+			name: "AlpineRegistry",
 			exit: 0,
 			dfd: e2e.DefFileDetails{
 				Bootstrap: "docker",
-				From:      "my-busybox",
+				From:      "my-alpine",
 				Registry:  c.env.TestRegistry,
 			},
 		},
 		{
-			name: "BusyBoxNamespace",
+			name: "AlpineNamespace",
 			exit: 255,
 			dfd: e2e.DefFileDetails{
 				Bootstrap: "docker",
-				From:      "my-busybox",
+				From:      "my-alpine",
 				Registry:  c.env.TestRegistry,
 				Namespace: "not-a-namespace",
 			},
@@ -584,7 +587,6 @@ func (c ctx) testDockerLabels(t *testing.T) {
 	)
 }
 
-//nolint:dupl
 func (c ctx) testDockerCMD(t *testing.T) {
 	imageDir, cleanup := e2e.MakeTempDir(t, c.env.TestDir, "docker-", "")
 	t.Cleanup(func() {
@@ -933,6 +935,28 @@ func (c ctx) testDockerUSER(t *testing.T) {
 				e2e.ExpectOutput(e2e.ContainMatch, `uid=2000(testuser) gid=2000(testgroup)`),
 			},
 		},
+		// `--oci` modes: check that we correctly error on conflict with `--home`
+		{
+			name:       "WithHomeOCIUser",
+			cmd:        "run",
+			profile:    e2e.OCIUserProfile,
+			args:       []string{"--home", "/tmp", dockerURI},
+			expectExit: 255,
+		},
+		{
+			name:       "WithHomeOCIFakeroot",
+			cmd:        "run",
+			profile:    e2e.OCIFakerootProfile,
+			args:       []string{"--home", "/tmp", dockerURI},
+			expectExit: 255,
+		},
+		{
+			name:       "WithHomeOCIRoot",
+			cmd:        "run",
+			profile:    e2e.OCIRootProfile,
+			args:       []string{"--home", "/tmp", dockerURI},
+			expectExit: 255,
+		},
 		// `--oci` modes: check that we don't override container-user's home directory
 		{
 			name:    "OrigHomeOCIUser",
@@ -978,6 +1002,145 @@ func (c ctx) testDockerUSER(t *testing.T) {
 	}
 }
 
+// Test that we can pull for different --platforms
+func (c ctx) testDockerPlatform(t *testing.T) {
+	tmpPath, err := fs.MakeTmpDir(c.env.TestDir, "docker-platform-", 0o755)
+	if err != nil {
+		t.Fatalf("failed to create temporary directory: %v", err)
+	}
+	t.Cleanup(func() {
+		if !t.Failed() {
+			os.RemoveAll(tmpPath)
+		}
+	})
+	tmpSIF := filepath.Join(tmpPath, "test.sif")
+
+	tests := []struct {
+		name     string
+		platform string
+		uri      string
+		exit     int
+	}{
+		{
+			name:     "MultiArchArm64",
+			platform: "linux/arm64/v8",
+			uri:      "docker://alpine:latest",
+			exit:     0,
+		},
+		{
+			name:     "MultiArchPpc64le",
+			platform: "linux/ppc64le",
+			uri:      "docker://alpine:latest",
+			exit:     0,
+		},
+		{
+			name:     "MultiArchBadPlatform",
+			platform: "windows/m68k",
+			uri:      "docker://alpine:latest",
+			exit:     255,
+		},
+		{
+			name:     "SingleArchArm64",
+			platform: "linux/arm64/v8",
+			uri:      "docker://arm64v8/alpine:latest",
+			exit:     0,
+		},
+		{
+			name:     "SingleArchPpc64le",
+			platform: "linux/ppc64le",
+			uri:      "docker://ppc64le/alpine:latest",
+			exit:     0,
+		},
+		{
+			name:     "SingleArchBadPlatform",
+			platform: "windows/m68k",
+			uri:      "docker://ppc64le/alpine:latest",
+			exit:     255,
+		},
+	}
+
+	for _, p := range []e2e.Profile{e2e.UserProfile, e2e.OCIUserProfile} {
+		for _, tt := range tests {
+			c.env.RunSingularity(
+				t,
+				e2e.AsSubtest(p.String()+"/"+tt.name),
+				e2e.WithProfile(p),
+				e2e.WithCommand("pull"),
+				e2e.WithArgs("--force", "--platform", tt.platform, tmpSIF, tt.uri),
+				e2e.ExpectExit(tt.exit),
+				e2e.PostRun(func(t *testing.T) {
+					if t.Failed() || tt.exit != 0 {
+						return
+					}
+					if p.OCI() {
+						checkOCISIFPlatform(t, tmpSIF, tt.platform)
+					} else {
+						checkNativeSIFPlatform(t, tmpSIF, tt.platform)
+					}
+				}),
+			)
+		}
+	}
+}
+
+func checkOCISIFPlatform(t *testing.T, imgPath, platform string) {
+	wantPlatform, err := v1.ParsePlatform(platform)
+	if err != nil {
+		t.Errorf("while parsing platform %v", err)
+	}
+
+	fi, err := sif.LoadContainerFromPath(imgPath, sif.OptLoadWithFlag(os.O_RDONLY))
+	defer fi.UnloadContainer()
+	if err != nil {
+		t.Errorf("while loading SIF: %v", err)
+	}
+
+	ix, err := ocisif.ImageIndexFromFileImage(fi)
+	if err != nil {
+		t.Errorf("while obtaining image index: %v", err)
+	}
+	idxManifest, err := ix.IndexManifest()
+	if err != nil {
+		t.Errorf("while obtaining index manifest: %v", err)
+	}
+	if len(idxManifest.Manifests) != 1 {
+		t.Errorf("image has multiple manifests")
+	}
+	imageDigest := idxManifest.Manifests[0].Digest
+	img, err := ix.Image(imageDigest)
+	if err != nil {
+		t.Errorf("while initializing image: %v", err)
+	}
+	cfg, err := img.ConfigFile()
+	if err != nil {
+		t.Errorf("while fetching image config: %v", err)
+	}
+	if !cfg.Platform().Equals(*wantPlatform) {
+		t.Errorf("wrong platform - wanted %q, got %q", wantPlatform.String(), cfg.Platform().String())
+	}
+}
+
+func checkNativeSIFPlatform(t *testing.T, imgPath, platform string) {
+	wantPlatform, err := v1.ParsePlatform(platform)
+	if err != nil {
+		t.Errorf("while parsing platform %v", err)
+	}
+
+	fi, err := sif.LoadContainerFromPath(imgPath, sif.OptLoadWithFlag(os.O_RDONLY))
+	defer fi.UnloadContainer()
+	if err != nil {
+		t.Errorf("while loading SIF: %v", err)
+	}
+	d, err := fi.GetDescriptor(sif.WithPartitionType(sif.PartPrimSys))
+	if err != nil {
+		t.Errorf("while getting primary partition: %v", err)
+	}
+	_, _, arch, _ := d.PartitionMetadata() //nolint:dogsled
+	if arch != wantPlatform.Architecture {
+		t.Errorf("wrong architecture - wanted %q, got %q", wantPlatform.Architecture, arch)
+	}
+}
+
 // E2ETests is the main func to trigger the test suite
 func E2ETests(env e2e.TestEnv) testhelper.Tests {
 	c := ctx{
@@ -1000,10 +1163,13 @@ func E2ETests(env e2e.TestEnv) testhelper.Tests {
 			t.Run("cmdentrypoint", c.testDockerCMDENTRYPOINT)
 			t.Run("cmd quotes", c.testDockerCMDQuotes)
 			t.Run("user", c.testDockerUSER)
+			t.Run("platform", c.testDockerPlatform)
 			// Regressions
 			t.Run("issue 4524", c.issue4524)
 			t.Run("issue 1286", c.issue1286)
 			t.Run("issue 1528", c.issue1528)
+			t.Run("issue 1586", c.issue1586)
+			t.Run("issue 1670", c.issue1670)
 		},
 		// Tests that are especially slow, or run against a local docker
 		// registry, can be run in parallel, with `--disable-cache` used within
