@@ -1,4 +1,4 @@
-// Copyright (c) 2019-2022, Sylabs Inc. All rights reserved.
+// Copyright (c) 2019-2023, Sylabs Inc. All rights reserved.
 // Copyright (c) Contributors to the Apptainer project, established as
 //   Apptainer a Series of LF Projects LLC.
 // This software is licensed under a 3-clause BSD license. Please consult the
@@ -8,6 +8,7 @@
 package imgbuild
 
 import (
+	"errors"
 	"fmt"
 	"log"
 	"os"
@@ -16,11 +17,11 @@ import (
 	"strings"
 	"testing"
 
-	"github.com/sylabs/singularity/e2e/ecl"
-	"github.com/sylabs/singularity/e2e/internal/e2e"
-	"github.com/sylabs/singularity/e2e/internal/testhelper"
-	"github.com/sylabs/singularity/internal/pkg/test/tool/require"
-	"github.com/sylabs/singularity/internal/pkg/util/fs"
+	"github.com/sylabs/singularity/v4/e2e/ecl"
+	"github.com/sylabs/singularity/v4/e2e/internal/e2e"
+	"github.com/sylabs/singularity/v4/e2e/internal/testhelper"
+	"github.com/sylabs/singularity/v4/internal/pkg/test/tool/require"
+	"github.com/sylabs/singularity/v4/internal/pkg/util/fs"
 )
 
 var testFileContent = "Test file content\n"
@@ -187,7 +188,6 @@ func (c imgBuildTests) buildFrom(t *testing.T) {
 
 		t.Run(profile.String(), func(t *testing.T) {
 			for _, tc := range tt {
-
 				dn, cleanup := c.tempDir(t, "build-from")
 				t.Cleanup(func() {
 					if !t.Failed() {
@@ -1550,7 +1550,7 @@ func (c imgBuildTests) buildBindMount(t *testing.T) {
 	}
 }
 
-// testWritableTmpfs checks that we can run the build using a writeable tmpfs in the %test step
+// testWritableTmpfs checks that we can run the build using a writable tmpfs in the %test step
 func (c imgBuildTests) testWritableTmpfs(t *testing.T) {
 	e2e.EnsureImage(t, c.env)
 
@@ -1742,6 +1742,205 @@ cat /proc/$$/cmdline`
 	)
 }
 
+func (c imgBuildTests) buildDefinitionWithBuildArgs(t *testing.T) {
+	busyboxSIF := e2e.BusyboxSIF(t)
+	fileContent := `HOME=/root
+DEMO=demo=with===equals==signs
+`
+	argfile, err := e2e.WriteTempFile(c.env.TestDir, "argfile-", fileContent)
+	if err != nil {
+		log.Fatal(err)
+	}
+	t.Cleanup(func() {
+		if !t.Failed() {
+			os.Remove(argfile)
+		}
+	})
+
+	tests := []struct {
+		name         string
+		buildArgs    []string
+		buildArgFile string
+		verify       []string
+		deffile      string
+		exit         int
+		err          string
+		output       string
+	}{
+		{
+			name: "ok case single stage build",
+			buildArgs: []string{
+				fmt.Sprintf("IMAGE=%s", busyboxSIF),
+				fmt.Sprintf("SCRIPT_PATH=%s", filepath.Join("..", "test", "build-args", "script.sh")),
+			},
+			verify: []string{
+				fmt.Sprintf("From: %s", busyboxSIF),
+				"export AUTHOR=jason",
+				"export VERSION=1",
+				"Author jason",
+				"Version 1",
+				"../test/build-args/script.sh",
+			},
+			deffile: filepath.Join("..", "test", "build-args", "single-stage.def"),
+			exit:    0,
+			err:     "",
+		},
+		{
+			name: "ko case single stage build",
+			buildArgs: []string{
+				fmt.Sprintf("SCRIPT_PATH=%s", filepath.Join("..", "test", "build-args", "script.sh")),
+			},
+			verify:  []string{},
+			deffile: filepath.Join("..", "test", "build-args", "single-stage.def"),
+			exit:    255,
+			err:     "IMAGE is not defined",
+		},
+		{
+			name: "ok case multiple stage build",
+			buildArgs: []string{
+				"DEVEL_IMAGE=golang:1.12.3-alpine3.9",
+				"FINAL_IMAGE=alpine:3.9",
+				"HOME=/root",
+			},
+			verify: []string{
+				"From: golang:1.12.3-alpine3.9",
+				"From: alpine:3.9",
+				"cd /root",
+			},
+			deffile: filepath.Join("..", "test", "build-args", "multiple-stage.def"),
+			exit:    0,
+			err:     "",
+		},
+		{
+			name: "ok case multiple stage build with arg file",
+			buildArgs: []string{
+				"DEVEL_IMAGE=golang:1.12.3-alpine3.9",
+				"FINAL_IMAGE=alpine:3.9",
+			},
+			buildArgFile: argfile,
+			verify: []string{
+				"From: golang:1.12.3-alpine3.9",
+				"From: alpine:3.9",
+				"cd /root",
+			},
+			deffile: filepath.Join("..", "test", "build-args", "multiple-stage.def"),
+			exit:    0,
+			err:     "",
+		},
+		{
+			name: "ko case multiple stage build",
+			buildArgs: []string{
+				"DEVEL_IMAGE=golang:1.12.3-alpine3.9",
+				"FINAL_IMAGE=alpine:3.9",
+			},
+			verify:  []string{},
+			deffile: filepath.Join("..", "test", "build-args", "multiple-stage.def"),
+			exit:    255,
+			err:     "HOME is not defined",
+		},
+		{
+			name: "equal signs in vals",
+			buildArgs: []string{
+				fmt.Sprintf("IMAGE=%s", busyboxSIF),
+				fmt.Sprintf("SCRIPT_PATH=%s", filepath.Join("..", "test", "build-args", "script.sh")),
+				"AUTHOR=Equals=In=My=Name",
+				"DEMO=demo=with===equals==signs",
+			},
+			verify: []string{
+				"Author Equals=In=My=Name",
+				"This is a demo=with===equals==signs for templating definition file",
+			},
+			deffile: filepath.Join("..", "test", "build-args", "single-stage.def"),
+			exit:    0,
+			err:     "",
+		},
+		{
+			name: "equal signs in argfile vals",
+			buildArgs: []string{
+				fmt.Sprintf("IMAGE=%s", busyboxSIF),
+				fmt.Sprintf("SCRIPT_PATH=%s", filepath.Join("..", "test", "build-args", "script.sh")),
+				"AUTHOR=Equals=In=My=Name",
+			},
+			buildArgFile: argfile,
+			verify: []string{
+				"Author Equals=In=My=Name",
+				"This is a demo=with===equals==signs for templating definition file",
+			},
+			deffile: filepath.Join("..", "test", "build-args", "single-stage.def"),
+			exit:    0,
+			err:     "",
+		},
+	}
+
+	t.Run("build definition", func(t *testing.T) {
+		for _, tt := range tests {
+			dn, cleanup := c.tempDir(t, "build-definition-template")
+			t.Cleanup(func() {
+				if !t.Failed() {
+					cleanup()
+				}
+			})
+
+			imagePath := path.Join(dn, "container")
+			args := []string{}
+
+			if tt.buildArgs != nil {
+				for _, arg := range tt.buildArgs {
+					args = append(args, "--build-arg")
+					args = append(args, arg)
+				}
+			}
+
+			if tt.buildArgFile != "" {
+				args = append(args, "--build-arg-file")
+				args = append(args, tt.buildArgFile)
+			}
+
+			args = append(args, imagePath)
+			args = append(args, tt.deffile)
+
+			expects := []e2e.SingularityCmdResultOp{}
+			if tt.verify != nil {
+				for _, v := range tt.verify {
+					expects = append(expects, e2e.ExpectOutput(e2e.ContainMatch, v))
+				}
+			}
+			expects = append(expects, e2e.ExpectOutput(e2e.UnwantedExactMatch, "{{"))
+			expects = append(expects, e2e.ExpectOutput(e2e.UnwantedExactMatch, "}}"))
+			c.env.RunSingularity(
+				t,
+				e2e.AsSubtest(tt.name),
+				e2e.WithProfile(e2e.RootProfile),
+				e2e.WithCommand("build"),
+				e2e.WithArgs(args...),
+				e2e.PostRun(func(t *testing.T) {
+					if t.Failed() {
+						return
+					}
+
+					if _, err := os.Stat(imagePath); errors.Is(err, os.ErrNotExist) {
+						return
+					}
+
+					c.env.RunSingularity(
+						t,
+						e2e.AsSubtest(tt.name+" verification"),
+						e2e.WithProfile(e2e.RootProfile),
+						e2e.WithCommand("sif"),
+						e2e.WithArgs("dump", "1", imagePath),
+						e2e.ExpectExit(0,
+							expects...,
+						),
+					)
+				}),
+				e2e.ExpectExit(tt.exit,
+					e2e.ExpectError(e2e.ContainMatch, tt.err),
+				),
+			)
+		}
+	})
+}
+
 // actionNoSetgoups checks that supplementary groups are visible, mapped to
 // nobody, in the container with --fakeroot --no-setgroups.
 func (c imgBuildTests) buildNoSetgroups(t *testing.T) {
@@ -1788,36 +1987,38 @@ func E2ETests(env e2e.TestEnv) testhelper.Tests {
 	}
 
 	return testhelper.Tests{
-		"bad path":                        c.badPath,                   // try to build from a non existent path
-		"build encrypt with PEM file":     c.buildEncryptPemFile,       // build encrypted images with certificate
-		"build encrypted with passphrase": c.buildEncryptPassphrase,    // build encrypted images with passphrase
-		"definition":                      c.buildDefinition,           // builds from definition template
-		"from local image":                c.buildLocalImage,           // build and image from an existing image
-		"from":                            c.buildFrom,                 // builds from definition file and URI
-		"multistage":                      c.buildMultiStageDefinition, // multistage build from definition templates
-		"non-root build":                  c.nonRootBuild,              // build sifs from non-root
-		"build and update sandbox":        c.buildUpdateSandbox,        // build/update sandbox
-		"fingerprint check":               c.buildWithFingerprint,      // definition file includes fingerprint check
-		"build with bind mount":           c.buildBindMount,            // build image with bind mount
-		"test with writable tmpfs":        c.testWritableTmpfs,         // build image, using writable tmpfs in the test step
-		"library host":                    c.buildLibraryHost,          // build image with hostname in library URI
-		"proot":                           c.buildProot,                // build image as an unpriv user with proot
-		"customShebang":                   c.buildCustomShebang,        // build image with custom #! in %test and %runscript
-		"no-setgroups":                    c.buildNoSetgroups,          // build with --fakeroot --no-setgroups
-		"issue 3848":                      c.issue3848,                 // https://github.com/hpcng/singularity/issues/3848
-		"issue 4203":                      c.issue4203,                 // https://github.com/sylabs/singularity/issues/4203
-		"issue 4407":                      c.issue4407,                 // https://github.com/sylabs/singularity/issues/4407
-		"issue 4583":                      c.issue4583,                 // https://github.com/sylabs/singularity/issues/4583
-		"issue 4820":                      c.issue4820,                 // https://github.com/sylabs/singularity/issues/4820
-		"issue 4837":                      c.issue4837,                 // https://github.com/sylabs/singularity/issues/4837
-		"issue 4967":                      c.issue4967,                 // https://github.com/sylabs/singularity/issues/4967
-		"issue 4969":                      c.issue4969,                 // https://github.com/sylabs/singularity/issues/4969
-		"issue 5166":                      c.issue5166,                 // https://github.com/sylabs/singularity/issues/5166
-		"issue 5250":                      c.issue5250,                 // https://github.com/sylabs/singularity/issues/5250
-		"issue 5315":                      c.issue5315,                 // https://github.com/sylabs/singularity/issues/5315
-		"issue 5435":                      c.issue5435,                 // https://github.com/hpcng/singularity/issues/5435
-		"issue 5668":                      c.issue5668,                 // https://github.com/hpcng/singularity/issues/5435
-		"issue 5690":                      c.issue5690,                 // https://github.com/hpcng/singularity/issues/5690
-		"issue 1273":                      c.issue1273,                 // https://github.com/sylabs/singularity/issues/1273
+		"bad path":                        c.badPath,                      // try to build from a non existent path
+		"build encrypt with PEM file":     c.buildEncryptPemFile,          // build encrypted images with certificate
+		"build encrypted with passphrase": c.buildEncryptPassphrase,       // build encrypted images with passphrase
+		"definition":                      c.buildDefinition,              // builds from definition template
+		"from local image":                c.buildLocalImage,              // build and image from an existing image
+		"from":                            c.buildFrom,                    // builds from definition file and URI
+		"multistage":                      c.buildMultiStageDefinition,    // multistage build from definition templates
+		"non-root build":                  c.nonRootBuild,                 // build sifs from non-root
+		"build and update sandbox":        c.buildUpdateSandbox,           // build/update sandbox
+		"fingerprint check":               c.buildWithFingerprint,         // definition file includes fingerprint check
+		"build with bind mount":           c.buildBindMount,               // build image with bind mount
+		"test with writable tmpfs":        c.testWritableTmpfs,            // build image, using writable tmpfs in the test step
+		"library host":                    c.buildLibraryHost,             // build image with hostname in library URI
+		"proot":                           c.buildProot,                   // build image as an unpriv user with proot
+		"customShebang":                   c.buildCustomShebang,           // build image with custom #! in %test and %runscript
+		"no-setgroups":                    c.buildNoSetgroups,             // build with --fakeroot --no-setgroups
+		"buildArgs":                       c.buildDefinitionWithBuildArgs, // builds from definition with build args (build arg file) support
+		"issue 3848":                      c.issue3848,                    // https://github.com/hpcng/singularity/issues/3848
+		"issue 4203":                      c.issue4203,                    // https://github.com/sylabs/singularity/issues/4203
+		"issue 4407":                      c.issue4407,                    // https://github.com/sylabs/singularity/issues/4407
+		"issue 4583":                      c.issue4583,                    // https://github.com/sylabs/singularity/issues/4583
+		"issue 4820":                      c.issue4820,                    // https://github.com/sylabs/singularity/issues/4820
+		"issue 4837":                      c.issue4837,                    // https://github.com/sylabs/singularity/issues/4837
+		"issue 4967":                      c.issue4967,                    // https://github.com/sylabs/singularity/issues/4967
+		"issue 4969":                      c.issue4969,                    // https://github.com/sylabs/singularity/issues/4969
+		"issue 5166":                      c.issue5166,                    // https://github.com/sylabs/singularity/issues/5166
+		"issue 5250":                      c.issue5250,                    // https://github.com/sylabs/singularity/issues/5250
+		"issue 5315":                      c.issue5315,                    // https://github.com/sylabs/singularity/issues/5315
+		"issue 5435":                      c.issue5435,                    // https://github.com/hpcng/singularity/issues/5435
+		"issue 5668":                      c.issue5668,                    // https://github.com/hpcng/singularity/issues/5435
+		"issue 5690":                      c.issue5690,                    // https://github.com/hpcng/singularity/issues/5690
+		"issue 1273":                      c.issue1273,                    // https://github.com/sylabs/singularity/issues/1273
+		"issue 1812":                      c.issue1812,                    // https://github.com/sylabs/singularity/issues/1812
 	}
 }
