@@ -22,40 +22,42 @@ import (
 	"testing"
 
 	// Tests imports
-	"github.com/sylabs/singularity/e2e/actions"
-	e2ebuildcfg "github.com/sylabs/singularity/e2e/buildcfg"
-	"github.com/sylabs/singularity/e2e/cache"
-	"github.com/sylabs/singularity/e2e/cgroups"
-	"github.com/sylabs/singularity/e2e/cmdenvvars"
-	"github.com/sylabs/singularity/e2e/config"
-	"github.com/sylabs/singularity/e2e/delete"
-	"github.com/sylabs/singularity/e2e/docker"
-	"github.com/sylabs/singularity/e2e/ecl"
-	singularityenv "github.com/sylabs/singularity/e2e/env"
-	"github.com/sylabs/singularity/e2e/gpu"
-	"github.com/sylabs/singularity/e2e/help"
-	"github.com/sylabs/singularity/e2e/imgbuild"
-	"github.com/sylabs/singularity/e2e/inspect"
-	"github.com/sylabs/singularity/e2e/instance"
-	"github.com/sylabs/singularity/e2e/key"
-	"github.com/sylabs/singularity/e2e/oci"
-	"github.com/sylabs/singularity/e2e/overlay"
-	"github.com/sylabs/singularity/e2e/plugin"
-	"github.com/sylabs/singularity/e2e/pull"
-	"github.com/sylabs/singularity/e2e/push"
-	"github.com/sylabs/singularity/e2e/remote"
-	"github.com/sylabs/singularity/e2e/run"
-	"github.com/sylabs/singularity/e2e/runhelp"
-	"github.com/sylabs/singularity/e2e/security"
-	"github.com/sylabs/singularity/e2e/sign"
-	"github.com/sylabs/singularity/e2e/verify"
-	"github.com/sylabs/singularity/e2e/version"
+	"github.com/sylabs/singularity/v4/e2e/actions"
+	e2ebuildcfg "github.com/sylabs/singularity/v4/e2e/buildcfg"
+	"github.com/sylabs/singularity/v4/e2e/cache"
+	"github.com/sylabs/singularity/v4/e2e/cgroups"
+	"github.com/sylabs/singularity/v4/e2e/cmdenvvars"
+	"github.com/sylabs/singularity/v4/e2e/config"
+	"github.com/sylabs/singularity/v4/e2e/delete"
+	"github.com/sylabs/singularity/v4/e2e/docker"
+	"github.com/sylabs/singularity/v4/e2e/ecl"
+	singularityenv "github.com/sylabs/singularity/v4/e2e/env"
+	"github.com/sylabs/singularity/v4/e2e/gpu"
+	"github.com/sylabs/singularity/v4/e2e/help"
+	"github.com/sylabs/singularity/v4/e2e/imgbuild"
+	"github.com/sylabs/singularity/v4/e2e/inspect"
+	"github.com/sylabs/singularity/v4/e2e/instance"
+	"github.com/sylabs/singularity/v4/e2e/key"
+	"github.com/sylabs/singularity/v4/e2e/keyserver"
+	"github.com/sylabs/singularity/v4/e2e/oci"
+	"github.com/sylabs/singularity/v4/e2e/overlay"
+	"github.com/sylabs/singularity/v4/e2e/plugin"
+	"github.com/sylabs/singularity/v4/e2e/pull"
+	"github.com/sylabs/singularity/v4/e2e/push"
+	"github.com/sylabs/singularity/v4/e2e/registry"
+	"github.com/sylabs/singularity/v4/e2e/remote"
+	"github.com/sylabs/singularity/v4/e2e/run"
+	"github.com/sylabs/singularity/v4/e2e/runhelp"
+	"github.com/sylabs/singularity/v4/e2e/security"
+	"github.com/sylabs/singularity/v4/e2e/sign"
+	"github.com/sylabs/singularity/v4/e2e/verify"
+	"github.com/sylabs/singularity/v4/e2e/version"
 
-	"github.com/sylabs/singularity/e2e/internal/e2e"
-	"github.com/sylabs/singularity/e2e/internal/testhelper"
-	"github.com/sylabs/singularity/internal/pkg/buildcfg"
-	"github.com/sylabs/singularity/pkg/util/slice"
-	useragent "github.com/sylabs/singularity/pkg/util/user-agent"
+	"github.com/sylabs/singularity/v4/e2e/internal/e2e"
+	"github.com/sylabs/singularity/v4/e2e/internal/testhelper"
+	"github.com/sylabs/singularity/v4/internal/pkg/buildcfg"
+	"github.com/sylabs/singularity/v4/pkg/util/slice"
+	useragent "github.com/sylabs/singularity/v4/pkg/util/user-agent"
 )
 
 var (
@@ -81,11 +83,13 @@ var e2eGroups = map[string]testhelper.Group{
 	"INSPECT":    inspect.E2ETests,
 	"INSTANCE":   instance.E2ETests,
 	"KEY":        key.E2ETests,
+	"KEYSERVER":  keyserver.E2ETests,
 	"OCI":        oci.E2ETests,
 	"OVERLAY":    overlay.E2ETests,
 	"PLUGIN":     plugin.E2ETests,
 	"PULL":       pull.E2ETests,
 	"PUSH":       push.E2ETests,
+	"REGISTRY":   registry.E2ETests,
 	"REMOTE":     remote.E2ETests,
 	"RUN":        run.E2ETests,
 	"RUNHELP":    runhelp.E2ETests,
@@ -183,6 +187,7 @@ func Run(t *testing.T) {
 	}
 
 	for _, cf := range configFiles {
+		//nolint:forcetypeassert
 		if fi, err := os.Stat(cf); err != nil {
 			t.Fatalf("%s is not installed on this system: %v", cf, err)
 		} else if !fi.Mode().IsRegular() {
@@ -194,9 +199,9 @@ func Run(t *testing.T) {
 
 	// Provision local registry
 	testenv.TestRegistry = e2e.StartRegistry(t, testenv)
-	testenv.TestRegistryImage = fmt.Sprintf("docker://%s/my-busybox:latest", testenv.TestRegistry)
+	testenv.TestRegistryImage = fmt.Sprintf("docker://%s/my-alpine:latest", testenv.TestRegistry)
 
-	// Copy small test image (busybox:latest) into local registry from DockerHub
+	// Copy small test image (alpine:latest) into local registry from DockerHub
 	insecureSource := false
 	insecureValue := os.Getenv("E2E_DOCKER_MIRROR_INSECURE")
 	if insecureValue != "" {
@@ -205,7 +210,7 @@ func Run(t *testing.T) {
 			t.Fatalf("could not convert E2E_DOCKER_MIRROR_INSECURE=%s: %s", insecureValue, err)
 		}
 	}
-	e2e.CopyOCIImage(t, "docker://busybox:latest", testenv.TestRegistryImage, insecureSource, true)
+	e2e.CopyOCIImage(t, "docker://alpine:latest", testenv.TestRegistryImage, insecureSource, true)
 
 	// SIF base test path, built on demand by e2e.EnsureImage
 	imagePath := path.Join(name, "test.sif")
@@ -217,7 +222,12 @@ func Run(t *testing.T) {
 	t.Log("Path to test OCI archive:", ociArchivePath)
 	testenv.OCIArchivePath = ociArchivePath
 
-	// Docker Archive test image path, built on demand by e2e.EnsureDockerArhive
+	// OCI-SIF Image, retrieved on demand by e2e.EnsureOCISIF
+	ociSifPath := path.Join(name, "oci-sif.sif")
+	t.Log("Path to test OCI-SIF image:", ociArchivePath)
+	testenv.OCISIFPath = ociSifPath
+
+	// Docker Archive test image path, built on demand by e2e.EnsureDockerArchive
 	dockerArchivePath := path.Join(name, "docker.tar")
 	t.Log("Path to test Docker archive:", dockerArchivePath)
 	testenv.DockerArchivePath = dockerArchivePath
@@ -225,6 +235,12 @@ func Run(t *testing.T) {
 	// Local registry ORAS SIF image, built on demand by e2e.EnsureORASImage
 	testenv.OrasTestImage = fmt.Sprintf("oras://%s/oras_test_sif:latest", testenv.TestRegistry)
 
+	// Local registry ORAS OCI-SIF image, built on demand by e2e.EnsureORASOCISIF
+	testenv.OrasTestOCISIF = fmt.Sprintf("oras://%s/oras_test_oci-sif:latest", testenv.TestRegistry)
+
+	// OCI-SIF image pushed as OCI image to local registry
+	testenv.TestRegistryOCISIF = fmt.Sprintf("docker://%s/registry_test_oci-sif:latest", testenv.TestRegistry)
+
 	t.Cleanup(func() {
 		if !t.Failed() {
 			os.Remove(imagePath)
