@@ -15,21 +15,21 @@ import (
 	"syscall"
 
 	specs "github.com/opencontainers/runtime-spec/specs-go"
-	"github.com/sylabs/singularity/internal/pkg/buildcfg"
-	fakerootutil "github.com/sylabs/singularity/internal/pkg/fakeroot"
-	"github.com/sylabs/singularity/internal/pkg/plugin"
-	"github.com/sylabs/singularity/internal/pkg/runtime/engine"
-	"github.com/sylabs/singularity/internal/pkg/runtime/engine/config/oci/generate"
-	"github.com/sylabs/singularity/internal/pkg/runtime/engine/config/starter"
-	fakerootConfig "github.com/sylabs/singularity/internal/pkg/runtime/engine/fakeroot/config"
-	"github.com/sylabs/singularity/internal/pkg/security/seccomp"
-	"github.com/sylabs/singularity/internal/pkg/util/fs"
-	fakerootcallback "github.com/sylabs/singularity/pkg/plugin/callback/runtime/fakeroot"
-	"github.com/sylabs/singularity/pkg/runtime/engine/config"
-	"github.com/sylabs/singularity/pkg/sylog"
-	"github.com/sylabs/singularity/pkg/util/capabilities"
-	"github.com/sylabs/singularity/pkg/util/fs/proc"
-	"github.com/sylabs/singularity/pkg/util/singularityconf"
+	"github.com/sylabs/singularity/v4/internal/pkg/buildcfg"
+	fakerootutil "github.com/sylabs/singularity/v4/internal/pkg/fakeroot"
+	"github.com/sylabs/singularity/v4/internal/pkg/plugin"
+	"github.com/sylabs/singularity/v4/internal/pkg/runtime/engine"
+	"github.com/sylabs/singularity/v4/internal/pkg/runtime/engine/config/oci/generate"
+	"github.com/sylabs/singularity/v4/internal/pkg/runtime/engine/config/starter"
+	fakerootConfig "github.com/sylabs/singularity/v4/internal/pkg/runtime/engine/fakeroot/config"
+	"github.com/sylabs/singularity/v4/internal/pkg/security/seccomp"
+	"github.com/sylabs/singularity/v4/internal/pkg/util/fs"
+	fakerootcallback "github.com/sylabs/singularity/v4/pkg/plugin/callback/runtime/fakeroot"
+	"github.com/sylabs/singularity/v4/pkg/runtime/engine/config"
+	"github.com/sylabs/singularity/v4/pkg/sylog"
+	"github.com/sylabs/singularity/v4/pkg/util/capabilities"
+	"github.com/sylabs/singularity/v4/pkg/util/fs/proc"
+	"github.com/sylabs/singularity/v4/pkg/util/singularityconf"
 )
 
 // EngineOperations is a Singularity fakeroot runtime engine that implements engine.Operations.
@@ -116,6 +116,7 @@ func (e *EngineOperations) PrepareConfig(starterConfig *starter.Config) error {
 	if len(callbacks) > 1 {
 		return fmt.Errorf("multiple plugins have registered hook callback for fakeroot")
 	} else if len(callbacks) == 1 {
+		//nolint:forcetypeassert
 		getIDRange = callbacks[0].(fakerootcallback.UserMapping)
 	}
 
@@ -250,7 +251,7 @@ func fakerootSeccompProfile() *specs.LinuxSeccomp {
 //
 // This will be executed as a fake root user in a new user
 // namespace (PrepareConfig will set both).
-func (e *EngineOperations) StartProcess(masterConn net.Conn) error {
+func (e *EngineOperations) StartProcess(_ net.Conn) error {
 	const (
 		mountInfo    = "/proc/self/mountinfo"
 		selinuxMount = "/sys/fs/selinux"
@@ -339,6 +340,7 @@ func (e *EngineOperations) MonitorContainer(pid int, signals chan os.Signal) (sy
 	}()
 
 	for {
+		//nolint:forcetypeassert
 		select {
 		case s := <-signals:
 			// Signal received
@@ -352,6 +354,7 @@ func (e *EngineOperations) MonitorContainer(pid int, signals chan os.Signal) (sy
 				// https://github.com/golang/go/issues/24543.
 				break
 			default:
+				//nolint:forcetypeassert
 				if err := syscall.Kill(pid, s.(syscall.Signal)); err != nil {
 					return status, fmt.Errorf("interrupted by signal %s", s.String())
 				}
@@ -373,12 +376,12 @@ func (e *EngineOperations) CleanupContainer(context.Context, error, syscall.Wait
 }
 
 // CleanupHost does nothing for the fakeroot engine.
-func (e *EngineOperations) CleanupHost(ctx context.Context) error {
+func (e *EngineOperations) CleanupHost(context.Context) error {
 	return nil
 }
 
 // PostStartProcess does nothing for the fakeroot engine.
-func (e *EngineOperations) PostStartProcess(ctx context.Context, pid int) error {
+func (e *EngineOperations) PostStartProcess(context.Context, int) error {
 	return nil
 }
 
