@@ -9,40 +9,40 @@ package singularity
 
 import (
 	"context"
+	"errors"
 	"fmt"
 	"io"
 	"os"
+	osuser "os/user"
 	"path/filepath"
+	"reflect"
 	"strconv"
 	"strings"
 	"syscall"
 
 	specs "github.com/opencontainers/runtime-spec/specs-go"
-	"github.com/sylabs/singularity/internal/pkg/buildcfg"
-	"github.com/sylabs/singularity/internal/pkg/cgroups"
-	"github.com/sylabs/singularity/internal/pkg/plugin"
-	"github.com/sylabs/singularity/internal/pkg/runtime/engine/singularity/rpc/client"
-	"github.com/sylabs/singularity/internal/pkg/util/fs"
-	"github.com/sylabs/singularity/internal/pkg/util/fs/files"
-	"github.com/sylabs/singularity/internal/pkg/util/fs/layout"
-	"github.com/sylabs/singularity/internal/pkg/util/fs/layout/layer/overlay"
-	"github.com/sylabs/singularity/internal/pkg/util/fs/layout/layer/underlay"
-	"github.com/sylabs/singularity/internal/pkg/util/fs/mount"
-	fsoverlay "github.com/sylabs/singularity/internal/pkg/util/fs/overlay"
-	"github.com/sylabs/singularity/internal/pkg/util/gpu"
-	"github.com/sylabs/singularity/internal/pkg/util/mainthread"
-	"github.com/sylabs/singularity/internal/pkg/util/priv"
-	"github.com/sylabs/singularity/internal/pkg/util/user"
-	"github.com/sylabs/singularity/pkg/image"
-	"github.com/sylabs/singularity/pkg/network"
-	singularitycallback "github.com/sylabs/singularity/pkg/plugin/callback/runtime/engine/singularity"
-	singularity "github.com/sylabs/singularity/pkg/runtime/engine/singularity/config"
-	"github.com/sylabs/singularity/pkg/sylog"
-	"github.com/sylabs/singularity/pkg/util/fs/proc"
-	"github.com/sylabs/singularity/pkg/util/loop"
-	"github.com/sylabs/singularity/pkg/util/namespaces"
-	"github.com/sylabs/singularity/pkg/util/singularityconf"
-	"github.com/sylabs/singularity/pkg/util/slice"
+	"github.com/sylabs/singularity/v4/internal/pkg/buildcfg"
+	"github.com/sylabs/singularity/v4/internal/pkg/cgroups"
+	"github.com/sylabs/singularity/v4/internal/pkg/runtime/engine/singularity/rpc/client"
+	"github.com/sylabs/singularity/v4/internal/pkg/util/fs"
+	"github.com/sylabs/singularity/v4/internal/pkg/util/fs/files"
+	"github.com/sylabs/singularity/v4/internal/pkg/util/fs/layout"
+	"github.com/sylabs/singularity/v4/internal/pkg/util/fs/layout/layer/overlay"
+	"github.com/sylabs/singularity/v4/internal/pkg/util/fs/layout/layer/underlay"
+	"github.com/sylabs/singularity/v4/internal/pkg/util/fs/mount"
+	fsoverlay "github.com/sylabs/singularity/v4/internal/pkg/util/fs/overlay"
+	"github.com/sylabs/singularity/v4/internal/pkg/util/gpu"
+	"github.com/sylabs/singularity/v4/internal/pkg/util/mainthread"
+	"github.com/sylabs/singularity/v4/internal/pkg/util/priv"
+	"github.com/sylabs/singularity/v4/internal/pkg/util/user"
+	"github.com/sylabs/singularity/v4/pkg/image"
+	"github.com/sylabs/singularity/v4/pkg/network"
+	singularity "github.com/sylabs/singularity/v4/pkg/runtime/engine/singularity/config"
+	"github.com/sylabs/singularity/v4/pkg/sylog"
+	"github.com/sylabs/singularity/v4/pkg/util/fs/proc"
+	"github.com/sylabs/singularity/v4/pkg/util/namespaces"
+	"github.com/sylabs/singularity/v4/pkg/util/singularityconf"
+	"github.com/sylabs/singularity/v4/pkg/util/slice"
 	"golang.org/x/sys/unix"
 	"golang.org/x/term"
 )
@@ -54,7 +54,6 @@ import (
 var (
 	cryptDev       string
 	networkSetup   *network.Setup
-	imageDriver    image.Driver
 	umountPoints   []string
 	cgroupsManager *cgroups.Manager
 )
@@ -86,8 +85,15 @@ type container struct {
 	skippedMount  []string
 	suidFlag      uintptr
 	devSourcePath string
+	imageBind     map[string]string
+	skipCwd       bool
 }
 
+const (
+	tmpPath    = "/tmp"
+	vartmpPath = "/var/tmp"
+)
+
 //nolint:maintidx
 func create(ctx context.Context, engine *EngineOperations, rpcOps *client.RPC, pid int) error {
 	var err error
@@ -125,6 +131,7 @@ func create(ctx context.Context, engine *EngineOperations, rpcOps *client.RPC, p
 		mountInfoPath: fmt.Sprintf("/proc/%d/mountinfo", pid),
 		skippedMount:  make([]string, 0),
 		suidFlag:      syscall.MS_NOSUID,
+		imageBind:     make(map[string]string),
 	}
 
 	cwd := engine.EngineConfig.GetCwd()
@@ -163,32 +170,18 @@ func create(ctx context.Context, engine *EngineOperations, rpcOps *client.RPC, p
 		c.userNS, _ = namespaces.IsInsideUserNamespace(os.Getpid())
 	}
 
-	// load image driver plugins
-	callbackType := (singularitycallback.RegisterImageDriver)(nil) // nolint:staticcheck
-	callbacks, err := plugin.LoadCallbacks(callbackType)
-	if err != nil {
-		return fmt.Errorf("while loading plugins callbacks '%T': %s", callbackType, err)
-	}
-	for _, callback := range callbacks {
-		if err := callback.(singularitycallback.RegisterImageDriver)(c.userNS); err != nil { // nolint:staticcheck
-			return fmt.Errorf("while registering image driver: %s", err)
-		}
-	}
-
-	driverName := c.engine.EngineConfig.File.ImageDriver // nolint:staticcheck
-	imageDriver = image.GetDriver(driverName)
-	if driverName != "" && imageDriver == nil {
-		return fmt.Errorf("%q: no such image driver", driverName)
-	}
-
 	p := &mount.Points{}
 	system := &mount.System{Points: p, Mount: c.mount}
 
-	if err := c.setupSessionLayout(system); err != nil {
+	createCwdDirTag := mount.AuthorizedTag(mount.LayerTag)
+	if c.engine.EngineConfig.GetSessionLayer() == singularity.UnderlayLayer {
+		createCwdDirTag = mount.PreLayerTag
+	}
+	if err := system.RunBeforeTag(createCwdDirTag, c.createCwdDir); err != nil {
 		return err
 	}
 
-	if err := c.setupImageDriver(system); err != nil {
+	if err := c.setupSessionLayout(system); err != nil {
 		return err
 	}
 
@@ -238,9 +231,24 @@ func create(ctx context.Context, engine *EngineOperations, rpcOps *client.RPC, p
 	if err := c.addUserbindsMount(system); err != nil {
 		return err
 	}
-	if err := c.addTmpMount(system); err != nil {
+
+	// The source of /tmp and /var/tmp may be a tmpfs (--contain) or within a
+	// --workdir. This interacts with session setup, so must be handled now.
+	tmpSource, vartmpSource, err := c.createTmpSource()
+	if err != nil {
 		return err
 	}
+	// The actual addition of the mount of /tmp & /var/tmp must be delayed until
+	// we can inspect /var/tmp in the rootfs to see if it is a symlink to /tmp.
+	if tmpSource != "" {
+		f := func(system *mount.System) error {
+			return c.addTmpMount(system, tmpSource, vartmpSource)
+		}
+		if err := system.RunBeforeTag(mount.TmpTag, f); err != nil {
+			return err
+		}
+	}
+
 	if err := c.addScratchMount(system); err != nil {
 		return err
 	}
@@ -417,126 +425,11 @@ func (c *container) mount(point *mount.Point, system *mount.System) error {
 	return nil
 }
 
-// setupImageDriver prepare the image driver configured in singularity.conf
-// to start it after the session setup.
-func (c *container) setupImageDriver(system *mount.System) error {
-	if imageDriver == nil {
-		return nil
-	}
-
-	const sessionPath = "/driver"
-
-	fuseDriver := imageDriver.Features()&image.FuseFeature != 0
-
-	if err := c.session.AddDir(sessionPath); err != nil {
-		return fmt.Errorf("while creating session driver directory: %s", err)
-	}
-	sp, _ := c.session.GetPath(sessionPath)
-
-	params := &image.DriverParams{
-		SessionPath: sp,
-		UsernsFd:    -1,
-		FuseFd:      -1,
-		Config:      c.engine.CommonConfig,
-	}
-
-	if c.userNS {
-		fds, err := c.getFuseFdFromRPC(nil)
-		if err != nil {
-			return fmt.Errorf("while getting /proc/self/ns/user file descriptor: %s", err)
-		}
-		params.UsernsFd = fds[0]
-	}
-
-	if fuseDriver {
-		fuseFd, fuseRPCFd, err := c.openFuseFdFromRPC()
-		if err != nil {
-			return fmt.Errorf("while requesting /dev/fuse file descriptor from RPC: %s", err)
-		}
-		params.FuseFd = fuseFd
-		system.RunAfterTag(mount.SessionTag, func(system *mount.System) error {
-			defer unix.Close(params.FuseFd)
-
-			fakeroot := c.engine.EngineConfig.GetFakeroot()
-			fakerootHybrid := fakeroot && os.Geteuid() != 0
-
-			uid := os.Getuid()
-			gid := os.Getgid()
-			rootmode := syscall.S_IFDIR & syscall.S_IFMT
-
-			// as fakeroot can change UID/GID, we allow others users
-			// to access FUSE mount point
-			allowOther := ""
-			if fakeroot {
-				allowOther = ",allow_other"
-			}
-			if fakerootHybrid {
-				uid = 0
-				gid = 0
-
-				// with hybrid workflow this process is actually running as the
-				// user but outside of the fakeroot user namespace, it means that
-				// the FUSE kernel code prevent us from accessing the mount point
-				// where images (rootfs, overlay ...) resides if we are not in the
-				// user namespace, so we redirect session VFS calls via RPC in order
-				// to be in the user namespace when dealing with filesystem related calls.
-				c.session.VFS = c.rpcOps
-
-				if params.UsernsFd != -1 {
-					defer unix.Close(params.UsernsFd)
-				}
-			} else {
-				if params.UsernsFd != -1 {
-					unix.Close(params.UsernsFd)
-					params.UsernsFd = -1
-				}
-			}
-
-			opts := fmt.Sprintf("fd=%d,rootmode=%o,user_id=%d,group_id=%d%s",
-				fuseRPCFd,
-				rootmode,
-				uid,
-				gid,
-				allowOther,
-			)
-
-			sylog.Debugf("Add FUSE mount for image driver with options %s", opts)
-			err := c.rpcOps.Mount("fuse", sp, "fuse", syscall.MS_NOSUID|syscall.MS_NODEV, opts)
-			if err != nil {
-				return fmt.Errorf("while mounting fuse image driver: %s", err)
-			}
-
-			umountPoints = append(umountPoints, sp)
-
-			sylog.Debugf("Starting image driver %s", c.engine.EngineConfig.File.ImageDriver) // nolint:staticcheck
-			if err := imageDriver.Start(params); err != nil {
-				return fmt.Errorf("failed to start driver: %s", err)
-			}
-
-			return nil
-		})
-		return nil
-	}
-
-	system.RunAfterTag(mount.SessionTag, func(system *mount.System) error {
-		if params.UsernsFd != -1 {
-			defer unix.Close(params.UsernsFd)
-		}
-		sylog.Debugf("Starting image driver %s", c.engine.EngineConfig.File.ImageDriver) // nolint:staticcheck
-		if err := imageDriver.Start(params); err != nil {
-			return fmt.Errorf("failed to start driver: %s", err)
-		}
-		return nil
-	})
-
-	return nil
-}
-
 // setPropagationMount will apply propagation flag set by
 // configuration directive, when applied master process
 // won't see mount done by RPC server anymore. Typically
 // called after SharedTag mounts
-func (c *container) setPropagationMount(system *mount.System) error {
+func (c *container) setPropagationMount(*mount.System) error {
 	pflags := uintptr(syscall.MS_REC)
 
 	if c.engine.EngineConfig.File.MountSlave {
@@ -554,7 +447,7 @@ func (c *container) setPropagationMount(system *mount.System) error {
 // point preventing this process from accessing /proc/<rpc_pid>/mountinfo
 // without error, so we bind mount /proc/self/mountinfo from RPC process
 // to a session file and read mount information from there.
-func (c *container) addMountInfo(system *mount.System) error {
+func (c *container) addMountInfo(*mount.System) error {
 	const (
 		mountinfo = "/mountinfo"
 		self      = "/proc/self/mountinfo"
@@ -572,7 +465,7 @@ func (c *container) addMountInfo(system *mount.System) error {
 	return nil
 }
 
-func (c *container) chdirFinal(system *mount.System) error {
+func (c *container) chdirFinal(*mount.System) error {
 	if _, err := c.rpcOps.Chdir(c.session.FinalPath()); err != nil {
 		return err
 	}
@@ -639,23 +532,6 @@ func (c *container) mountGeneric(mnt *mount.Point, tag mount.AuthorizedTag) (err
 		if tag == mount.RootfsTag && dest == c.session.RootFsPath() {
 			source = "."
 		}
-
-		// overlay requires root filesystem UID/GID since upper/work
-		// directories are owned by root
-		if tag == mount.LayerTag && mnt.Type == "overlay" {
-			if imageDriver != nil && c.engine.EngineConfig.File.EnableOverlay == "driver" {
-				if imageDriver.Features()&image.OverlayFeature != 0 {
-					params := &image.MountParams{
-						Source:     source,
-						Target:     dest,
-						Filesystem: mnt.Type,
-						Flags:      flags,
-						FSOptions:  opts,
-					}
-					return imageDriver.Mount(params, c.rpcOps.Mount)
-				}
-			}
-		}
 	}
 
 mount:
@@ -757,31 +633,17 @@ func (c *container) mountImage(mnt *mount.Point) error {
 		}
 	}
 
-	if imageDriver != nil && imageDriver.Features()&image.ImageFeature != 0 {
-		params := &image.MountParams{
-			Source:     mnt.Source,
-			Target:     mnt.Destination,
-			Filesystem: mountType,
-			Flags:      flags,
-			Offset:     offset,
-			Size:       sizelimit,
-			Key:        key,
-			FSOptions:  opts,
-		}
-		return imageDriver.Mount(params, c.rpcOps.Mount)
-	}
-
 	attachFlag := os.O_RDWR
-	loopFlags := uint32(loop.FlagsAutoClear)
+	loopFlags := uint32(unix.LO_FLAGS_AUTOCLEAR)
 
 	if flags&syscall.MS_RDONLY == 1 {
-		loopFlags |= loop.FlagsReadOnly
+		loopFlags |= unix.LO_FLAGS_READ_ONLY
 		attachFlag = os.O_RDONLY
 	}
 
-	info := &loop.Info64{
+	info := &unix.LoopInfo64{
 		Offset:    offset,
-		SizeLimit: sizelimit,
+		Sizelimit: sizelimit,
 		Flags:     loopFlags,
 	}
 
@@ -906,8 +768,11 @@ func (c *container) addRootfsMount(system *mount.System) error {
 	return nil
 }
 
-func (c *container) overlayUpperWork(system *mount.System) error {
-	ov := c.session.Layer.(*overlay.Overlay)
+func (c *container) overlayUpperWork(*mount.System) error {
+	ov, ok := c.session.Layer.(*overlay.Overlay)
+	if !ok {
+		return fmt.Errorf("expected overlay layer, but found layer of type %v", reflect.TypeOf(c.session.Layer))
+	}
 
 	createUpperWork := func(path, label string) error {
 		fi, err := c.rpcOps.Lstat(path)
@@ -937,7 +802,10 @@ func (c *container) overlayUpperWork(system *mount.System) error {
 
 func (c *container) addOverlayMount(system *mount.System) error {
 	nb := 0
-	ov := c.session.Layer.(*overlay.Overlay)
+	ov, ok := c.session.Layer.(*overlay.Overlay)
+	if !ok {
+		return fmt.Errorf("expected overlay layer, but found layer of type %v", reflect.TypeOf(c.session.Layer))
+	}
 	hasUpper := false
 
 	if c.engine.EngineConfig.GetWritableTmpfs() {
@@ -1025,14 +893,6 @@ func (c *container) addOverlayMount(system *mount.System) error {
 					allowed = true
 				}
 
-				// If a (deprecated) image driver plugin is enabled, then proceed with the mount.
-				// The image driver will be responsible for handling priv / unpriv concerns.
-				if c.engine.EngineConfig.File.EnableOverlay == "driver" {
-					if imageDriver != nil && imageDriver.Features()&image.OverlayFeature != 0 {
-						allowed = true
-					}
-				}
-
 				if !allowed {
 					return fmt.Errorf("only root user can use sandbox as overlay in setuid mode")
 				}
@@ -1143,22 +1003,14 @@ func (c *container) addImageBindMount(system *mount.System) error {
 				if err != nil {
 					return fmt.Errorf("while getting partitions for %s: %s", img.Path, err)
 				}
-				for _, part := range partitions {
-					if part.ID == partID {
-						data = &part
-						break
-					}
-				}
+				data = getPartitionByID(partitions, partID)
 			} else {
 				// take the first data partition found
 				partitions, err := img.GetDataPartitions()
 				if err != nil {
 					return fmt.Errorf("while getting data partition for %s: %s", img.Path, err)
 				}
-				for _, part := range partitions {
-					data = &part
-					break
-				}
+				data = getFirstPartition(partitions)
 			}
 
 			if data == nil {
@@ -1211,9 +1063,28 @@ func (c *container) addImageBindMount(system *mount.System) error {
 				return nil
 			})
 
-			if err := system.Points.AddBind(mount.UserbindsTag, src, destination, syscall.MS_BIND); err != nil {
-				return fmt.Errorf("while adding data bind %s -> %s: %s", src, destination, err)
-			}
+			// to honor bind ordering specified by the users
+			// we add bind mount later in addUserbindsMount
+			c.imageBind[destination] = src
+		}
+	}
+
+	return nil
+}
+
+func getFirstPartition(partitions []image.Section) *image.Section {
+	if len(partitions) > 0 {
+		return &partitions[0]
+	}
+
+	return nil
+}
+
+func getPartitionByID(partitions []image.Section, partID uint32) *image.Section {
+	var part image.Section
+	for _, part = range partitions {
+		if part.ID == partID {
+			return &part
 		}
 	}
 
@@ -1376,7 +1247,6 @@ func (c *container) addDevMount(system *mount.System) error {
 					return fmt.Errorf("problem resolving 'tty' group gid: %s", err)
 				}
 				options = fmt.Sprintf("%s,gid=%d", options, group.GID)
-
 			} else {
 				sylog.Debugf("Not setting /dev/pts filesystem gid: user namespace enabled")
 			}
@@ -1390,7 +1260,6 @@ func (c *container) addDevMount(system *mount.System) error {
 			if err := c.session.AddSymlink("/dev/ptmx", "/dev/pts/ptmx"); err != nil {
 				return fmt.Errorf("failed to create /dev/ptmx symlink: %s", err)
 			}
-
 		}
 		// add /dev/console mount pointing to original tty if there is one
 		for fd := 0; fd <= 2; fd++ {
@@ -1423,7 +1292,6 @@ func (c *container) addDevMount(system *mount.System) error {
 					consinfo.Rdev == fdinfo.Rdev {
 					sylog.Debugf("Fd %d is tty pointing to nonexistent %s but /dev/console is good", fd, ttylink)
 					ttylink = "/dev/console"
-
 				} else {
 					sylog.Debugf("Fd %d is tty but %s doesn't exist, skipping", fd, ttylink)
 					continue
@@ -1634,13 +1502,14 @@ func (c *container) getHomePaths() (source string, dest string, err error) {
 		dest = filepath.Clean(c.engine.EngineConfig.GetHomeDest())
 		source, err = filepath.Abs(filepath.Clean(c.engine.EngineConfig.GetHomeSource()))
 	} else {
-		pw, err := user.CurrentOriginal()
-		if err == nil {
-			source = pw.Dir
+		source = c.engine.EngineConfig.JSON.UserInfo.Home
+		if source != "" {
 			if c.engine.EngineConfig.GetFakeroot() {
+				// Mount user home directory onto /root for any root-mapped
+				// namespace
 				dest = "/root"
 			} else {
-				dest = pw.Dir
+				dest = source
 			}
 		}
 	}
@@ -1703,7 +1572,7 @@ func (c *container) addHomeLayer(system *mount.System, source, dest string) erro
 
 // addHomeNoLayer is responsible for staging the home directory and adding the base
 // directory of the staged home into the container when overlay/underlay are unavailable
-func (c *container) addHomeNoLayer(system *mount.System, source, dest string) error {
+func (c *container) addHomeNoLayer(system *mount.System, dest string) error {
 	flags := uintptr(syscall.MS_BIND | syscall.MS_REC)
 
 	homeBase := fs.RootDir(dest)
@@ -1757,7 +1626,7 @@ func (c *container) addHomeMount(system *mount.System) error {
 	sessionLayer := c.engine.EngineConfig.GetSessionLayer()
 	sylog.Debugf("Adding home directory mount [%v:%v] to list using layer: %s\n", stagingDir, dest, sessionLayer)
 	if !c.isLayerEnabled() {
-		return c.addHomeNoLayer(system, stagingDir, dest)
+		return c.addHomeNoLayer(system, dest)
 	}
 	return c.addHomeLayer(system, stagingDir, dest)
 }
@@ -1767,8 +1636,14 @@ func (c *container) addUserbindsMount(system *mount.System) error {
 	defaultFlags := uintptr(syscall.MS_BIND | c.suidFlag | syscall.MS_NODEV | syscall.MS_REC)
 
 	for _, b := range c.engine.EngineConfig.GetBindPath() {
-		// ignore image bind
+		// data image bind
 		if b.ID() != "" || b.ImageSrc() != "" {
+			source, ok := c.imageBind[b.Destination]
+			if ok {
+				if err := system.Points.AddBind(mount.UserbindsTag, source, b.Destination, syscall.MS_BIND); err != nil {
+					return fmt.Errorf("while adding data bind %s -> %s: %s", source, b.Destination, err)
+				}
+			}
 			continue
 		}
 
@@ -1840,60 +1715,58 @@ func (c *container) addUserbindsMount(system *mount.System) error {
 	return nil
 }
 
-func (c *container) addTmpMount(system *mount.System) error {
-	const (
-		tmpPath    = "/tmp"
-		varTmpPath = "/var/tmp"
-	)
-
+// createTmpSource will create directories that are used as the source of /tmp
+// and /var/tmp binds, if necessary. It returns the paths to be used as the
+// source of the mounts, or "" for both if they should be skipped.
+func (c *container) createTmpSource() (tmpSource, vartmpSource string, err error) {
 	sylog.Debugf("Checking for 'mount tmp' in configuration file")
 	if !c.engine.EngineConfig.File.MountTmp || c.engine.EngineConfig.GetNoTmp() {
 		sylog.Verbosef("Skipping tmp dir mounting (per config)")
-		return nil
+		return "", "", nil
 	}
 
-	tmpSource := tmpPath
-	vartmpSource := varTmpPath
+	tmpSource = tmpPath
+	vartmpSource = vartmpPath
 
 	if c.engine.EngineConfig.GetContain() {
 		workdir := c.engine.EngineConfig.GetWorkdir()
 		if workdir != "" {
 			if !c.engine.EngineConfig.File.UserBindControl {
 				sylog.Warningf("User bind control is disabled by system administrator")
-				return nil
+				return "", "", nil
 			}
 
 			vartmpSource = "var_tmp"
 
 			workdir, err := filepath.Abs(filepath.Clean(workdir))
 			if err != nil {
-				sylog.Warningf("Can't determine absolute path of workdir %s", workdir)
+				return "", "", fmt.Errorf("can't determine absolute path of workdir %s: %s", workdir, err)
 			}
 
 			tmpSource = filepath.Join(workdir, tmpSource)
 			vartmpSource = filepath.Join(workdir, vartmpSource)
 
 			if err := fs.Mkdir(tmpSource, os.ModeSticky|0o777); err != nil && !os.IsExist(err) {
-				return fmt.Errorf("failed to create %s: %s", tmpSource, err)
+				return "", "", fmt.Errorf("failed to create %s: %s", tmpSource, err)
 			}
 			if err := fs.Mkdir(vartmpSource, os.ModeSticky|0o777); err != nil && !os.IsExist(err) {
-				return fmt.Errorf("failed to create %s: %s", vartmpSource, err)
+				return "", "", fmt.Errorf("failed to create %s: %s", vartmpSource, err)
 			}
 		} else {
 			if _, err := c.session.GetPath(tmpSource); err != nil {
 				if err := c.session.AddDir(tmpSource); err != nil {
-					return err
+					return "", "", err
 				}
 				if err := c.session.Chmod(tmpSource, os.ModeSticky|0o777); err != nil {
-					return err
+					return "", "", err
 				}
 			}
 			if _, err := c.session.GetPath(vartmpSource); err != nil {
 				if err := c.session.AddDir(vartmpSource); err != nil {
-					return err
+					return "", "", err
 				}
 				if err := c.session.Chmod(vartmpSource, os.ModeSticky|0o777); err != nil {
-					return err
+					return "", "", err
 				}
 			}
 			tmpSource, _ = c.session.GetPath(tmpSource)
@@ -1902,22 +1775,39 @@ func (c *container) addTmpMount(system *mount.System) error {
 	}
 
 	c.session.OverrideDir(tmpPath, tmpSource)
-	c.session.OverrideDir(varTmpPath, vartmpSource)
+	c.session.OverrideDir(vartmpPath, vartmpSource)
+
+	return tmpSource, vartmpSource, nil
+}
+
+// addTmpMount adds bind mount definitions for /tmp and /var/tmp in the
+// container, from the provided sources. /var/tmp is not mounted if it is a
+// symlink to /tmp in the container rootfs.
+func (c *container) addTmpMount(system *mount.System, tmpSource, vartmpSource string) error {
+	tmpResolved := fs.EvalRelative(tmpPath, c.session.FinalPath())
+	varTmpResolved := fs.EvalRelative(vartmpPath, c.session.FinalPath())
+	sylog.Debugf("Container /tmp resolves to %q", tmpResolved)
+	sylog.Debugf("Container /var/tmp resolves to %q", varTmpResolved)
 
 	flags := uintptr(syscall.MS_BIND | c.suidFlag | syscall.MS_NODEV | syscall.MS_REC)
 
 	if err := system.Points.AddBind(mount.TmpTag, tmpSource, tmpPath, flags); err == nil {
 		system.Points.AddRemount(mount.TmpTag, tmpPath, flags)
-		sylog.Verbosef("Default mount: %s:%s", tmpPath, tmpPath)
+		sylog.Verbosef("Default mount: %s:%s", tmpSource, tmpPath)
 	} else {
 		return fmt.Errorf("could not mount container's %s directory: %s", tmpPath, err)
 	}
 
-	if err := system.Points.AddBind(mount.TmpTag, vartmpSource, varTmpPath, flags); err == nil {
-		system.Points.AddRemount(mount.TmpTag, varTmpPath, flags)
-		sylog.Verbosef("Default mount: %s:%s", varTmpPath, varTmpPath)
+	if varTmpResolved == tmpResolved {
+		sylog.Debugf("Container /tmp and /var/tmp resolve to same location. Skipping /var/tmp bind mount.")
+		return nil
+	}
+
+	if err := system.Points.AddBind(mount.TmpTag, vartmpSource, vartmpPath, flags); err == nil {
+		system.Points.AddRemount(mount.TmpTag, vartmpPath, flags)
+		sylog.Verbosef("Default mount: %s:%s", vartmpSource, vartmpPath)
 	} else {
-		return fmt.Errorf("could not mount container's %s directory: %s", varTmpPath, err)
+		return fmt.Errorf("could not mount container's %s directory: %s", vartmpPath, err)
 	}
 	return nil
 }
@@ -1939,9 +1829,12 @@ func (c *container) addScratchMount(system *mount.System) error {
 
 	workdir := c.engine.EngineConfig.GetWorkdir()
 	hasWorkdir := workdir != ""
-
+	var err error
 	if hasWorkdir {
-		workdir = filepath.Clean(workdir)
+		workdir, err = filepath.Abs(filepath.Clean(workdir))
+		if err != nil {
+			return fmt.Errorf("can't determine absolute path of workdir %s: %s", workdir, err)
+		}
 		sourceDir := filepath.Join(workdir, scratchSessionDir)
 		if err := fs.MkdirAll(sourceDir, 0o750); err != nil {
 			return fmt.Errorf("could not create scratch working directory %s: %s", sourceDir, err)
@@ -1995,70 +1888,164 @@ func (c *container) isMounted(dest string) bool {
 }
 
 func (c *container) addCwdMount(system *mount.System) error {
-	if c.engine.EngineConfig.GetContain() {
-		sylog.Verbosef("Not mounting current directory: contain was requested")
+	if c.skipCwd {
 		return nil
 	}
-	if !c.engine.EngineConfig.File.UserBindControl {
-		sylog.Warningf("Not mounting current directory: user bind control is disabled by system administrator")
-		return nil
-	}
-	if c.engine.EngineConfig.GetNoCwd() {
-		sylog.Debugf("Skipping current directory mount by user request.")
-		return nil
-	}
-	cwd := c.engine.EngineConfig.GetCwd()
-	if cwd == "" {
-		sylog.Warningf("No current working directory set: skipping mount")
-	}
 
-	current, err := filepath.EvalSymlinks(cwd)
+	cwdHost := filepath.Clean(c.engine.EngineConfig.GetCwd())
+	if cwdHost == "" {
+		sylog.Warningf("No current working directory set: skipping mount")
+		return nil
+	}
+
+	cwdHostResolved, err := filepath.EvalSymlinks(cwdHost)
 	if err != nil {
 		return fmt.Errorf("could not obtain current directory path: %s", err)
 	}
-	sylog.Debugf("Using %s as current working directory", cwd)
 
-	switch current {
-	case "/", "/etc", "/bin", "/mnt", "/usr", "/var", "/opt", "/sbin", "/lib", "/lib64":
-		sylog.Verbosef("Not mounting CWD within operating system directory: %s", current)
-		return nil
-	}
-	if strings.HasPrefix(current, "/sys") || strings.HasPrefix(current, "/proc") || strings.HasPrefix(current, "/dev") {
-		sylog.Verbosef("Not mounting CWD within virtual directory: %s", current)
-		return nil
-	}
+	cwdHostSymlink := cwdHost != cwdHostResolved
 
-	dest := fs.EvalRelative(cwd, c.session.FinalPath())
-	dest = filepath.Join(c.session.FinalPath(), dest)
+	cwdContainerResolved := fs.EvalRelative(cwdHost, c.session.FinalPath())
+	cwdContainerResolved = filepath.Join(c.session.FinalPath(), cwdContainerResolved)
+	cwdContainerSymlink := cwdContainerResolved != cwdHost
 
-	fi, err := c.rpcOps.Stat(dest)
+	fi, err := c.rpcOps.Stat(cwdContainerResolved)
 	if err != nil {
 		if os.IsNotExist(err) {
-			sylog.Verbosef("Not mounting CWD, %s doesn't exist within container", cwd)
+			sylog.Verbosef("Not mounting CWD, %s doesn't exist within container", cwdContainerResolved)
 		}
-		sylog.Verbosef("Not mounting CWD, while getting %s information: %s", cwd, err)
+		sylog.Verbosef("Not mounting CWD, while getting %s information: %s", cwdContainerResolved, err)
 		return nil
 	}
+	//nolint:forcetypeassert
 	cst := fi.Sys().(*syscall.Stat_t)
 
 	var hst syscall.Stat_t
-	if err := syscall.Stat(cwd, &hst); err != nil {
+	if err := syscall.Stat(cwdHost, &hst); err != nil {
 		return err
 	}
+
 	// same ino/dev, the current working directory is available within the container
 	if hst.Dev == cst.Dev && hst.Ino == cst.Ino {
-		sylog.Verbosef("%s found within container", cwd)
+		sylog.Verbosef("%s found within container", cwdHost)
 		return nil
-	} else if c.isMounted(dest) {
-		sylog.Verbosef("Not mounting CWD (already mounted in container): %s", cwd)
+	} else if c.isMounted(cwdContainerResolved) {
+		sylog.Verbosef("Not mounting CWD (already mounted in container): %s", cwdHost)
+		return nil
+	} else if cwdHostSymlink && cwdContainerSymlink && cwdContainerResolved != cwdHostResolved {
+		// symlink case when both destination exists on host and in container but are differents
+		sylog.Verbosef("Not mounting CWD, detected symlinks with different destination between host/container")
 		return nil
 	}
 
 	flags := uintptr(syscall.MS_BIND | c.suidFlag | syscall.MS_NODEV | syscall.MS_REC)
-	if err := system.Points.AddBind(mount.CwdTag, cwd, cwd, flags); err != nil {
-		return fmt.Errorf("could not bind cwd directory %s into container: %s", cwd, err)
+	if err := system.Points.AddBind(mount.CwdTag, cwdHost, cwdHost, flags); err != nil {
+		if errors.Is(err, mount.ErrMountExists) {
+			return nil
+		}
+		return fmt.Errorf("could not bind cwd directory %s into container: %s", cwdHost, err)
 	}
-	return system.Points.AddRemount(mount.CwdTag, cwd, flags)
+	return system.Points.AddRemount(mount.CwdTag, cwdHost, flags)
+}
+
+func (c *container) createCwdDir(system *mount.System) error {
+	if c.engine.EngineConfig.GetContain() {
+		c.skipCwd = true
+		sylog.Verbosef("Not mounting current directory: contain was requested")
+		return nil
+	}
+	if !c.engine.EngineConfig.File.UserBindControl {
+		c.skipCwd = true
+		sylog.Warningf("Not mounting current directory: user bind control is disabled by system administrator")
+		return nil
+	}
+	if c.engine.EngineConfig.GetNoCwd() {
+		c.skipCwd = true
+		sylog.Debugf("Skipping current directory mount by user request.")
+		return nil
+	}
+
+	cwdHost := filepath.Clean(c.engine.EngineConfig.GetCwd())
+	if cwdHost == "" || cwdHost == "/" {
+		c.skipCwd = true
+		sylog.Warningf("No current working directory set: skipping mount")
+		return nil
+	} else if cwdHost[0] != '/' {
+		return fmt.Errorf("current working directory %s is not an absolute path", cwdHost)
+	}
+
+	cwdHostResolved, err := filepath.EvalSymlinks(cwdHost)
+	if err != nil {
+		return fmt.Errorf("could not obtain current directory path: %s", err)
+	}
+	sylog.Debugf("Using %s as current working directory", cwdHost)
+
+	cwdPaths := []string{cwdHost}
+	cwdHostSymlink := cwdHost != cwdHostResolved
+
+	// handle new symlink /bin -> usr/bin, /sbin -> usr/sbin ...
+	if cwdHostSymlink {
+		cwdPaths = append(cwdPaths, cwdHostResolved)
+	}
+
+	for _, cwdPath := range cwdPaths {
+		switch cwdPath {
+		case "/", "/etc", "/bin", "/mnt", "/usr", "/var", "/opt", "/sbin", "/lib", "/lib64":
+			sylog.Verbosef("Not mounting CWD within operating system directory: %s", cwdPath)
+			c.skipCwd = true
+			return nil
+		}
+		if strings.HasPrefix(cwdPath, "/sys") || strings.HasPrefix(cwdPath, "/proc") || strings.HasPrefix(cwdPath, "/dev") {
+			sylog.Verbosef("Not mounting CWD within virtual directory: %s", cwdPath)
+			c.skipCwd = true
+			return nil
+		}
+	}
+
+	pathComponents := strings.Split(cwdHost, string(os.PathSeparator))
+	existingPath := false
+	path := ""
+
+	// check if first or last path component exists in container either
+	// in container image or via user/home binds
+	for i, pathComponent := range pathComponents[1:] {
+		if i == 0 {
+			path = filepath.Join(string(os.PathSeparator), pathComponent)
+		} else {
+			path = filepath.Join(path, pathComponent)
+		}
+		_, err := c.session.GetOverridePath(path)
+		existingPath = err == nil
+		if err != nil {
+			pathContainerResolved := fs.EvalRelative(path, c.session.RootFsPath())
+			if pathContainerResolved != path {
+				return nil
+			}
+			info, err := c.rpcOps.Lstat(filepath.Join(c.session.RootFsPath(), pathContainerResolved))
+			if err != nil {
+				existingPath = false
+			} else if !info.IsDir() {
+				existingPath = true
+			}
+		}
+	}
+
+	if !existingPath && c.session.Layer != nil {
+		if c.engine.EngineConfig.GetSessionLayer() == singularity.UnderlayLayer {
+			flags := uintptr(syscall.MS_BIND | c.suidFlag | syscall.MS_NODEV | syscall.MS_REC)
+			if err := system.Points.AddBind(mount.CwdTag, cwdHost, cwdHost, flags); err != nil {
+				return fmt.Errorf("could not bind cwd directory %s into container: %s", cwdHost, err)
+			}
+			return system.Points.AddRemount(mount.CwdTag, cwdHost, flags)
+		}
+		sessionPath := filepath.Join(c.session.Layer.Dir(), cwdHost)
+		// ignore error and let addCwdMount failing properly
+		if err := c.session.AddDir(sessionPath); err != nil {
+			sylog.Warningf("Not creating container current working directory: %s", err)
+		}
+	}
+
+	return nil
 }
 
 func (c *container) addLibsMount(system *mount.System) error {
@@ -2154,18 +2141,21 @@ func (c *container) addFilesMount(system *mount.System) error {
 }
 
 func (c *container) addIdentityMount(system *mount.System) error {
-	if (os.Geteuid() == 0 && c.engine.EngineConfig.GetTargetUID() == 0) ||
-		c.engine.EngineConfig.GetFakeroot() {
-		sylog.Verbosef("Not updating passwd/group files, running as root!")
-		return nil
-	}
-
 	rootfs := c.session.RootFsPath()
 	defer c.session.Update()
 
 	uid := os.Getuid()
 	if uid == 0 && c.engine.EngineConfig.GetTargetUID() != 0 {
 		uid = c.engine.EngineConfig.GetTargetUID()
+	} else if c.engine.EngineConfig.GetFakeroot() {
+		uid = 0
+	}
+
+	if (uid == 0) &&
+		c.engine.EngineConfig.GetWritableImage() &&
+		(!c.engine.EngineConfig.GetWritableTmpfs()) {
+		sylog.Verbosef("skipping bind-mount of /etc/passwd and /etc/group (rootfs is writable and not a tmpfs, and container is running as root)")
+		return nil
 	}
 
 	if c.engine.EngineConfig.File.ConfigPasswd {
@@ -2174,7 +2164,7 @@ func (c *container) addIdentityMount(system *mount.System) error {
 		if err != nil {
 			sylog.Warningf("%s", err)
 		} else {
-			content, err := files.Passwd(passwd, home, uid)
+			content, err := files.Passwd(passwd, home, uid, c)
 			if err != nil {
 				sylog.Warningf("%s", err)
 			} else {
@@ -2197,7 +2187,7 @@ func (c *container) addIdentityMount(system *mount.System) error {
 
 	if c.engine.EngineConfig.File.ConfigGroup {
 		group := filepath.Join(rootfs, "/etc/group")
-		content, err := files.Group(group, uid, c.engine.EngineConfig.GetTargetGID())
+		content, err := files.Group(group, uid, c.engine.EngineConfig.GetTargetGID(), c)
 		if err != nil {
 			sylog.Warningf("%s", err)
 		} else {
@@ -2428,7 +2418,7 @@ func (c *container) getFuseFdFromRPC(fds []int) ([]int, error) {
 
 	bufSpace := (len(fds) + 1) * 4
 	buf := make([]byte, unix.CmsgSpace(bufSpace))
-	_, _, _, _, err := unix.Recvmsg(socketPair[0], nil, buf, 0)
+	_, _, _, _, err := unix.Recvmsg(socketPair[0], nil, buf, 0) //nolint:dogsled
 	if err != nil {
 		return nil, fmt.Errorf("while receiving file descriptors: %s", err)
 	}
@@ -2441,7 +2431,8 @@ func (c *container) getFuseFdFromRPC(fds []int) ([]int, error) {
 	newfds := make([]int, 0, 1)
 
 	for _, msg := range msgs {
-		pfds, err := unix.ParseUnixRights(&msg)
+		m := msg
+		pfds, err := unix.ParseUnixRights(&m)
 		if err != nil {
 			return nil, fmt.Errorf("while getting file descriptor: %s", err)
 		}
@@ -2455,43 +2446,6 @@ func (c *container) getFuseFdFromRPC(fds []int) ([]int, error) {
 	return newfds, nil
 }
 
-// openFuseFdFromRPC returns fuse file descriptor opened by RPC server,
-// the first returned argument corresponds to the file descriptor to use
-// by the caller while the second argument corresponds to the file
-// descriptor used by the RPC server.
-func (c *container) openFuseFdFromRPC() (int, int, error) {
-	socketPair := c.engine.EngineConfig.GetUnixSocketPair()
-
-	fuseRPCFd, err := c.rpcOps.OpenSendFuseFd(socketPair[1])
-	if err != nil {
-		return -1, -1, fmt.Errorf("while requesting a fuse file descriptor open/send: %s", err)
-	}
-
-	bufSpace := 4
-	buf := make([]byte, unix.CmsgSpace(bufSpace))
-	_, _, _, _, err = unix.Recvmsg(socketPair[0], nil, buf, 0)
-	if err != nil {
-		return -1, -1, fmt.Errorf("while receiving file descriptors: %s", err)
-	}
-
-	msgs, err := unix.ParseSocketControlMessage(buf)
-	if err != nil {
-		return -1, -1, fmt.Errorf("while parsing socket control message: %s", err)
-	}
-
-	fuseFd := -1
-
-	if len(msgs) > 0 {
-		fds, err := unix.ParseUnixRights(&msgs[0])
-		if err != nil {
-			return -1, -1, fmt.Errorf("while getting file descriptor: %s", err)
-		}
-		fuseFd = fds[0]
-	}
-
-	return fuseFd, fuseRPCFd, nil
-}
-
 // addFuseMount transforms the plugin configuration into a series of
 // mount requests for FUSE filesystems
 func (c *container) addFuseMount(system *mount.System) (int, error) {
@@ -2651,3 +2605,36 @@ func (c *container) getBindFlags(source string, defaultFlags uintptr) (uintptr,
 
 	return defaultFlags | addFlags, nil
 }
+
+func (c *container) GetPwUID(uid uint32) (*user.User, error) {
+	if c.engine.EngineConfig.JSON.UserInfo.Username == "" {
+		return nil, osuser.UnknownUserIdError(uid)
+	}
+	return &user.User{
+		Name:  c.engine.EngineConfig.JSON.UserInfo.Username,
+		UID:   uint32(c.engine.EngineConfig.JSON.UserInfo.UID),
+		GID:   uint32(c.engine.EngineConfig.JSON.UserInfo.GID),
+		Gecos: c.engine.EngineConfig.JSON.UserInfo.Gecos,
+		Dir:   c.engine.EngineConfig.JSON.UserInfo.Home,
+		Shell: c.engine.EngineConfig.JSON.UserInfo.Shell,
+	}, nil
+}
+
+func (c *container) GetGrGID(gid uint32) (*user.Group, error) {
+	name, ok := c.engine.EngineConfig.JSON.UserInfo.Groups[int(gid)]
+	if ok {
+		return &user.Group{
+			Name: name,
+			GID:  gid,
+		}, nil
+	}
+	return nil, osuser.UnknownGroupIdError(fmt.Sprintf("%d", gid))
+}
+
+func (c *container) Getgroups() ([]int, error) {
+	gids := make([]int, 0, len(c.engine.EngineConfig.JSON.UserInfo.Groups))
+	for gid := range c.engine.EngineConfig.JSON.UserInfo.Groups {
+		gids = append(gids, gid)
+	}
+	return gids, nil
+}
