@@ -1,5 +1,7 @@
-// Copyright (c) 2020, Control Command Inc. All rights reserved.
 // Copyright (c) 2018-2023, Sylabs Inc. All rights reserved.
+// Copyright (c) Contributors to the Apptainer project, established as
+//   Apptainer a Series of LF Projects LLC.
+// Copyright (c) 2020, Control Command Inc. All rights reserved.
 // This software is licensed under a 3-clause BSD license. Please consult the
 // LICENSE.md file distributed with the sources of this project regarding your
 // rights to use or distribute this software.
@@ -20,29 +22,29 @@ import (
 
 	"github.com/ProtonMail/go-crypto/openpgp"
 	specs "github.com/opencontainers/runtime-spec/specs-go"
-	"github.com/sylabs/singularity/internal/pkg/buildcfg"
-	"github.com/sylabs/singularity/internal/pkg/cgroups"
-	fakerootutil "github.com/sylabs/singularity/internal/pkg/fakeroot"
-	"github.com/sylabs/singularity/internal/pkg/instance"
-	"github.com/sylabs/singularity/internal/pkg/plugin"
-	"github.com/sylabs/singularity/internal/pkg/runtime/engine/config/starter"
-	"github.com/sylabs/singularity/internal/pkg/security"
-	"github.com/sylabs/singularity/internal/pkg/security/seccomp"
-	"github.com/sylabs/singularity/internal/pkg/syecl"
-	"github.com/sylabs/singularity/internal/pkg/util/fs"
-	"github.com/sylabs/singularity/internal/pkg/util/fs/overlay"
-	"github.com/sylabs/singularity/internal/pkg/util/mainthread"
-	"github.com/sylabs/singularity/internal/pkg/util/user"
-	"github.com/sylabs/singularity/pkg/image"
-	fakerootcallback "github.com/sylabs/singularity/pkg/plugin/callback/runtime/fakeroot"
-	"github.com/sylabs/singularity/pkg/runtime/engine/config"
-	singularityConfig "github.com/sylabs/singularity/pkg/runtime/engine/singularity/config"
-	"github.com/sylabs/singularity/pkg/sylog"
-	"github.com/sylabs/singularity/pkg/sypgp"
-	"github.com/sylabs/singularity/pkg/util/capabilities"
-	"github.com/sylabs/singularity/pkg/util/fs/proc"
-	"github.com/sylabs/singularity/pkg/util/namespaces"
-	"github.com/sylabs/singularity/pkg/util/singularityconf"
+	"github.com/sylabs/singularity/v4/internal/pkg/buildcfg"
+	"github.com/sylabs/singularity/v4/internal/pkg/cgroups"
+	fakerootutil "github.com/sylabs/singularity/v4/internal/pkg/fakeroot"
+	"github.com/sylabs/singularity/v4/internal/pkg/instance"
+	"github.com/sylabs/singularity/v4/internal/pkg/plugin"
+	"github.com/sylabs/singularity/v4/internal/pkg/runtime/engine/config/starter"
+	"github.com/sylabs/singularity/v4/internal/pkg/security"
+	"github.com/sylabs/singularity/v4/internal/pkg/security/seccomp"
+	"github.com/sylabs/singularity/v4/internal/pkg/syecl"
+	"github.com/sylabs/singularity/v4/internal/pkg/sypgp"
+	"github.com/sylabs/singularity/v4/internal/pkg/util/fs"
+	"github.com/sylabs/singularity/v4/internal/pkg/util/fs/overlay"
+	"github.com/sylabs/singularity/v4/internal/pkg/util/mainthread"
+	"github.com/sylabs/singularity/v4/internal/pkg/util/user"
+	"github.com/sylabs/singularity/v4/pkg/image"
+	fakerootcallback "github.com/sylabs/singularity/v4/pkg/plugin/callback/runtime/fakeroot"
+	"github.com/sylabs/singularity/v4/pkg/runtime/engine/config"
+	singularityConfig "github.com/sylabs/singularity/v4/pkg/runtime/engine/singularity/config"
+	"github.com/sylabs/singularity/v4/pkg/sylog"
+	"github.com/sylabs/singularity/v4/pkg/util/capabilities"
+	"github.com/sylabs/singularity/v4/pkg/util/fs/proc"
+	"github.com/sylabs/singularity/v4/pkg/util/namespaces"
+	"github.com/sylabs/singularity/v4/pkg/util/singularityconf"
 	"golang.org/x/sys/unix"
 )
 
@@ -108,8 +110,8 @@ func (e *EngineOperations) PrepareConfig(starterConfig *starter.Config) error {
 
 	// Save the current working directory if not set
 	if e.EngineConfig.GetCwd() == "" {
-		if pwd, err := os.Getwd(); err == nil {
-			e.EngineConfig.SetCwd(pwd)
+		if cwd, err := os.Getwd(); err == nil {
+			e.EngineConfig.SetCwd(cwd)
 		} else {
 			sylog.Warningf("can't determine current working directory")
 			e.EngineConfig.SetCwd("/")
@@ -128,11 +130,13 @@ func (e *EngineOperations) PrepareConfig(starterConfig *starter.Config) error {
 
 	uid := e.EngineConfig.GetTargetUID()
 	gids := e.EngineConfig.GetTargetGID()
+	useTargetIDs := e.EngineConfig.GetFakeroot()
 
 	if os.Getuid() == 0 && (uid != 0 || len(gids) > 0) {
 		starterConfig.SetTargetUID(uid)
 		starterConfig.SetTargetGID(gids)
 		e.EngineConfig.OciConfig.SetProcessNoNewPrivileges(true)
+		useTargetIDs = true
 	}
 
 	if e.EngineConfig.GetInstanceJoin() {
@@ -140,6 +144,8 @@ func (e *EngineOperations) PrepareConfig(starterConfig *starter.Config) error {
 			return err
 		}
 	} else {
+		e.setUserInfo(useTargetIDs)
+
 		if err := e.prepareContainerConfig(starterConfig); err != nil {
 			return err
 		}
@@ -177,7 +183,7 @@ func (e *EngineOperations) PrepareConfig(starterConfig *starter.Config) error {
 		return err
 	}
 
-	if sendFd || e.EngineConfig.File.ImageDriver != "" { // nolint:staticcheck
+	if sendFd {
 		fds, err := unix.Socketpair(unix.AF_UNIX, unix.SOCK_STREAM|unix.SOCK_CLOEXEC, 0)
 		if err != nil {
 			return fmt.Errorf("failed to create socketpair to pass file descriptor: %s", err)
@@ -573,6 +579,7 @@ func (e *EngineOperations) prepareContainerConfig(starterConfig *starter.Config)
 		if len(callbacks) > 1 {
 			return fmt.Errorf("multiple plugins have registered hook callback for fakeroot")
 		} else if len(callbacks) == 1 {
+			//nolint:forcetypeassert
 			getIDRange = callbacks[0].(fakerootcallback.UserMapping)
 		}
 
@@ -755,6 +762,7 @@ func (e *EngineOperations) prepareInstanceJoinConfig(starterConfig *starter.Conf
 		if err != nil {
 			return fmt.Errorf("error while getting information for instance task directory: %s", err)
 		}
+		//nolint:forcetypeassert
 		st := fi.Sys().(*syscall.Stat_t)
 		if st.Uid != uint32(uid) || st.Gid != uint32(gid) {
 			return fmt.Errorf("instance process owned by %d:%d instead of %d:%d", st.Uid, st.Gid, uid, gid)
@@ -797,6 +805,7 @@ func (e *EngineOperations) prepareInstanceJoinConfig(starterConfig *starter.Conf
 		if err != nil {
 			return fmt.Errorf("error while getting information for parent task directory: %s", err)
 		}
+		//nolint:forcetypeassert
 		st = fi.Sys().(*syscall.Stat_t)
 		if st.Uid != uint32(uid) || st.Gid != uint32(gid) {
 			return fmt.Errorf("parent instance process owned by %d:%d instead of %d:%d", st.Uid, st.Gid, uid, gid)
@@ -999,7 +1008,6 @@ func (e *EngineOperations) setSessionLayer(img *image.Image) error {
 	writableTmpfs := e.EngineConfig.GetWritableTmpfs()
 	writableImage := e.EngineConfig.GetWritableImage()
 	hasOverlayImage := len(e.EngineConfig.GetOverlayImage()) > 0
-	overlayDriver := e.EngineConfig.File.EnableOverlay == "driver"
 
 	if writableImage && hasOverlayImage {
 		return fmt.Errorf("you could not use --overlay in conjunction with --writable")
@@ -1023,19 +1031,6 @@ func (e *EngineOperations) setSessionLayer(img *image.Image) error {
 		}
 	}
 
-	// overlay is handled by the image driver
-	if overlayDriver {
-		if e.EngineConfig.File.ImageDriver == "" { // nolint:staticcheck
-			return fmt.Errorf("you need to specify an image driver with 'enable overlay = driver'")
-		}
-		if !writableImage || hasSIFOverlay {
-			e.EngineConfig.SetSessionLayer(singularityConfig.OverlayLayer)
-			return nil
-		}
-		sylog.Debugf("Not attempting to use overlay or underlay: writable flag requested")
-		return nil
-	}
-
 	// Check for implicit user namespace, e.g when we run %test in a fakeroot build
 	// https://github.com/hpcng/singularity/issues/5315
 	userNS, _ := namespaces.IsInsideUserNamespace(os.Getpid())
@@ -1441,3 +1436,51 @@ func (e *EngineOperations) loadImage(path string, writable bool) (*image.Image,
 
 	return imgObject, imgErr
 }
+
+func (e *EngineOperations) setUserInfo(useTargetIDs bool) {
+	var gids []int
+
+	pw, err := user.CurrentOriginal()
+	if err != nil {
+		return
+	}
+
+	e.EngineConfig.JSON.UserInfo.Home = pw.Dir
+
+	if useTargetIDs {
+		pw, err = user.GetPwUID(uint32(e.EngineConfig.GetTargetUID()))
+		if err == nil {
+			e.EngineConfig.JSON.UserInfo.Username = pw.Name
+			e.EngineConfig.JSON.UserInfo.Gecos = pw.Gecos
+			e.EngineConfig.JSON.UserInfo.UID = int(pw.UID)
+			e.EngineConfig.JSON.UserInfo.GID = int(pw.GID)
+		}
+	} else {
+		e.EngineConfig.JSON.UserInfo.Username = pw.Name
+		e.EngineConfig.JSON.UserInfo.Gecos = pw.Gecos
+		e.EngineConfig.JSON.UserInfo.UID = int(pw.UID)
+		e.EngineConfig.JSON.UserInfo.GID = int(pw.GID)
+	}
+
+	e.EngineConfig.JSON.UserInfo.Groups = make(map[int]string)
+
+	if useTargetIDs {
+		gids = e.EngineConfig.GetTargetGID()
+	} else {
+		gids, err = os.Getgroups()
+		if err != nil {
+			return
+		}
+	}
+
+	if pw != nil {
+		gids = append(gids, int(pw.GID))
+	}
+
+	for _, gid := range gids {
+		group, err := user.GetGrGID(uint32(gid))
+		if err == nil {
+			e.EngineConfig.JSON.UserInfo.Groups[gid] = group.Name
+		}
+	}
+}
