@@ -20,35 +20,36 @@ import (
 
 	lccgroups "github.com/opencontainers/runc/libcontainer/cgroups"
 	"github.com/opencontainers/runtime-spec/specs-go"
-	sifuser "github.com/sylabs/sif/v2/pkg/user"
-	"github.com/sylabs/singularity/internal/pkg/buildcfg"
-	"github.com/sylabs/singularity/internal/pkg/cgroups"
-	"github.com/sylabs/singularity/internal/pkg/image/unpacker"
-	"github.com/sylabs/singularity/internal/pkg/instance"
-	"github.com/sylabs/singularity/internal/pkg/plugin"
-	"github.com/sylabs/singularity/internal/pkg/runtime/engine/config/oci"
-	"github.com/sylabs/singularity/internal/pkg/runtime/engine/config/oci/generate"
-	"github.com/sylabs/singularity/internal/pkg/runtime/launcher"
-	"github.com/sylabs/singularity/internal/pkg/security"
-	"github.com/sylabs/singularity/internal/pkg/util/bin"
-	"github.com/sylabs/singularity/internal/pkg/util/env"
-	"github.com/sylabs/singularity/internal/pkg/util/fs"
-	"github.com/sylabs/singularity/internal/pkg/util/gpu"
-	"github.com/sylabs/singularity/internal/pkg/util/shell/interpreter"
-	"github.com/sylabs/singularity/internal/pkg/util/starter"
-	"github.com/sylabs/singularity/internal/pkg/util/user"
-	imgutil "github.com/sylabs/singularity/pkg/image"
-	clicallback "github.com/sylabs/singularity/pkg/plugin/callback/cli"
-	singularitycallback "github.com/sylabs/singularity/pkg/plugin/callback/runtime/engine/singularity"
-	"github.com/sylabs/singularity/pkg/runtime/engine/config"
-	singularityConfig "github.com/sylabs/singularity/pkg/runtime/engine/singularity/config"
-	"github.com/sylabs/singularity/pkg/sylog"
-	"github.com/sylabs/singularity/pkg/util/bind"
-	"github.com/sylabs/singularity/pkg/util/capabilities"
-	"github.com/sylabs/singularity/pkg/util/cryptkey"
-	"github.com/sylabs/singularity/pkg/util/namespaces"
-	"github.com/sylabs/singularity/pkg/util/rlimit"
-	"github.com/sylabs/singularity/pkg/util/singularityconf"
+
+	// TODO - backport (partial) our own fuse mount handling
+	sifuser "github.com/sylabs/sif/v2/pkg/user" //nolint:staticcheck
+	"github.com/sylabs/singularity/v4/internal/pkg/buildcfg"
+	"github.com/sylabs/singularity/v4/internal/pkg/cgroups"
+	"github.com/sylabs/singularity/v4/internal/pkg/image/unpacker"
+	"github.com/sylabs/singularity/v4/internal/pkg/instance"
+	"github.com/sylabs/singularity/v4/internal/pkg/plugin"
+	"github.com/sylabs/singularity/v4/internal/pkg/runtime/engine/config/oci"
+	"github.com/sylabs/singularity/v4/internal/pkg/runtime/engine/config/oci/generate"
+	"github.com/sylabs/singularity/v4/internal/pkg/runtime/launcher"
+	"github.com/sylabs/singularity/v4/internal/pkg/security"
+	"github.com/sylabs/singularity/v4/internal/pkg/util/bin"
+	"github.com/sylabs/singularity/v4/internal/pkg/util/env"
+	"github.com/sylabs/singularity/v4/internal/pkg/util/fs"
+	"github.com/sylabs/singularity/v4/internal/pkg/util/gpu"
+	"github.com/sylabs/singularity/v4/internal/pkg/util/starter"
+	"github.com/sylabs/singularity/v4/internal/pkg/util/user"
+	"github.com/sylabs/singularity/v4/pkg/image"
+	imgutil "github.com/sylabs/singularity/v4/pkg/image"
+	clicallback "github.com/sylabs/singularity/v4/pkg/plugin/callback/cli"
+	"github.com/sylabs/singularity/v4/pkg/runtime/engine/config"
+	singularityConfig "github.com/sylabs/singularity/v4/pkg/runtime/engine/singularity/config"
+	"github.com/sylabs/singularity/v4/pkg/sylog"
+	"github.com/sylabs/singularity/v4/pkg/util/bind"
+	"github.com/sylabs/singularity/v4/pkg/util/capabilities"
+	"github.com/sylabs/singularity/v4/pkg/util/cryptkey"
+	"github.com/sylabs/singularity/v4/pkg/util/namespaces"
+	"github.com/sylabs/singularity/v4/pkg/util/rlimit"
+	"github.com/sylabs/singularity/v4/pkg/util/singularityconf"
 	"golang.org/x/sys/unix"
 )
 
@@ -70,6 +71,17 @@ func NewLauncher(opts ...launcher.Option) (*Launcher, error) {
 			return nil, fmt.Errorf("%w", err)
 		}
 	}
+	if len(lo.Devices) > 0 {
+		return nil, fmt.Errorf("CDI device mappings unsupported in native launcher")
+	}
+
+	if len(lo.CdiDirs) > 0 {
+		return nil, fmt.Errorf("CDI device mappings unsupported in native launcher")
+	}
+
+	if lo.NoCompat {
+		sylog.Warningf("--no-compat applies to --oci mode only, ignoring")
+	}
 
 	// Initialize empty default Singularity Engine and OCI configuration
 	engineConfig := singularityConfig.NewConfig()
@@ -96,11 +108,13 @@ func NewLauncher(opts ...launcher.Option) (*Launcher, error) {
 // This includes interactive containers, instances, and joining an existing instance.
 //
 //nolint:maintidx
-func (l *Launcher) Exec(ctx context.Context, image string, process string, args []string, instanceName string) error {
+func (l *Launcher) Exec(ctx context.Context, ep launcher.ExecParams) error {
 	var err error
 
-	// Native runtime expects command to execute as arg[0]
-	args = append([]string{process}, args...)
+	args, err := ep.ActionScriptArgs()
+	if err != nil {
+		return fmt.Errorf("while getting ProcessArgs: %w", err)
+	}
 
 	// Set arguments to pass to contained process.
 	l.generator.SetProcessArgs(args)
@@ -123,7 +137,7 @@ func (l *Launcher) Exec(ctx context.Context, image string, process string, args
 	}
 
 	// Set image to run, or instance to join, and SINGULARITY_CONTAINER/SINGULARITY_NAME env vars.
-	if err := l.setImageOrInstance(image, instanceName); err != nil {
+	if err := l.setImageOrInstance(ep.Image, ep.Instance); err != nil {
 		sylog.Errorf("While setting image/instance: %s", err)
 	}
 
@@ -131,12 +145,12 @@ func (l *Launcher) Exec(ctx context.Context, image string, process string, args
 	l.engineConfig.SetOverlayImage(l.cfg.OverlayPaths)
 	l.engineConfig.SetWritableImage(l.cfg.Writable)
 
-	// Check key is available for encrypted image, if applicable.
+	// Check image is something we can run, and key is available for encrypted image, if applicable.
 	// If we are joining an instance, then any encrypted image is already mounted.
 	if !l.engineConfig.GetInstanceJoin() {
-		err = l.checkEncryptionKey()
+		err = l.checkImage()
 		if err != nil {
-			sylog.Fatalf("While checking container encryption: %s", err)
+			sylog.Fatalf("While checking image: %s", err)
 		}
 	}
 
@@ -269,7 +283,7 @@ func (l *Launcher) Exec(ctx context.Context, image string, process string, args
 		}
 	}
 
-	l.setCgroups(instanceName)
+	l.setCgroups(ep.Instance)
 
 	// --boot flag requires privilege, so check for this.
 	err = launcher.WithPrivilege(l.cfg.Boot, "--boot", func() error { return nil })
@@ -288,9 +302,14 @@ func (l *Launcher) Exec(ctx context.Context, image string, process string, args
 		}
 	}
 
+	// --no-pid disables PID inferred above
+	if l.cfg.Namespaces.NoPID {
+		l.cfg.Namespaces.PID = false
+	}
+
 	// Setup instance specific configuration if required.
-	if instanceName != "" {
-		l.generator.AddProcessEnv("SINGULARITY_INSTANCE", instanceName)
+	if ep.Instance != "" {
+		l.generator.AddProcessEnv("SINGULARITY_INSTANCE", ep.Instance)
 		l.cfg.Namespaces.PID = true
 		l.engineConfig.SetInstance(true)
 		l.engineConfig.SetBootInstance(l.cfg.Boot)
@@ -299,16 +318,16 @@ func (l *Launcher) Exec(ctx context.Context, image string, process string, args
 			return fmt.Errorf("hidepid option set on /proc mount, require 'hidepid=0' to start instance with setuid workflow")
 		}
 
-		_, err := instance.Get(instanceName, instance.SingSubDir)
+		_, err := instance.Get(ep.Instance, instance.SingSubDir)
 		if err == nil {
-			return fmt.Errorf("instance %s already exists", instanceName)
+			return fmt.Errorf("instance %s already exists", ep.Instance)
 		}
 
 		if l.cfg.Boot {
 			l.cfg.Namespaces.UTS = true
 			l.cfg.Namespaces.Net = true
 			if len(l.cfg.Hostname) < 1 {
-				l.engineConfig.SetHostname(instanceName)
+				l.engineConfig.SetHostname(ep.Instance)
 			}
 			if !l.cfg.KeepPrivs {
 				l.engineConfig.SetDropCaps("CAP_SYS_BOOT,CAP_SYS_RAWIO")
@@ -329,13 +348,13 @@ func (l *Launcher) Exec(ctx context.Context, image string, process string, args
 	l.generator.AddProcessEnv("SINGULARITY_APPNAME", l.cfg.AppName)
 
 	// Get image ready to run, if needed, via FUSE mount / extraction / image driver handling.
-	if err := l.prepareImage(ctx, image); err != nil {
+	if err := l.prepareImage(ctx, ep.Image); err != nil {
 		return fmt.Errorf("while preparing image: %s", err)
 	}
 
 	// Call the starter binary using our prepared config.
 	if l.engineConfig.GetInstance() {
-		err = l.starterInstance(instanceName, useSuid)
+		err = l.starterInstance(ep.Instance, useSuid)
 	} else {
 		err = l.starterInteractive(useSuid)
 	}
@@ -478,15 +497,32 @@ func (l *Launcher) setImageOrInstance(image string, name string) error {
 	return nil
 }
 
-// checkEncryptionKey verifies key material is available if the image is encrypted.
-// Allows us to fail fast if required key material is not available / usable.
-func (l *Launcher) checkEncryptionKey() error {
-	sylog.Debugf("Checking for encrypted system partition")
+func (l *Launcher) checkImage() error {
 	img, err := imgutil.Init(l.engineConfig.GetImage(), false)
 	if err != nil {
 		return fmt.Errorf("could not open image %s: %w", l.engineConfig.GetImage(), err)
 	}
 
+	if img.Type == image.OCISIF {
+		return fmt.Errorf("native runtime does not support OCI-SIF images, use --oci mode")
+	}
+
+	if err := l.checkEncryptionKey(img); err != nil {
+		return err
+	}
+
+	// don't defer this call as in all cases it won't be
+	// called before execing starter, so it would leak the
+	// image file descriptor to the container process
+	img.File.Close()
+	return nil
+}
+
+// checkEncryptionKey verifies key material is available if the image is encrypted.
+// Allows us to fail fast if required key material is not available / usable.
+func (l *Launcher) checkEncryptionKey(img *imgutil.Image) error {
+	sylog.Debugf("Checking for encrypted system partition")
+
 	part, err := img.GetRootFsPartition()
 	if err != nil {
 		return fmt.Errorf("while getting root filesystem in %s: %w", l.engineConfig.GetImage(), err)
@@ -507,10 +543,6 @@ func (l *Launcher) checkEncryptionKey() error {
 
 		l.engineConfig.SetEncryptionKey(plaintextKey)
 	}
-	// don't defer this call as in all cases it won't be
-	// called before execing starter, so it would leak the
-	// image file descriptor to the container process
-	img.File.Close()
 	return nil
 }
 
@@ -646,9 +678,9 @@ func (l *Launcher) setHome() error {
 	targetUID := l.engineConfig.GetTargetUID()
 	if l.cfg.CustomHome && targetUID != 0 {
 		if targetUID > 500 {
-			if pwd, err := user.GetPwUID(uint32(targetUID)); err == nil {
-				sylog.Debugf("Target UID requested, set home directory to %s", pwd.Dir)
-				l.cfg.HomeDir = pwd.Dir
+			if pu, err := user.GetPwUID(uint32(targetUID)); err == nil {
+				sylog.Debugf("Target UID requested, set home directory to %s", pu.Dir)
+				l.cfg.HomeDir = pu.Dir
 				l.engineConfig.SetCustomHome(true)
 			} else {
 				sylog.Verbosef("Home directory for UID %d not found, home won't be mounted", targetUID)
@@ -851,12 +883,7 @@ func (l *Launcher) setEnv(ctx context.Context, args []string) error {
 			"SINGULARITY_IMAGE="+l.engineConfig.GetImage(),
 		)
 
-		content, err := os.ReadFile(l.cfg.EnvFile)
-		if err != nil {
-			return fmt.Errorf("could not read environment file %q: %w", l.cfg.EnvFile, err)
-		}
-
-		env, err := interpreter.EvaluateEnv(ctx, content, args, currentEnv)
+		env, err := env.FileMap(ctx, l.cfg.EnvFile, args, currentEnv)
 		if err != nil {
 			return fmt.Errorf("while processing %s: %w", l.cfg.EnvFile, err)
 		}
@@ -865,22 +892,12 @@ func (l *Launcher) setEnv(ctx context.Context, args []string) error {
 		sylog.Debugf("Setting environment variables from file %s", l.cfg.EnvFile)
 
 		// Update Env with those from file
-		for _, envar := range env {
-			e := strings.SplitN(envar, "=", 2)
-			if len(e) != 2 {
-				sylog.Warningf("Ignored environment variable %q: '=' is missing", envar)
-				continue
-			}
-			// Don't attempt to overwrite bash builtin readonly vars
-			// https://github.com/sylabs/singularity/issues/1263
-			if e[0] == "UID" || e[0] == "GID" {
-				continue
-			}
+		for k, v := range env {
 			// Ensure we don't overwrite --env variables with environment file
-			if _, ok := l.cfg.Env[e[0]]; ok {
-				sylog.Warningf("Ignored environment variable %s from %s: override from --env", e[0], l.cfg.EnvFile)
+			if _, ok := l.cfg.Env[k]; ok {
+				sylog.Warningf("Ignored environment variable %s from %s: override from --env", k, l.cfg.EnvFile)
 			} else {
-				l.cfg.Env[e[0]] = e[1]
+				l.cfg.Env[k] = v
 			}
 		}
 	}
@@ -904,15 +921,15 @@ func (l *Launcher) setEnv(ctx context.Context, args []string) error {
 
 // setProcessCwd sets the container process working directory
 func (l *Launcher) setProcessCwd() {
-	if pwd, err := os.Getwd(); err == nil {
-		l.engineConfig.SetCwd(pwd)
-		if l.cfg.PwdPath != "" {
-			l.generator.SetProcessCwd(l.cfg.PwdPath)
+	if cwd, err := os.Getwd(); err == nil {
+		l.engineConfig.SetCwd(cwd)
+		if l.cfg.CwdPath != "" {
+			l.generator.SetProcessCwd(l.cfg.CwdPath)
 		} else {
 			if l.engineConfig.GetContain() {
 				l.generator.SetProcessCwd(l.engineConfig.GetHomeDest())
 			} else {
-				l.generator.SetProcessCwd(pwd)
+				l.generator.SetProcessCwd(cwd)
 			}
 		}
 	} else {
@@ -976,52 +993,36 @@ func (l *Launcher) prepareImage(c context.Context, image string) error {
 	// namespace or if we are currently running inside a
 	// user namespace
 	if (l.cfg.Namespaces.User || insideUserNs) && fs.IsFile(image) {
-		convert := true
-		// load image driver plugins
-		if l.engineConfig.File.ImageDriver != "" { // nolint:staticcheck
-			callbackType := (singularitycallback.RegisterImageDriver)(nil) // nolint:staticcheck
-			callbacks, err := plugin.LoadCallbacks(callbackType)
-			if err != nil {
-				sylog.Debugf("Loading plugins callbacks '%T' failed: %s", callbackType, err)
-			} else {
-				for _, callback := range callbacks {
-					if err := callback.(singularitycallback.RegisterImageDriver)(true); err != nil { // nolint:staticcheck
-						sylog.Debugf("While registering image driver: %s", err)
-					}
-				}
-			}
-			driver := imgutil.GetDriver(l.engineConfig.File.ImageDriver) // nolint:staticcheck
-			if driver != nil && driver.Features()&imgutil.ImageFeature != 0 {
-				// the image driver indicates support for image so let's
-				// proceed with the image driver without conversion
-				convert = false
-			}
+		// Honor 'tmp sandbox' in singularity.conf, and allow negation with
+		// `--no-tmp-sandbox`.
+		canUseTmpSandbox := l.engineConfig.File.TmpSandboxAllowed
+		if l.cfg.NoTmpSandbox {
+			canUseTmpSandbox = false
 		}
 
-		if convert {
-			tryFUSE := l.cfg.SIFFUSE || l.engineConfig.File.SIFFUSE
-			if tryFUSE && l.cfg.Fakeroot {
-				return fmt.Errorf("fakeroot is not currently supported with FUSE SIF mounts")
-			}
+		tryFUSE := l.cfg.SIFFUSE || l.engineConfig.File.SIFFUSE
+		if tryFUSE && l.cfg.Fakeroot {
+			return fmt.Errorf("fakeroot is not currently supported with FUSE SIF mounts")
+		}
 
-			fuse, tempDir, imageDir, err := handleImage(c, image, tryFUSE)
+		fuse, tempDir, imageDir, err := handleImage(c, image, tryFUSE, canUseTmpSandbox)
+		if err != nil {
+			return fmt.Errorf("while handling %s: %w", image, err)
+		}
+		l.engineConfig.SetImage(imageDir)
+		l.engineConfig.SetImageFuse(fuse)
+		l.engineConfig.SetDeleteTempDir(tempDir)
+		l.generator.AddProcessEnv("SINGULARITY_CONTAINER", imageDir)
+		// if '--disable-cache' flag, then remove original SIF after converting to sandbox
+		if l.cfg.CacheDisabled {
+			sylog.Debugf("Removing tmp image: %s", image)
+			err := os.Remove(image)
 			if err != nil {
-				return fmt.Errorf("while handling %s: %w", image, err)
-			}
-			l.engineConfig.SetImage(imageDir)
-			l.engineConfig.SetImageFuse(fuse)
-			l.engineConfig.SetDeleteTempDir(tempDir)
-			l.generator.AddProcessEnv("SINGULARITY_CONTAINER", imageDir)
-			// if '--disable-cache' flag, then remove original SIF after converting to sandbox
-			if l.cfg.CacheDisabled {
-				sylog.Debugf("Removing tmp image: %s", image)
-				err := os.Remove(image)
-				if err != nil {
-					return fmt.Errorf("unable to remove tmp image: %s: %w", image, err)
-				}
+				return fmt.Errorf("unable to remove tmp image: %s: %w", image, err)
 			}
 		}
 	}
+
 	return nil
 }
 
@@ -1030,7 +1031,7 @@ func (l *Launcher) prepareImage(c context.Context, image string) error {
 // caller's responsibility to remove tempDir when no longer needed. If isFUSE is
 // returned true, then the imageDir is a FUSE mount, and must be unmounted
 // during cleanup.
-func handleImage(ctx context.Context, filename string, tryFUSE bool) (isFUSE bool, tempDir, imageDir string, err error) {
+func handleImage(ctx context.Context, filename string, tryFUSE, canUseTmpSandbox bool) (isFUSE bool, tempDir, imageDir string, err error) {
 	img, err := imgutil.Init(filename, false)
 	if err != nil {
 		return false, "", "", fmt.Errorf("could not open image %s: %s", filename, err)
@@ -1070,6 +1071,9 @@ func handleImage(ctx context.Context, filename string, tryFUSE bool) (isFUSE boo
 	}
 
 	// Fall back to extraction to directory
+	if !canUseTmpSandbox {
+		return false, "", "", fmt.Errorf("unpacking image to temporary sandbox dir required, but is prohibited by 'tmp sandbox = no' in singularity.conf or --no-tmp-sandbox command-line flag")
+	}
 	err = extractImage(img, imageDir)
 	if err == nil {
 		return false, tempDir, imageDir, nil
@@ -1204,11 +1208,11 @@ func (l *Launcher) starterInstance(name string, useSuid bool) error {
 	// Allow any plugins with callbacks to modify the assembled Config
 	runPluginCallbacks(cfg)
 
-	pwd, err := user.GetPwUID(uint32(os.Getuid()))
+	pu, err := user.GetPwUID(uint32(os.Getuid()))
 	if err != nil {
 		return fmt.Errorf("failed to retrieve user information for UID %d: %w", os.Getuid(), err)
 	}
-	procname, err := instance.ProcName(name, pwd.Name)
+	procname, err := instance.ProcName(name, pu.Name)
 	if err != nil {
 		return err
 	}
@@ -1272,6 +1276,7 @@ func runPluginCallbacks(cfg *config.Common) error {
 		return fmt.Errorf("while loading plugin callbacks '%T': %w", callbackType, err)
 	}
 	for _, c := range callbacks {
+		//nolint:forcetypeassert
 		c.(clicallback.SingularityEngineConfig)(cfg)
 	}
 	return nil
