@@ -16,12 +16,12 @@ import (
 	"time"
 
 	securejoin "github.com/cyphar/filepath-securejoin"
-	"github.com/sylabs/singularity/internal/pkg/util/bin"
-	"github.com/sylabs/singularity/internal/pkg/util/fs"
-	"github.com/sylabs/singularity/internal/pkg/util/user"
-	"github.com/sylabs/singularity/pkg/syfs"
-	"github.com/sylabs/singularity/pkg/sylog"
-	"github.com/sylabs/singularity/pkg/util/fs/lock"
+	"github.com/sylabs/singularity/v4/internal/pkg/util/bin"
+	"github.com/sylabs/singularity/v4/internal/pkg/util/fs"
+	"github.com/sylabs/singularity/v4/internal/pkg/util/rootless"
+	"github.com/sylabs/singularity/v4/pkg/syfs"
+	"github.com/sylabs/singularity/v4/pkg/sylog"
+	"github.com/sylabs/singularity/v4/pkg/util/fs/lock"
 )
 
 const (
@@ -54,14 +54,14 @@ func runtime() (path string, err error) {
 // runtimeStateDir returns path to use for crun/runc's state handling.
 func runtimeStateDir() (path string, err error) {
 	// Ensure we get correct uid for host if we were re-exec'd in id mapped userns
-	pw, err := user.CurrentOriginal()
+	u, err := rootless.GetUser()
 	if err != nil {
 		return "", err
 	}
-	if pw.UID == 0 {
+	if u.Uid == "0" {
 		return "/run/singularity-oci", nil
 	}
-	return fmt.Sprintf("/run/user/%d/singularity-oci", pw.UID), nil
+	return fmt.Sprintf("/run/user/%s/singularity-oci", u.Uid), nil
 }
 
 // stateDir returns the path to container state handled by conmon/singularity
@@ -72,7 +72,7 @@ func stateDir(containerID string) (string, error) {
 		return "", err
 	}
 
-	u, err := user.CurrentOriginal()
+	u, err := rootless.GetUser()
 	if err != nil {
 		return "", err
 	}
