@@ -1,4 +1,4 @@
-// Copyright (c) 2018-2022, Sylabs Inc. All rights reserved.
+// Copyright (c) 2018-2023, Sylabs Inc. All rights reserved.
 // This software is licensed under a 3-clause BSD license. Please consult the
 // LICENSE.md file distributed with the sources of this project regarding your
 // rights to use or distribute this software.
@@ -13,20 +13,18 @@ import (
 	"fmt"
 	"os"
 	"os/exec"
+	"os/signal"
 	"path/filepath"
-	"strings"
+	"strconv"
+	"syscall"
 
-	"github.com/sylabs/singularity/internal/pkg/buildcfg"
-	fakerootConfig "github.com/sylabs/singularity/internal/pkg/runtime/engine/fakeroot/config"
-	"github.com/sylabs/singularity/internal/pkg/util/starter"
-	"github.com/sylabs/singularity/pkg/ocibundle/tools"
-	"github.com/sylabs/singularity/pkg/runtime/engine/config"
-	"github.com/sylabs/singularity/pkg/sylog"
-	"github.com/sylabs/singularity/pkg/util/singularityconf"
+	"github.com/sylabs/singularity/v4/pkg/sylog"
 )
 
 // Delete deletes container resources
-func Delete(ctx context.Context, containerID string, systemdCgroups bool) error {
+//
+// FIXME: use context for cancellation, or remove.
+func Delete(_ context.Context, containerID string, systemdCgroups bool) error {
 	runtimeBin, err := runtime()
 	if err != nil {
 		return err
@@ -102,7 +100,7 @@ func Exec(containerID string, cmdArgs []string, systemdCgroups bool) error {
 }
 
 // Kill kills container process
-func Kill(containerID string, killSignal string, systemdCgroups bool) error {
+func Kill(containerID string, killSignal string) error {
 	runtimeBin, err := runtime()
 	if err != nil {
 		return err
@@ -179,7 +177,9 @@ func Resume(containerID string, systemdCgroups bool) error {
 }
 
 // Run runs a container (equivalent to create/start/delete)
-func Run(ctx context.Context, containerID, bundlePath, pidFile string, systemdCgroups bool) error {
+//
+// FIXME: use context for cancellation, or remove.
+func Run(_ context.Context, containerID, bundlePath, pidFile string, systemdCgroups bool) error {
 	runtimeBin, err := runtime()
 	if err != nil {
 		return err
@@ -209,6 +209,11 @@ func Run(ctx context.Context, containerID, bundlePath, pidFile string, systemdCg
 		runtimeArgs = append(runtimeArgs, "--pid-file="+pidFile)
 	}
 
+	signals := make(chan os.Signal, 2)
+	signal.Notify(signals)
+	sylog.Debugf("Starting signal proxy for container %s", containerID)
+	go signalProxy(containerID, signals)
+
 	runtimeArgs = append(runtimeArgs, containerID)
 	cmd := exec.Command(runtimeBin, runtimeArgs...)
 	cmd.Stdout = os.Stdout
@@ -218,71 +223,27 @@ func Run(ctx context.Context, containerID, bundlePath, pidFile string, systemdCg
 	return cmd.Run()
 }
 
-// RunWrapped runs a container via the OCI runtime, wrapped with prep / cleanup steps.
-func RunWrapped(ctx context.Context, containerID, bundlePath, pidFile string, systemdCgroups, writableTmpfs bool) error {
-	if writableTmpfs {
-		if err := prepareWriteableTmpfs(bundlePath); err != nil {
-			return err
+func signalProxy(containerID string, signals chan os.Signal) {
+	for {
+		s := <-signals
+		switch s {
+		case syscall.SIGCHLD:
+			break
+		case syscall.SIGURG:
+			// Ignore SIGURG, which is used for non-cooperative goroutine
+			// preemption starting with Go 1.14. For more information, see
+			// https://github.com/golang/go/issues/24543.
+			break
+		default:
+			sylog.Debugf("Received signal %s", s.String())
+			//nolint:forcetypeassert
+			sysSig := s.(syscall.Signal)
+			sylog.Debugf("Sending signal %s to container %s", sysSig.String(), containerID)
+			if err := Kill(containerID, strconv.Itoa(int(sysSig))); err != nil {
+				sylog.Errorf("Failed to send signal %s to container %s", sysSig.String(), containerID)
+			}
 		}
 	}
-
-	err := Run(ctx, containerID, bundlePath, pidFile, systemdCgroups)
-
-	if writableTmpfs {
-		// Cleanup actions log errors, but don't return - so we get as much cleanup done as possible.
-		if err := cleanupWritableTmpfs(bundlePath); err != nil {
-			sylog.Errorf("While cleaning up writable tmpfs: %v", err)
-		}
-	}
-
-	// Return any error from the actual container payload - preserve exit code.
-	return err
-}
-
-// RunWrappedNS reexecs singularity in a user namespace, with supplied uid/gid mapping, calling oci run.
-func RunWrappedNS(ctx context.Context, containerID, bundlePath, pidFile string, writableTmpfs bool) error {
-	absBundle, err := filepath.Abs(bundlePath)
-	if err != nil {
-		return fmt.Errorf("failed to determine bundle absolute path: %s", err)
-	}
-
-	if err := os.Chdir(absBundle); err != nil {
-		return fmt.Errorf("failed to change directory to %s: %s", absBundle, err)
-	}
-
-	args := []string{
-		filepath.Join(buildcfg.BINDIR, "singularity"),
-		"oci",
-		"run-wrapped",
-		"-b", absBundle,
-	}
-	if pidFile != "" {
-		args = append(args, "--pid-file="+pidFile)
-	}
-	if writableTmpfs {
-		args = append(args, "--writable-tmpfs")
-	}
-	args = append(args, containerID)
-
-	sylog.Debugf("Calling fakeroot engine to execute %q", strings.Join(args, " "))
-
-	cfg := &config.Common{
-		EngineName:  fakerootConfig.Name,
-		ContainerID: "fakeroot",
-		EngineConfig: &fakerootConfig.EngineConfig{
-			Envs:    os.Environ(),
-			Args:    args,
-			NoPIDNS: true,
-		},
-	}
-
-	return starter.Run(
-		"Singularity oci userns",
-		cfg,
-		starter.WithStdin(os.Stdin),
-		starter.WithStdout(os.Stdout),
-		starter.WithStderr(os.Stderr),
-	)
 }
 
 // Start starts a previously created container
@@ -362,17 +323,3 @@ func Update(containerID, cgFile string, systemdCgroups bool) error {
 	sylog.Debugf("Calling %s with args %v", runtimeBin, runtimeArgs)
 	return cmd.Run()
 }
-
-func prepareWriteableTmpfs(bundleDir string) error {
-	sylog.Debugf("Configuring writable tmpfs overlay for %s", bundleDir)
-	c := singularityconf.GetCurrentConfig()
-	if c == nil {
-		return fmt.Errorf("singularity configuration is not initialized")
-	}
-	return tools.CreateOverlayTmpfs(bundleDir, int(c.SessiondirMaxSize))
-}
-
-func cleanupWritableTmpfs(bundleDir string) error {
-	sylog.Debugf("Cleaning up writable tmpfs overlay for %s", bundleDir)
-	return tools.DeleteOverlay(bundleDir)
-}
