@@ -6,21 +6,21 @@
 package oci
 
 import (
-	"os"
+	"fmt"
+	"path/filepath"
 
 	"github.com/opencontainers/runtime-spec/specs-go"
-	"github.com/sylabs/singularity/internal/pkg/runtime/launcher"
-	"github.com/sylabs/singularity/pkg/sylog"
+
+	"github.com/sylabs/singularity/v4/internal/pkg/runtime/launcher"
+	"github.com/sylabs/singularity/v4/internal/pkg/util/rootless"
+	"github.com/sylabs/singularity/v4/pkg/sylog"
 )
 
-// defaultNamespaces matching native runtime with --compat / --containall.
+// defaultNamespaces matching native runtime with --compat / --containall, except PID which can be disabled.
 var defaultNamespaces = []specs.LinuxNamespace{
 	{
 		Type: specs.IPCNamespace,
 	},
-	{
-		Type: specs.PIDNamespace,
-	},
 	{
 		Type: specs.MountNamespace,
 	},
@@ -34,8 +34,7 @@ func minimalSpec() specs.Spec {
 		Version: specs.Version,
 	}
 	config.Root = &specs.Root{
-		Path: "rootfs",
-		// TODO - support read-only. At present we always have a writable tmpfs overlay, like native runtime --compat.
+		Path:     "rootfs",
 		Readonly: false,
 	}
 	config.Process = &specs.Process{
@@ -50,24 +49,6 @@ func minimalSpec() specs.Spec {
 		"PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin",
 	}
 
-	// TODO - these are appropriate minimum for rootless. We need to tie into
-	// Singularity's cap-add / cap-drop mechanism.
-	config.Process.Capabilities = &specs.LinuxCapabilities{
-		Bounding: []string{
-			"CAP_CHOWN",
-			"CAP_DAC_OVERRIDE",
-			"CAP_FOWNER",
-			"CAP_FSETID",
-			"CAP_KILL",
-			"CAP_NET_BIND_SERVICE",
-			"CAP_SETFCAP",
-			"CAP_SETGID",
-			"CAP_SETPCAP",
-			"CAP_SETUID",
-			"CAP_SYS_CHROOT",
-		},
-	}
-
 	// All mounts are added by the launcher, as it must handle flags.
 	config.Mounts = []specs.Mount{}
 
@@ -77,7 +58,7 @@ func minimalSpec() specs.Spec {
 
 // addNamespaces adds requested namespace, if appropriate, to an existing spec.
 // It is assumed that spec contains at least the defaultNamespaces.
-func addNamespaces(spec specs.Spec, ns launcher.Namespaces) specs.Spec {
+func addNamespaces(spec *specs.Spec, ns launcher.Namespaces) error {
 	if ns.IPC {
 		sylog.Infof("--oci runtime always uses an IPC namespace, ipc flag is redundant.")
 	}
@@ -92,17 +73,29 @@ func addNamespaces(spec specs.Spec, ns launcher.Namespaces) specs.Spec {
 	}
 
 	if ns.PID {
-		sylog.Infof("--oci runtime always uses a PID namespace, pid flag is redundant.")
+		sylog.Infof("--oci runtime uses a PID namespace by default, pid flag is redundant.")
+	}
+
+	if !ns.NoPID {
+		spec.Linux.Namespaces = append(
+			spec.Linux.Namespaces,
+			specs.LinuxNamespace{Type: specs.PIDNamespace},
+		)
 	}
 
 	if ns.User {
-		if os.Getuid() == 0 {
+		uid, err := rootless.Getuid()
+		if err != nil {
+			return err
+		}
+
+		if uid == 0 {
 			spec.Linux.Namespaces = append(
 				spec.Linux.Namespaces,
 				specs.LinuxNamespace{Type: specs.UserNamespace},
 			)
 		} else {
-			sylog.Infof("--oci runtime always uses a user namespace when run as a non-root userns, user flag is redundant.")
+			sylog.Infof("The --oci runtime always creates a user namespace when run as non-root, --userns / -u flag is redundant.")
 		}
 	}
 
@@ -113,5 +106,23 @@ func addNamespaces(spec specs.Spec, ns launcher.Namespaces) specs.Spec {
 		)
 	}
 
-	return spec
+	return nil
+}
+
+// noSetgroupsAnnotation will set the `run.oci.keep_original_groups=1` annotation
+// to disable the setgroups call when entering the container. Supported by crun, but not runc.
+func noSetgroupsAnnotation(spec *specs.Spec) error {
+	runtime, err := runtime()
+	if err != nil {
+		return err
+	}
+	if filepath.Base(runtime) != "crun" {
+		return fmt.Errorf("runtime '%q' does not support --no-setgroups", runtime)
+	}
+
+	if spec.Annotations == nil {
+		spec.Annotations = map[string]string{}
+	}
+	spec.Annotations["run.oci.keep_original_groups"] = "1"
+	return nil
 }
