@@ -6,8 +6,11 @@
 package launcher
 
 import (
+	"fmt"
+
 	"github.com/containers/image/v5/types"
-	"github.com/sylabs/singularity/pkg/util/cryptkey"
+	"github.com/sylabs/singularity/v4/internal/pkg/util/fs/overlay"
+	"github.com/sylabs/singularity/v4/pkg/util/cryptkey"
 )
 
 // Namespaces holds flags for the optional (non-mount) namespaces that can be
@@ -18,6 +21,8 @@ type Namespaces struct {
 	PID  bool
 	IPC  bool
 	Net  bool
+	// NoPID will force the PID namespace not to be used, even if set by default / other flags.
+	NoPID bool
 }
 
 // Options accumulates launch configuration from passed functional options. Note
@@ -25,7 +30,7 @@ type Namespaces struct {
 type Options struct {
 	// Writable marks the container image itself as writable.
 	Writable bool
-	// WriteableTmpfs applies an ephemeral writable overlay to the container.
+	// WritableTmpfs applies an ephemeral writable overlay to the container.
 	WritableTmpfs bool
 	// OverlayPaths holds paths to image or directory overlays to be applied.
 	OverlayPaths []string
@@ -111,8 +116,8 @@ type Options struct {
 
 	// ShellPath is a custom shell executable to be launched in the container.
 	ShellPath string
-	// PwdPath is the initial working directory in the container.
-	PwdPath string
+	// CwdPath is the initial working directory in the container.
+	CwdPath string
 
 	// Fakeroot enables the fake root mode, using user namespaces and subuid / subgid mapping.
 	Fakeroot bool
@@ -143,6 +148,20 @@ type Options struct {
 	// SysContext holds Docker/OCI image handling configuration.
 	// This will be used by a launcher handling OCI images directly.
 	SysContext *types.SystemContext
+
+	// NoTmpSandbox prohibits unpacking of images into temporary sandbox dirs.
+	NoTmpSandbox bool
+
+	// Devices contains the list of device mappings (if any), e.g. CDI mappings.
+	Devices []string
+
+	// CdiDirs contains the list of directories in which CDI should look for device definition JSON files
+	CdiDirs []string
+
+	// NoCompat indicates the container should be run in non-OCI compatible
+	// mode, i.e. with default mounts etc. as native mode. Effective for the OCI
+	// launcher only.
+	NoCompat bool
 }
 
 type Option func(co *Options) error
@@ -164,8 +183,16 @@ func OptWritableTmpfs(b bool) Option {
 }
 
 // OptOverlayPaths sets overlay images and directories to apply to the container.
+// Relative paths are resolved to absolute paths at this point.
 func OptOverlayPaths(op []string) Option {
 	return func(lo *Options) error {
+		var err error
+		for i, p := range op {
+			op[i], err = overlay.AbsOverlay(p)
+			if err != nil {
+				return fmt.Errorf("could not convert %q to absolute path: %w", p, err)
+			}
+		}
 		lo.OverlayPaths = op
 		return nil
 	}
@@ -405,10 +432,10 @@ func OptShellPath(s string) Option {
 	}
 }
 
-// OptPwdPath specifies the initial working directory in the container.
-func OptPwdPath(p string) Option {
+// OptCwdPath specifies the initial working directory in the container.
+func OptCwdPath(p string) Option {
 	return func(lo *Options) error {
-		lo.PwdPath = p
+		lo.CwdPath = p
 		return nil
 	}
 }
@@ -485,6 +512,14 @@ func OptSIFFuse(b bool) Option {
 	}
 }
 
+// OptNoTmpSandbox prohibits unpacking of images into temporary sandbox dirs.
+func OptNoTmpSandbox(b bool) Option {
+	return func(lo *Options) error {
+		lo.NoTmpSandbox = b
+		return nil
+	}
+}
+
 // OptCacheDisabled indicates caching of images was disabled in the CLI.
 func OptCacheDisabled(b bool) Option {
 	return func(lo *Options) error {
@@ -500,3 +535,27 @@ func OptSysContext(sc *types.SystemContext) Option {
 		return nil
 	}
 }
+
+// OptDevice sets CDI device mappings to apply.
+func OptDevice(op []string) Option {
+	return func(lo *Options) error {
+		lo.Devices = op
+		return nil
+	}
+}
+
+// OptCdiDirs sets CDI spec search-directories to apply.
+func OptCdiDirs(op []string) Option {
+	return func(lo *Options) error {
+		lo.CdiDirs = op
+		return nil
+	}
+}
+
+// OptNoCompat disable OCI compatible mode, for singularity native mode default behaviors.
+func OptNoCompat(b bool) Option {
+	return func(lo *Options) error {
+		lo.NoCompat = b
+		return nil
+	}
+}
