@@ -11,10 +11,12 @@ import (
 	"os"
 	"os/exec"
 	"path/filepath"
+	"strings"
+	"sync"
 	"syscall"
 
-	"github.com/sylabs/singularity/internal/pkg/util/bin"
-	"github.com/sylabs/singularity/pkg/sylog"
+	"github.com/sylabs/singularity/v4/internal/pkg/util/bin"
+	"github.com/sylabs/singularity/v4/pkg/sylog"
 	"golang.org/x/sys/unix"
 )
 
@@ -30,7 +32,7 @@ const (
 	upperDir
 )
 
-type fs struct {
+type filesys struct {
 	name       string
 	overlayDir dir
 }
@@ -44,7 +46,7 @@ const (
 	panfs  int64 = 0xAAD7AAEA
 )
 
-var incompatibleFs = map[int64]fs{
+var incompatibleFilesys = map[int64]filesys{
 	// NFS filesystem
 	nfs: {
 		name:       "NFS",
@@ -82,10 +84,10 @@ func check(path string, d dir) error {
 	stfs := &unix.Statfs_t{}
 
 	if err := statfs(path, stfs); err != nil {
-		return fmt.Errorf("could not retrieve underlying filesystem information for %s: %s", path, err)
+		return fmt.Errorf("could not retrieve underlying filesystem information for %s: %w", path, err)
 	}
 
-	fs, ok := incompatibleFs[int64(stfs.Type)]
+	fs, ok := incompatibleFilesys[int64(stfs.Type)]
 	if !ok || (ok && fs.overlayDir&d == 0) {
 		return nil
 	}
@@ -141,7 +143,7 @@ var ErrNoRootlessOverlay = errors.New("rootless overlay not supported by kernel"
 func CheckRootless() error {
 	mountBin, err := bin.FindBin("mount")
 	if err != nil {
-		return fmt.Errorf("while looking for mount command: %s", err)
+		return fmt.Errorf("while looking for mount command: %w", err)
 	}
 
 	tmpDir, err := os.MkdirTemp("", "check-overlay")
@@ -202,3 +204,104 @@ func CheckRootless() error {
 	sylog.Debugf("Rootless overlay appears supported on this system.")
 	return nil
 }
+
+// Info about kernel support for unprivileged overlays
+var unprivOverlays struct {
+	kernelSupport bool
+	initOnce      sync.Once
+	err           error
+}
+
+// UnprivOverlaysSupported checks whether there is kernel support for unprivileged overlays. The actual check is performed only once and cached in the unprivOverlays variable, above.
+func UnprivOverlaysSupported() (bool, error) {
+	unprivOverlays.initOnce.Do(func() {
+		err := CheckRootless()
+		if err == nil {
+			unprivOverlays.kernelSupport = true
+			return
+		}
+
+		if err == ErrNoRootlessOverlay {
+			unprivOverlays.kernelSupport = false
+			return
+		}
+
+		unprivOverlays.err = err
+	})
+
+	if unprivOverlays.err != nil {
+		return false, unprivOverlays.err
+	}
+
+	return unprivOverlays.kernelSupport, nil
+}
+
+// ensureOverlayDir checks if a directory already exists; if it doesn't, and
+// createIfMissing is true, it attempts to create it with the specified
+// permissions.
+func EnsureOverlayDir(dir string, createIfMissing bool, createPerm os.FileMode) error {
+	if len(dir) == 0 {
+		return fmt.Errorf("internal error: ensureOverlayDir() called with empty dir name")
+	}
+
+	_, err := os.Stat(dir)
+	if err == nil {
+		return nil
+	}
+
+	if !os.IsNotExist(err) {
+		return err
+	}
+
+	if !createIfMissing {
+		return fmt.Errorf("missing overlay dir %q", dir)
+	}
+
+	// Create the requested dir
+	if err := os.Mkdir(dir, createPerm); err != nil {
+		return fmt.Errorf("failed to create %q: %w", dir, err)
+	}
+
+	return nil
+}
+
+// detachAndDelete performs an unmount system call on the specified directory,
+// followed by deletion of the directory and all of its contents.
+func DetachAndDelete(overlayDir string) error {
+	sylog.Debugf("Detaching overlayDir %q", overlayDir)
+	if err := syscall.Unmount(overlayDir, syscall.MNT_DETACH); err != nil {
+		return fmt.Errorf("failed to unmount %s: %w", overlayDir, err)
+	}
+
+	sylog.Debugf("Removing overlayDir %q", overlayDir)
+	if err := os.RemoveAll(overlayDir); err != nil {
+		return fmt.Errorf("failed to remove %s: %w", overlayDir, err)
+	}
+	return nil
+}
+
+// DetachMount performs an unmount system call on the specified directory.
+func DetachMount(dir string) error {
+	sylog.Debugf("Calling syscall.Unmount() to detach %q", dir)
+	if err := syscall.Unmount(dir, syscall.MNT_DETACH); err != nil {
+		return fmt.Errorf("failed to detach %s: %w", dir, err)
+	}
+
+	return nil
+}
+
+// AbsOverlay takes an overlay description string (a path, optionally followed by a colon with an option string, like ":ro" or ":rw"), and replaces any relative path in the description string with an absolute one.
+func AbsOverlay(desc string) (string, error) {
+	splitted := strings.SplitN(desc, ":", 2)
+	barePath := splitted[0]
+	absBarePath, err := filepath.Abs(barePath)
+	if err != nil {
+		return "", err
+	}
+	absDesc := absBarePath
+	if len(splitted) > 1 {
+		absDesc += ":" + splitted[1]
+	}
+
+	return absDesc, nil
+}
