@@ -51,7 +51,7 @@ type File struct {
 	MaxLoopDevices          uint     `default:"256" directive:"max loop devices"`
 	SessiondirMaxSize       uint     `default:"64" directive:"sessiondir max size"`
 	MountDev                string   `default:"yes" authorized:"yes,no,minimal" directive:"mount dev"`
-	EnableOverlay           string   `default:"try" authorized:"yes,no,try,driver" directive:"enable overlay"`
+	EnableOverlay           string   `default:"try" authorized:"yes,no,try" directive:"enable overlay"`
 	BindPath                []string `default:"/etc/localtime,/etc/hosts" directive:"bind path"`
 	LimitContainerOwners    []string `directive:"limit container owners"`
 	LimitContainerGroups    []string `directive:"limit container groups"`
@@ -71,13 +71,13 @@ type File struct {
 	MksquashfsMem           string   `directive:"mksquashfs mem"`
 	NvidiaContainerCliPath  string   `directive:"nvidia-container-cli path"`
 	UnsquashfsPath          string   `directive:"unsquashfs path"`
-	// Deprecated: ImageDriver is deprecated and will be removed in 4.0.
-	ImageDriver         string `directive:"image driver"`
-	DownloadConcurrency uint   `default:"3" directive:"download concurrency"`
-	DownloadPartSize    uint   `default:"5242880" directive:"download part size"`
-	DownloadBufferSize  uint   `default:"32768" directive:"download buffer size"`
-	SystemdCgroups      bool   `default:"yes" authorized:"yes,no" directive:"systemd cgroups"`
-	SIFFUSE             bool   `default:"no" authorized:"yes,no" directive:"sif fuse"`
+	DownloadConcurrency     uint     `default:"3" directive:"download concurrency"`
+	DownloadPartSize        uint     `default:"5242880" directive:"download part size"`
+	DownloadBufferSize      uint     `default:"32768" directive:"download buffer size"`
+	SystemdCgroups          bool     `default:"yes" authorized:"yes,no" directive:"systemd cgroups"`
+	SIFFUSE                 bool     `default:"no" authorized:"yes,no" directive:"sif fuse"`
+	OCIMode                 bool     `default:"no" authorized:"yes,no" directive:"oci mode"`
+	TmpSandboxAllowed       bool     `default:"yes" authorized:"yes,no" directive:"tmp sandbox"`
 }
 
 const TemplateAsset = `# SINGULARITY.CONF
@@ -97,6 +97,15 @@ const TemplateAsset = `# SINGULARITY.CONF
 # distributions.
 allow setuid = {{ if eq .AllowSetuid true }}yes{{ else }}no{{ end }}
 
+# OCI MODE: [BOOL]
+# DEFAULT: no
+# Should we use the OCI runtime, and push/pull OCI-SIF images by default?
+# Mimics always specifying --oci on the command line.
+# Can be reversed by specifying --no-oci on the command line.
+# Note that OCI mode requires unprivileged user namespace creation and
+# subuid / subgid mappings.
+oci mode = {{ if eq .OCIMode true }}yes{{ else }}no{{ end }}
+
 # MAX LOOP DEVICES: [INT]
 # DEFAULT: 256
 # Set the maximum number of loop devices that Singularity should ever attempt
@@ -144,6 +153,8 @@ mount sys = {{ if eq .MountSys true }}yes{{ else }}no{{ end }}
 # Should we automatically bind mount /dev within the container? If 'minimal'
 # is chosen, then only 'null', 'zero', 'random', 'urandom', and 'shm' will
 # be included (the same effect as the --contain options)
+#
+# Must be set to 'yes' or 'minimal' to use --oci mode.
 mount dev = {{ .MountDev }}
 
 # MOUNT DEVPTS: [BOOL]
@@ -152,6 +163,8 @@ mount dev = {{ .MountDev }}
 # /dev, or -C is passed?  Note, this requires that your kernel was
 # configured with CONFIG_DEVPTS_MULTIPLE_INSTANCES=y, or that you're
 # running kernel 4.7 or newer.
+#
+# Must be set to 'yes' to use --oci mode.
 mount devpts = {{ if eq .MountDevPts true }}yes{{ else }}no{{ end }}
 
 # MOUNT HOME: [BOOL]
@@ -183,9 +196,12 @@ mount hostfs = {{ if eq .MountHostfs true }}yes{{ else }}no{{ end }}
 # the container. The file or directory must exist within the container on
 # which to attach to. you can specify a different source and destination
 # path (respectively) with a colon; otherwise source and dest are the same.
-# NOTE: these are ignored if singularity is invoked with --contain except
+#
+# In native mode, these are ignored if singularity is invoked with --contain except
 # for /etc/hosts and /etc/localtime. When invoked with --contain and --net,
 # /etc/hosts would contain a default generated content for localhost resolution.
+#
+# In OCI mode these are only mounted when --no-compat is specified.
 #bind path = /etc/singularity/default-nsswitch.conf:/etc/nsswitch.conf
 #bind path = /opt
 #bind path = /scratch
@@ -207,12 +223,11 @@ user bind control = {{ if eq .UserBindControl true }}yes{{ else }}no{{ end }}
 # command line option.
 enable fusemount = {{ if eq .EnableFusemount true }}yes{{ else }}no{{ end }}
 
-# ENABLE OVERLAY: [yes/no/try/driver]
+# ENABLE OVERLAY: [yes/no/try]
 # DEFAULT: try
 # Enabling this option will make it possible to specify bind paths to locations
 # that do not currently exist within the container.  If 'try' is chosen,
 # overlayfs will be tried but if it is unavailable it will be silently ignored.
-# If 'driver' is chosen, overlayfs is handled by the image driver.
 enable overlay = {{ .EnableOverlay }}
 
 # ENABLE UNDERLAY: [yes/no]
@@ -237,12 +252,30 @@ mount slave = {{ if eq .MountSlave true }}yes{{ else }}no{{ end }}
 # In --oci mode, each tmpfs mount in the container can be up to this size.
 sessiondir max size = {{ .SessiondirMaxSize }}
 
+# *****************************************************************************
+# WARNING
+#
+# The 'limit container' and 'allow container' directives are not effective if
+# unprivileged user namespaces are enabled. They are only effectively applied
+# when Singularity is running using the native runtime in setuid mode, and
+# unprivileged container execution is not possible on the host.
+#
+# You must disable unprivileged user namespace creation on the host if you rely
+# on the these directives to limit container execution. This will disable OCI
+# mode, which is unprivileged and cannot enforce these limits.
+#
+# See the 'Security' and 'Configuration Files' sections of the Admin Guide for
+# more information.
+# *****************************************************************************
+
 # LIMIT CONTAINER OWNERS: [STRING]
 # DEFAULT: NULL
 # Only allow containers to be used that are owned by a given user. If this
 # configuration is undefined (commented or set to NULL), all containers are
-# allowed to be used. This feature only applies when Singularity is running in
-# SUID mode and the user is non-root.
+# allowed to be used. 
+#
+# Only effective in setuid mode, with unprivileged user namespace creation disabled.
+# Ignored for the root user.
 #limit container owners = gmk, singularity, nobody
 {{ range $index, $owner := .LimitContainerOwners }}
 {{- if eq $index 0 }}limit container owners = {{ else }}, {{ end }}{{$owner}}
@@ -252,8 +285,10 @@ sessiondir max size = {{ .SessiondirMaxSize }}
 # DEFAULT: NULL
 # Only allow containers to be used that are owned by a given group. If this
 # configuration is undefined (commented or set to NULL), all containers are
-# allowed to be used. This feature only applies when Singularity is running in
-# SUID mode and the user is non-root.
+# allowed to be used.
+#
+# Only effective in setuid mode, with unprivileged user namespace creation disabled.
+# Ignored for the root user.
 #limit container groups = group1, singularity, nobody
 {{ range $index, $group := .LimitContainerGroups }}
 {{- if eq $index 0 }}limit container groups = {{ else }}, {{ end }}{{$group}}
@@ -263,9 +298,10 @@ sessiondir max size = {{ .SessiondirMaxSize }}
 # DEFAULT: NULL
 # Only allow containers to be used that are located within an allowed path
 # prefix. If this configuration is undefined (commented or set to NULL),
-# containers will be allowed to run from anywhere on the file system. This
-# feature only applies when Singularity is running in SUID mode and the user is
-# non-root.
+# containers will be allowed to run from anywhere on the file system.
+#
+# Only effective in setuid mode, with unprivileged user namespace creation disabled.
+# Ignored for the root user.
 #limit container paths = /scratch, /tmp, /global
 {{ range $index, $path := .LimitContainerPaths }}
 {{- if eq $index 0 }}limit container paths = {{ else }}, {{ end }}{{$path}}
@@ -274,7 +310,10 @@ sessiondir max size = {{ .SessiondirMaxSize }}
 # ALLOW CONTAINER ${TYPE}: [BOOL]
 # DEFAULT: yes
 # This feature limits what kind of containers that Singularity will allow
-# users to use (note this does not apply for root).
+# users to use.
+#
+# Only effective in setuid mode, with unprivileged user namespace creation disabled.
+# Ignored for the root user.
 #
 # Allow use of unencrypted SIF containers
 allow container sif = {{ if eq .AllowContainerSIF true}}yes{{ else }}no{{ end }}
@@ -359,7 +398,8 @@ always use rocm = {{ if eq .AlwaysUseRocm true }}yes{{ else }}no{{ end }}
 
 # ROOT DEFAULT CAPABILITIES: [full/file/no]
 # DEFAULT: full
-# Define default root capability set kept during runtime
+# Define default root capability set kept during runtime.
+# Applies to the singularity runtime only, not --oci mode.
 # - full: keep all capabilities (same as --keep-privs)
 # - file: keep capabilities configured for root in
 #         ${prefix}/etc/singularity/capability.json
@@ -457,19 +497,6 @@ mksquashfs procs = {{ .MksquashfsProcs }}
 # usage and optimize kernel cache (useful for MPI)
 shared loop devices = {{ if eq .SharedLoopDevices true }}yes{{ else }}no{{ end }}
 
-# IMAGE DRIVER: [STRING]
-# DEFAULT: Undefined
-#
-# DEPRECATED - Will be removed in 4.0.
-#
-# This option specifies the name of an image driver provided by a plugin that
-# will be used to handle image mounts. If the 'enable overlay' option is set
-# to 'driver' the driver name specified here will also be used to handle
-# overlay mounts.
-# If the driver name specified has not been registered via a plugin installation
-# the run-time will abort.
-image driver = {{ .ImageDriver }}
-
 # DOWNLOAD CONCURRENCY: [UINT]
 # DEFAULT: 3
 # This option specifies how many concurrent streams when downloading (pulling)
